// This software including the source code and documentation (the "SOFTWARE")
// is copyrighted.  You are free to use the SOFTWARE for any non-commercial
// purpose whatsoever provided you do not remove this copyright notice.
// If you wish to use the SOFTWARE for commercial purposes, please contact us
// to obtain a license.  The SOFTWARE is provided "as is", with absolutely no
// warranty.  Use it at your own risk.
// Website: http://www.math.ie/checker.html

#include <stdio.h>
#include <vector>
#include <algorithm>
#include <memory.h>
#include <fstream>
#include <ctype.h>
#include "myset.h"
#include "unav36.h"
using namespace std;

#define ZeroMemory( x ) memset( x, 0, sizeof( x ) )

int GrabInt( char*& pch )
   {
   int val = atoi( pch );
   // skip number
   while ( isdigit( *pch ) )
      pch++;
   // skip any whitespace
   while ( isspace( *pch ) )
      pch++;
   return val;
   }


int ReadAdditionalUnavoidables( char const* pszFilename, vector< MySet >& sets )
   {
   ifstream file( pszFilename );
   if ( ! file )
      return -1;

   char szLine[ 100 ];
   int cSets = 0;
   while ( file.getline( szLine, sizeof( szLine ) ) )
      {
      char* pch = szLine;
      while ( isspace( *pch ) )
         pch++;
      MySet set( false );
      if ( *pch == '{' )
         {
         // format 1: {12,16,32,36}
         while ( *pch++ != '}' )
            {
            int i = *pch++ - '0' - 1;
            int j = *pch++ - '0' - 1;
            set.AddElement( i, j );
            }
         }
      else
         {
         // format 2: 4 1 5 19 33
         int count = GrabInt( pch );
         for ( int i = 0;  i < count;  i++ )
            {
            int cell = GrabInt( pch );
            set.AddElement( cell / 9, cell % 9 );
            }
         }
      set.Sort();
      sets.push_back( set );
      cSets++;
      }

   return cSets;
   }


// from nppcsu.exe
// written by Guenter Stertenbrink, sterten@aol.com
// Website: http://magictour.free.fr/sudoku.htm
void GFindUnavoidableSets_nppcsu( int A[ 82 ], vector< MySet >& sets )
   {
   int c1,i1,i2,c,u,v,i,j,k,q,x,y,z;
   int G[88][88],B[88],X[88],Y[88],U[88],V[19];

   for ( i = 1, x = 1;  x <= 9;  x++ )
      for ( y = 1;  y <= 9;  y++ )
         {
         X[ i ] = x;
         Y[ i ] = y;
         B[ i ] = 3*((x-1)/3) + (y-1)/3 + 1;
         i++;
         }

   ZeroMemory( G );
   for ( i = 1;  i <= 81;  i++ )
      for ( j = 1;  j <= 81;  j++ )
         {
         if ( i != j  &&  (X[ i ] == X[ j ]  ||  Y[ i ] == Y[ j ]  ||  B[ i ] == B[ j ]) )
            G[ i ][ j ] = 1;
         }

   for ( i1 = 1;  i1 <= 8;  i1++ )
      for ( i2 = i1 + 1;  i2 <= 9;  i2++ )
         {
         z = 0;
         for ( k = 1;  k <= 81;  k++ )
            if ( A[ k ] == i1  ||  A[ k ] == i2 )
               V[ ++z ] = k;

         ZeroMemory( U );
         U[ 0 ] = 1;
         c = 0;

m34:
         c++;
         for ( k = 1;  k <= 18  &&  U[ V[ k ] ] != 0;  k++ )
            ;
         if ( k > 18 )
            goto m75;

         U[ V[ k ] ] = c;

m22:
         q = 0;
         for ( u = 1;  u <= 18;  u++ )
            for ( v = 1;  v <= 18;  v++ )
               {
               if ( G[ V[ u ] ][ V[ v ] ] > 0  &&  U[ V[ u ] ] == 0  &&
                    U[ V[ v ] ] == c )
                  {
                  q++;
                  U[ V[ u ] ] = c;
                  }

               if ( G[ V[ u ] ][ V[ v ] ] > 0  &&  U[ V[ u ] ] == c  && 
                    U[ V[ v ] ] == 0 )
                  {
                  q++;
                  U[ V[ v ] ] = c;
                  }
               }

         if ( q != 0 )
            goto m22;
         else
            goto m34;

m75:
         if ( c > 2 )
            {
            for ( c1 = 1;  c1 < c;  c1++ )
               {
               MySet set( false );
               for ( i = 1;  i <= 9;  i++ )
                  for ( j = 1;  j <= 9;  j++ )
                     if ( U[ i*9 - 9 + j ] == c1 )
                        set.AddElement( i - 1, j - 1 );
               // use unavoidable sets of size 8-14
               if ( set.GetSize() >= 8  &&  set.GetSize() <= 14 )
                  {
                  set.Sort();
                  sets.push_back( set );
                  }
               }
            }
         }
   }


void GFindUnavoidableSets( int A[ 82 ], vector< MySet >& sets )
   {
   GFindUnavoidableSets_nppcsu( A, sets );
   GFindUnavoidableSets_unav36( A, sets );
   }
