// This software including the source code and documentation (the "SOFTWARE")
// is copyrighted.  You are free to use the SOFTWARE for any non-commercial
// purpose whatsoever provided you do not remove this copyright notice.
// If you wish to use the SOFTWARE for commercial purposes, please contact us
// to obtain a license.  The SOFTWARE is provided "as is", with absolutely no
// warranty.  Use it at your own risk.
// Website: http://www.math.ie/checker.html

#ifndef _stopwatch_H
#define _stopwatch_H 1

#include "int64.h"

#ifdef _WIN32
   extern "C"
      {
      __declspec(dllimport) void __stdcall QueryPerformanceCounter( __int64* );
      __declspec(dllimport) void __stdcall QueryPerformanceFrequency( __int64* );
      }
#else
   #include <sys/time.h>
#endif

class Stopwatch
   {
   public:
      Stopwatch() : m_nTicks( 0 ), m_bRunning( false ) {}

      inline void Start();
      inline void Pause();
      inline float GetTimeElapsed() const; // in seconds

   private:
      int64 m_nTicks;
      bool  m_bRunning;
   };


inline void Stopwatch::Start()
   {
   m_bRunning = true;
#ifdef _WIN32
   int64 counter;
   QueryPerformanceCounter( &counter );
   m_nTicks -= counter;
#else
   timeval tv;
   gettimeofday( &tv, NULL );
   m_nTicks -= (static_cast< int64 >( tv.tv_sec ) * 1000000) + tv.tv_usec;
#endif
   }


inline void Stopwatch::Pause()
   {
#ifdef _WIN32
   int64 counter;
   QueryPerformanceCounter( &counter );
   m_nTicks += counter;
#else
   timeval tv;
   gettimeofday( &tv, NULL );
   m_nTicks += (static_cast< int64 >( tv.tv_sec )  * 1000000) + tv.tv_usec;
#endif
   m_bRunning = false;
   }


inline float Stopwatch::GetTimeElapsed() const // in seconds
   {
#ifdef _WIN32
   int64 freq;
   QueryPerformanceFrequency( &freq );
   if ( ! m_bRunning )  
      return static_cast< float >( m_nTicks ) / freq;
   else
      {
      int64 counter;
      QueryPerformanceCounter( &counter );
      return static_cast< float >( m_nTicks + counter ) / freq;
      }
#else
   if ( ! m_bRunning )
      return static_cast< float >( m_nTicks ) / 1000000;
   else
      {
      timeval tv;
      gettimeofday( &tv, NULL );
      return static_cast< float >( m_nTicks + (static_cast< int64 >( tv.tv_sec ) * 1000000) + tv.tv_usec ) / 1000000;
      }
#endif
   }

#endif
