// This software including the source code and documentation (the "SOFTWARE")
// is copyrighted.  You are free to use the SOFTWARE for any non-commercial
// purpose whatsoever provided you do not remove this copyright notice.
// If you wish to use the SOFTWARE for commercial purposes, please contact us
// to obtain a license.  The SOFTWARE is provided "as is", with absolutely no
// warranty.  Use it at your own risk.
// Website: http://www.math.ie/checker.html

#include "unavsets.h"
#include "misc.h"
#include "config.h"
#include "myset.h"
#include <fstream>

// finds all unavoidable sets of size 4 or size 6
void FindUnavoidableSets( int A[9][9], vector< MySet >& sets, bool swapped /* = false */ )
   {
   // first find sets of size 4
   for ( int box = 0;  box < 6;  box++ )
      for ( int row = 0;  row < 3;  row++ )
         for ( int x1 = 0;  x1 < 2;  x1++ )
            for ( int x2 = x1 + 1;  x2 < 3;  x2++ )
               for ( int box2 = box + 3;  box2 < 9;  box2 += 3 )
                  for ( int row2 = 0;  row2 < 3;  row2++ )
                     {
                     int xLeft   = xcoord( box, row, x1 );
                     int yTop    = ycoord( box, row, x1 );
                     int xRight  = xcoord( box2, row2, x2 );
                     int yBottom = ycoord( box2, row2, x2 );
                     int a1 = A[ yTop ][ xLeft  ];
                     int b1 = A[ yTop ][ xRight ];
                     int a2 = A[ yBottom ][ xLeft  ];
                     int b2 = A[ yBottom ][ xRight ];
                     if ( a1 == b2  &&  a2 == b1 )
                        {
                        // found an unavoidable set
                        MySet set( swapped );
                        set.AddElement( yTop, xLeft );
                        set.AddElement( yTop, xRight );
                        set.AddElement( yBottom, xLeft );
                        set.AddElement( yBottom, xRight );
                        set.Sort();
                        sets.push_back( set );
                        }
                     }

   // find sets of size 6 - Type I
   for ( int col = 0;  col < 3;  col++ )
      for ( int x1 = 0;  x1 < 2;  x1++ )
         for ( int x2 = x1 + 1;  x2 < 3;  x2++ )
            for ( int y1 = 0;  y1 < 3;  y1++ )
               {
               int a1 = A[ ycoord( col, y1, x1 ) ][ xcoord( col, y1, x1 ) ];
               int b1 = A[ ycoord( col, y1, x2 ) ][ xcoord( col, y1, x2 ) ];
               for ( int y2 = 0;  y2 < 3;  y2++ )
                  {
                  int a2 = A[ ycoord( col + 3, y2, x1 ) ][ xcoord( col + 3, y2, x1 ) ];
                  int b2 = A[ ycoord( col + 3, y2, x2 ) ][ xcoord( col + 3, y2, x2 ) ];
                  if ( a1 == b2 )
                     {
                     for ( int y3 = 0;  y3 < 3;  y3++ )
                        {
                        int a3 = A[ ycoord( col + 6, y3, x1 ) ][ xcoord( col + 6, y3, x1 ) ];
                        int b3 = A[ ycoord( col + 6, y3, x2 ) ][ xcoord( col + 6, y3, x2 ) ];
                        if ( a2 == b3  &&  a3 == b1 )
                           {
                           // found an unavoidable set
                           MySet set( swapped );
                           set.AddElement( y1, x1 + 3*col );
                           set.AddElement( y1, x2 + 3*col );
                           set.AddElement( y2 + 3, x1 + 3*col );
                           set.AddElement( y2 + 3, x2 + 3*col );
                           set.AddElement( y3 + 6, x1 + 3*col );
                           set.AddElement( y3 + 6, x2 + 3*col );
                           set.Sort();
                           sets.push_back( set );
                           }
                        }
                     }
                  else if ( b1 == a2 )
                     {
                     for ( int y3 = 0;  y3 < 3;  y3++ )
                        {
                        int a3 = A[ ycoord( col + 6, y3, x1 ) ][ xcoord( col + 6, y3, x1 ) ];
                        int b3 = A[ ycoord( col + 6, y3, x2 ) ][ xcoord( col + 6, y3, x2 ) ];
                        if ( b2 == a3  &&  b3 == a1 )
                           {
                           // found an unavoidable set
                           MySet set( swapped );
                           set.AddElement( y1, x1 + 3*col );
                           set.AddElement( y1, x2 + 3*col );
                           set.AddElement( y2 + 3, x1 + 3*col );
                           set.AddElement( y2 + 3, x2 + 3*col );
                           set.AddElement( y3 + 6, x1 + 3*col );
                           set.AddElement( y3 + 6, x2 + 3*col );
                           set.Sort();
                           sets.push_back( set );
                           }
                        }
                     }
                  }
               }

   // find sets of size 6 - type II
   int const off1[ 6 ][ 6 ] = { {0,2,1,1,0,2}, {0,1,2,1,2,0}, {0,1,2,2,0,1},
                               {0,2,1,2,1,0}, {1,0,2,2,1,0}, {1,2,0,2,0,1} };
   for ( int col = 0;  col < 3;  col++ )
      for ( int y1 = 0;  y1 < 3;  y1++ )
         for ( int y2 = 0;  y2 < 3;  y2++ )
            for ( int y3 = 0;  y3 < 3;  y3++ )
               for ( int i = 0;  i < 6;  i++ )
                  {
                  if ( A[y1][3*col + off1[i][0]] == A[y2 + 3][3*col + off1[i][1]]  &&
                       A[y1][3*col + off1[i][0]] == A[y3 + 6][3*col + off1[i][2]]  &&
                       A[y1][3*col + off1[i][3]] == A[y2 + 3][3*col + off1[i][4]]  &&
                       A[y1][3*col + off1[i][3]] == A[y3 + 6][3*col + off1[i][5]] )
                     {
                     // found an unavoidable set
                     MySet set( swapped );
                     set.AddElement( y1, 3*col + off1[i][0] );
                     set.AddElement( y1, 3*col + off1[i][3] );
                     set.AddElement( y2 + 3, 3*col + off1[i][1] );
                     set.AddElement( y2 + 3, 3*col + off1[i][4] );
                     set.AddElement( y3 + 6, 3*col + off1[i][2] );
                     set.AddElement( y3 + 6, 3*col + off1[i][5] );
                     set.Sort();
                     sets.push_back( set );
                     }
                  }

   // find sets of size 6 - type III
   int const off2[ 2 ][ 3 ] = { {2,0,1}, {1,2,0} };
   for ( int vband = 0;  vband < 3;  vband++ )
      for ( int hband1 = 0;  hband1 < 2;  hband1++ )
         for ( int hband2 = hband1 + 1;  hband2 < 3;  hband2++ )
            for ( int row1 = 0;  row1 < 3;  row1++ )
               for ( int row2 = 0;  row2 < 3;  row2++ )
                  for ( int i = 0;  i < 2;  i++ )
                     {
                     int y1 = 3*hband1 + row1;
                     int y2 = 3*hband2 + row2;
                     int x  = 3*vband;
                     bool bMatch = true;
                     for ( int j = 0;  j < 3  &&  bMatch;  j++ )
                        if ( A[ y1 ][ x + j ] != A[ y2 ][ x + off2[i][j] ] )
                           bMatch = false;

                     if ( bMatch )
                        {
                        // found an unavoidable set
                        MySet set( swapped );
                        set.AddElement( y1, x );
                        set.AddElement( y1, x + 1 );
                        set.AddElement( y1, x + 2 );
                        set.AddElement( y2, x );
                        set.AddElement( y2, x + 1 );
                        set.AddElement( y2, x + 2 );
                        set.Sort();
                        sets.push_back( set );
                        }
                     }

   // find unavoidable sets of size 6 - type IV
   for ( int vband1 = 0;  vband1 < 3;  vband1++ )
      for ( int vband2 = 0;  vband2 < 3;  vband2++ )
         {
         if ( vband1 == vband2 )
            continue;
         for ( int hband1 = 0;  hband1 < 3;  hband1++ )
            for ( int hband2 = 0;  hband2 < 3;  hband2++ )
               {
               if ( hband1 == hband2 )
                  continue;
               for ( int col1 = 0;  col1 < 3;  col1++ )
                  {
                  int x1 = 3*vband1 + col1;
                  for ( int col2 = 0;  col2 < 3;  col2++ )
                     {
                     int x2 = 3*vband1 + col2;
                     if ( col1 == col2 )
                        continue;
                     for ( int col3 = 0;  col3 < 3;  col3++ )
                        {
                        int x3 = 3*vband2 + col3;
                        for ( int row1 = 0;  row1 < 3;  row1++ )
                           {
                           int y1 = 3*hband1 + row1;
                           for ( int row2 = 0;  row2 < 2;  row2++ )
                              {
                              int y2 = 3*hband2 + row2;
                              for ( int row3 = row2 + 1;  row3 < 3;  row3++ )
                                 {
                                 int y3 = 3*hband2 + row3;
                                 if ( A[ y1 ][ x1 ] == A[ y2 ][ x3 ]  &&
                                      A[ y1 ][ x1 ] == A[ y3 ][ x2 ]  &&
                                      A[ y1 ][ x2 ] == A[ y2 ][ x1 ]  &&
                                      A[ y1 ][ x2 ] == A[ y3 ][ x3 ] )
                                    {
                                    MySet set( swapped );
                                    set.AddElement( y1, x1 );
                                    set.AddElement( y1, x2 );
                                    set.AddElement( y2, x1 );
                                    set.AddElement( y2, x3 );
                                    set.AddElement( y3, x2 );
                                    set.AddElement( y3, x3 );
                                    set.Sort();
                                    sets.push_back( set );
                                    }
                                 }
                              }
                           }
                        }
                     }
                  }
               }
         }
   }


void RemoveDuplicateSets( vector< MySet >& sets )
   {
   // throw away duplicate sets
   sort( sets.begin(), sets.end() );
   for ( int i = 0;  i < static_cast< int >( sets.size() ) - 1; )
      {
      if ( sets[ i ] == sets[ i + 1 ] )
         sets.erase( sets.begin() + i );
      else
         i++;
      }
   }


void RemoveNonUnavoidableSets( vector< MySet >& sets, char const* pszGrid )
   {
   // delete sets that hit the diagonal or the back diagonal
   // (these sets may not actually be unavoidable for sudoku-x)
   for ( int i = 0;  i < static_cast< int >( sets.size() );  )
      {
      if ( ! sets[ i ].IsUnavoidable( pszGrid ) )
         sets.erase( sets.begin() + i );
      else
         i++;
      }
   }


char const Perms3[ 6 ][ 3 ] = { {0,1,2}, {0,2,1}, {1,0,2}, {1,2,0}, {2,0,1}, {2,1,0} };

#define COPY_PERM( iPerm, offset )        \
   for ( int x = 0;  x < 3;  x++ )     \
      aiRows[ offset + x ] = Perms3[ iPerm ][ x ];

// generate all possible permutations of the rows indices 0..8
// (there are 6^4: can swap the three bands, in within each band
// the three rows of the band)
void ComputeRowPermutations( char RowPerms[ 6*6*6*6 ][ 9 ] )
   {
   char aiRows[ 9 ];
   int counter = 0;
   for ( int i = 0;  i < 6;  i++ )
      {
      // permute top three rows
      COPY_PERM( i, 0 );
      for ( int j = 0;  j < 6;  j++ )
         {
         // permute middle three rows
         COPY_PERM( j, 3 );
         for ( int k = 0;  k < 6;  k++ )
            {
            // permute bottom three rows
            COPY_PERM( k, 6 );
            for ( int l = 0;  l < 6;  l++ )
               {
               // permute bands
               for ( int band = 0;  band < 3;  band++ )
                  for ( int s = 0;  s < 3;  s++ )
                     {
                     RowPerms[ counter ][ 3*band + s ] =
                        aiRows[ 3*band + s ] + 3*Perms3[ l ][ band ];
                     }
               counter++;
               }
            }
         }
      }
   }

struct SmallSet
   {
   char size;
   char data[ 20 ];
   };

struct UnavPat
   {
   int digits;             // # of different digits occurring
   SmallSet groups[ 9 ];   // each group consists of those cells containing the same digit
                           // we only use groups[ 0 ], ..., groups[ digits - 1 ]
   };

UnavPat unavpat[] =
   {
      // 9 sets of size 8
      {4, { {2,{17,98}}, {2,{18,67}}, {2,{27,68}}, {2,{28,97}} } },
      {2, { {4,{14,28,33,67}}, {4,{17,23,34,68}} } },
      {4, { {2,{24,76}}, {2,{26,44}}, {2,{45,74}}, {2,{46,75}} } },
      {3, { {3,{13,28,67}}, {2,{14,23}}, {3,{17,24,68}} } },
      {2, { {4,{11,26,48,53}}, {4,{16,23,41,58}} } },
      {3, { {3,{17,55,69}}, {3,{18,57,65}}, {2,{19,68}} } },
      {4, { {2,{41,65}}, {2,{45,76}}, {2,{46,61}}, {2,{66,75}} } },
      {2, { {4,{24,39,78,85}}, {4,{28,35,74,89}} } },
      {2, { {4,{46,67,88,94}}, {4,{47,64,86,98}} } },
      // 3 sets of size 9
      {3, { {3,{13,35,84}}, {3,{14,46,85}}, {3,{16,33,45}} } },
      {4, { {3,{44,75,97}}, {2,{45,76}}, {2,{46,94}}, {2,{77,96}} } },
      {3, { {3,{17,79,84}}, {3,{18,87,94}}, {3,{19,74,98}} } },
      // 47 sets of size 10 (these may not be all)
      {4, { {3,{38,75,99}}, {2,{39,58}}, {3,{59,78,92}}, {2,{72,95}} } },
      {4, { {3,{43,59,81}}, {3,{49,62,83}}, {2,{51,69}}, {2,{61,82}} } },
      {4, { {3,{14,28,47}}, {2,{17,34}}, {3,{24,37,98}}, {2,{48,97}} } },
      {3, { {4,{38,59,71,87}}, {4,{39,57,78,84}}, {2,{74,81}} } },
      {5, { {2,{51,69}}, {2,{54,61}}, {2,{55,84}}, {2,{59,65}}, {2,{64,85}} } },
      {5, { {2,{71,99}}, {2,{72,95}}, {2,{75,98}}, {2,{78,91}}, {2,{79,92}} } },
      {4, { {3,{31,53,82}}, {3,{32,51,93}}, {2,{83,97}}, {2,{87,92}} } },
      {5, { {2,{15,86}}, {2,{16,55}}, {2,{34,85}}, {2,{35,54}}, {2,{56,84}} } },
      {3, { {2,{11,35}}, {4,{15,37,78,93}}, {4,{18,31,73,97}} } },
      {3, { {3,{42,67,78}}, {4,{48,62,76,89}}, {3,{69,77,86}} } },
      {4, { {3,{24,66,75}}, {2,{26,44}}, {3,{35,46,74}}, {2,{36,65}} } },
      {3, { {3,{12,28,89}}, {3,{14,22,76}}, {4,{19,26,74,88}} } },
      {3, { {3,{28,42,69}}, {3,{29,78,86}}, {4,{48,62,76,89}} } },
      {3, { {3,{11,29,48}}, {3,{18,31,77}}, {4,{21,37,49,78}} } },
      {4, { {3,{24,79,96}}, {2,{26,44}}, {2,{34,46}}, {3,{36,74,99}} } },
      {2, { {5,{13,24,39,72,87}}, {5,{17,23,34,79,82}} } },
      {2, { {5,{12,46,57,63,85}}, {5,{13,47,52,65,86}} } },
      {5, { {2,{45,86}}, {2,{46,95}}, {2,{64,96}}, {2,{65,94}}, {2,{66,85}} } },
      {5, { {2,{27,68}}, {2,{28,57}}, {2,{58,97}}, {2,{67,99}}, {2,{69,98}} } },
      {2, { {5,{14,45,51,69,96}}, {5,{16,49,54,61,95}} } },
      {4, { {2,{42,65}}, {3,{43,58,64}}, {2,{44,62}}, {3,{45,53,68}} } },
      {4, { {2,{14,29}}, {3,{15,42,64}}, {2,{19,25}}, {3,{24,45,62}} } },
      {4, { {2,{12,35}}, {2,{13,62}}, {3,{15,23,38}}, {3,{28,32,63}} } },
      {5, { {2,{25,38}}, {2,{27,49}}, {2,{28,47}}, {2,{29,35}}, {2,{39,48}} } },
      {3, { {4,{13,25,42,64}}, {4,{15,22,44,53}}, {2,{54,63}} } },
      {3, { {4,{24,38,85,93}}, {4,{28,35,74,83}}, {2,{73,94}} } },
      {4, { {3,{15,76,91}}, {3,{16,81,95}}, {2,{71,88}}, {2,{78,86}} } },
      {4, { {3,{24,57,65}}, {2,{25,38}}, {2,{28,34}}, {3,{35,54,67}} } },
      {3, { {4,{24,65,72,96}}, {4,{25,66,84,92}}, {2,{74,82}} } },
      {3, { {2,{14,85}}, {4,{15,39,76,88}}, {4,{19,36,78,84}} } },
      {4, { {3,{11,62,73}}, {2,{13,91}}, {3,{61,72,96}}, {2,{76,93}} } },
      {2, { {5,{38,56,69,77,84}}, {5,{39,57,66,74,88}} } },
      {4, { {2,{15,37}}, {3,{17,45,58}}, {2,{18,35}}, {3,{38,47,55}} } },
      {4, { {3,{25,52,66}}, {3,{26,55,67}}, {2,{54,62}}, {2,{57,64}} } },
      {3, { {4,{12,26,74,83}}, {3,{13,72,94}}, {3,{24,86,93}} } },
      {3, { {4,{25,42,86,93}}, {4,{26,43,72,95}}, {2,{76,82}} } },
      {2, { {5,{15,29,47,76,88}}, {5,{17,26,49,78,85}} } },
      {3, { {4,{37,43,54,99}}, {3,{39,47,53}}, {3,{44,59,97}} } },
      {4, { {2,{31,42}}, {3,{32,48,63}}, {2,{33,71}}, {3,{41,68,73}} } },
      {3, { {3,{51,64,73}}, {3,{52,71,86}}, {4,{54,63,76,82}} } },
      {4, { {3,{24,55,62}}, {2,{25,56}}, {3,{26,44,52}}, {2,{42,64}} } },
      {4, { {3,{17,49,61}}, {2,{19,97}}, {3,{41,68,99}}, {2,{67,98}} } },
      {2, { {5,{12,26,37,75,83}}, {5,{17,23,35,72,86}} } },
      {3, { {3,{13,38,47}}, {3,{17,22,83}}, {4,{27,33,48,82}} } },
      {3, { {4,{15,39,76,97}}, {3,{17,35,79}}, {3,{36,77,95}} } },
      {4, { {2,{14,46}}, {2,{15,44}}, {3,{16,71,95}}, {3,{45,76,91}} } },
      {3, { {4,{25,37,58,66}}, {4,{28,36,45,67}}, {2,{48,55}} } },
      // size 11 (44 types, may not be all of them)
      {4, { {3,{34,58,66}}, {3,{36,54,75}}, {2,{53,68}}, {3,{55,63,76}} } },
      {4, { {3,{34,55,96}}, {3,{35,59,64}}, {3,{46,69,94}}, {2,{49,56}} } },
      {5, { {2,{21,83}}, {2,{23,61}}, {3,{43,59,81}}, {2,{49,63}}, {2,{51,69}} } },
      {5, { {3,{16,28,99}}, {2,{19,36}}, {2,{26,37}}, {2,{27,98}}, {2,{39,97}} } },
      {4, { {3,{11,34,95}}, {3,{13,24,37}}, {3,{15,23,94}}, {2,{17,31}} } },
      {5, { {3,{14,65,86}}, {2,{16,74}}, {2,{64,75}}, {2,{76,88}}, {2,{78,85}} } },
      {4, { {4,{16,37,65,98}}, {3,{18,26,97}}, {2,{25,66}}, {2,{27,35}} } },
      {5, { {2,{72,94}}, {2,{73,85}}, {3,{74,83,98}}, {2,{75,92}}, {2,{88,95}} } },
      {5, { {2,{15,37}}, {2,{17,79}}, {3,{19,35,68}}, {2,{38,67}}, {2,{69,77}} } },
      {3, { {4,{21,35,53,66}}, {4,{26,32,45,61}}, {3,{33,42,55}} } },
      {4, { {2,{16,37}}, {3,{17,49,52}}, {3,{19,36,48}}, {3,{38,42,57}} } },
      {4, { {3,{24,55,67}}, {3,{25,46,51}}, {3,{26,47,64}}, {2,{41,57}} } },
      {4, { {2,{13,34}}, {3,{14,65,96}}, {3,{16,23,95}}, {3,{25,33,64}} } },
      {5, { {2,{65,86}}, {3,{66,72,95}}, {2,{76,89}}, {2,{79,92}}, {2,{85,99}} } },
      {3, { {4,{34,76,89,92}}, {4,{36,64,72,95}}, {3,{65,84,99}} } },
      {5, { {3,{37,78,95}}, {2,{38,89}}, {2,{39,87}}, {2,{75,88}}, {2,{85,97}} } },
      {4, { {2,{16,39}}, {4,{19,26,67,88}}, {2,{28,36}}, {3,{38,69,87}} } },
      {4, { {2,{12,73}}, {2,{13,28}}, {4,{18,21,72,87}}, {3,{23,77,81}} } },
      {4, { {3,{14,46,75}}, {2,{15,64}}, {3,{44,76,97}}, {3,{65,77,94}} } },
      {3, { {4,{28,49,65,76}}, {3,{29,53,68}}, {4,{46,59,63,75}} } },
      {3, { {3,{14,81,95}}, {4,{15,67,78,84}}, {4,{68,75,87,91}} } },
      {5, { {3,{42,54,68}}, {2,{46,52}}, {2,{48,66}}, {2,{56,67}}, {2,{57,64}} } },
      {4, { {3,{11,29,63}}, {2,{12,81}}, {2,{19,32}}, {4,{23,39,61,82}} } },
      {4, { {2,{42,64}}, {3,{44,86,98}}, {2,{46,52}}, {4,{56,62,88,94}} } },
      {3, { {4,{21,46,62,83}}, {4,{22,53,67,81}}, {3,{43,57,66}} } },
      {3, { {4,{14,41,75,92}}, {4,{15,43,71,84}}, {3,{42,83,94}} } },
      {4, { {3,{24,55,96}}, {3,{25,73,94}}, {3,{56,85,93}}, {2,{75,83}} } },
      {3, { {3,{27,79,94}}, {4,{29,33,71,87}}, {4,{31,74,83,97}} } },
      {4, { {2,{54,96}}, {2,{55,74}}, {4,{56,68,75,97}}, {3,{67,78,94}} } },
      {4, { {3,{11,87,93}}, {2,{13,27}}, {2,{17,88}}, {4,{18,23,81,97}} } },
      {4, { {3,{11,42,69}}, {4,{12,49,53,91}}, {2,{59,63}}, {2,{61,93}} } },
      {3, { {4,{46,68,89,94}}, {3,{48,56,62}}, {4,{52,64,86,99}} } },
      {3, { {4,{12,39,88,93}}, {4,{18,22,83,99}}, {3,{29,33,82}} } },
      {3, { {4,{12,39,88,93}}, {4,{15,33,82,98}}, {3,{18,35,99}} } },
      {4, { {4,{12,26,44,91}}, {3,{14,31,92}}, {2,{21,36}}, {2,{34,46}} } },
      {3, { {4,{55,68,76,97}}, {4,{58,64,87,95}}, {3,{66,77,84}} } },
      {5, { {3,{41,68,93}}, {2,{48,51}}, {2,{53,91}}, {2,{55,63}}, {2,{58,65}} } },
      {4, { {2,{12,28}}, {3,{13,42,68}}, {3,{18,21,63}}, {3,{22,48,61}} } },
      {4, { {2,{16,39}}, {4,{19,24,56,67}}, {3,{27,36,54}}, {2,{37,69}} } },
      {3, { {4,{42,67,76,99}}, {3,{44,57,62}}, {4,{49,54,77,96}} } },
      {4, { {2,{11,63}}, {3,{13,26,44}}, {3,{14,31,46}}, {3,{23,36,61}} } },
      {3, { {3,{34,75,99}}, {4,{35,54,69,98}}, {4,{58,65,79,94}} } },
      {3, { {3,{44,75,92}}, {4,{45,58,89,94}}, {4,{59,72,85,98}} } },
      {3, { {4,{16,39,74,87}}, {4,{17,34,79,85}}, {3,{19,35,86}} } },
      // size 12 (417 types, but again it is not proved that these are all)
      {5, { {3,{21,43,56}}, {3,{23,51,82}}, {2,{32,83}}, {2,{33,42}}, {2,{46,52}} } },
      {4, { {2,{24,66}}, {4,{26,55,78,84}}, {3,{56,75,98}}, {3,{64,88,96}} } },
      {3, { {5,{13,27,69,88,91}}, {5,{18,31,67,83,99}}, {2,{21,37}} } },
      {5, { {3,{11,43,92}}, {3,{13,62,91}}, {2,{42,68}}, {2,{46,63}}, {2,{48,66}} } },
      {6, { {2,{21,82}}, {2,{22,51}}, {2,{52,64}}, {2,{54,68}}, {2,{58,61}}, {2,{62,81}} } },
      {6, { {2,{21,82}}, {2,{22,51}}, {2,{31,72}}, {2,{32,61}}, {2,{52,71}}, {2,{62,81}} } },
      {5, { {3,{35,47,66}}, {3,{36,45,84}}, {2,{41,67}}, {2,{44,61}}, {2,{64,86}} } },
      {6, { {2,{51,83}}, {2,{52,64}}, {2,{53,82}}, {2,{54,68}}, {2,{58,61}}, {2,{62,81}} } },
      {5, { {3,{45,51,67}}, {3,{47,55,62}}, {2,{52,64}}, {2,{54,68}}, {2,{58,61}} } },
      {4, { {3,{17,69,98}}, {3,{19,58,97}}, {3,{24,56,68}}, {3,{26,59,64}} } },
      {5, { {2,{18,69}}, {2,{19,98}}, {3,{24,56,68}}, {3,{26,59,64}}, {2,{58,99}} } },
      {5, { {3,{15,39,84}}, {3,{19,25,87}}, {2,{24,85}}, {2,{27,34}}, {2,{37,89}} } },
      {5, { {3,{22,49,53}}, {2,{23,82}}, {2,{32,83}}, {3,{33,42,54}}, {2,{44,59}} } },
      {4, { {4,{12,28,35,59}}, {3,{15,22,37}}, {2,{27,98}}, {3,{39,58,97}} } },
      {6, { {2,{14,86}}, {2,{15,74}}, {2,{16,95}}, {2,{44,75}}, {2,{45,96}}, {2,{46,84}} } },
      {3, { {4,{11,49,72,88}}, {4,{12,47,79,93}}, {4,{13,48,81,97}} } },
      {4, { {4,{21,36,43,55}}, {2,{26,38}}, {4,{28,33,47,51}}, {2,{45,57}} } },
      {3, { {5,{11,25,39,56,67}}, {5,{16,29,31,52,65}}, {2,{57,62}} } },
      {4, { {2,{12,53}}, {4,{13,42,89,91}}, {4,{31,52,83,99}}, {2,{32,41}} } },
      {5, { {3,{12,35,64}}, {3,{14,23,82}}, {2,{24,65}}, {2,{25,33}}, {2,{32,83}} } },
      {5, { {3,{12,35,83}}, {3,{14,23,82}}, {2,{15,64}}, {2,{24,65}}, {2,{25,33}} } },
      {4, { {3,{25,39,58}}, {3,{28,35,77}}, {3,{37,59,65}}, {3,{55,67,78}} } },
      {4, { {4,{21,45,52,83}}, {3,{23,42,71}}, {3,{43,55,81}}, {2,{51,72}} } },
      {5, { {2,{41,64}}, {2,{43,58}}, {3,{44,53,67}}, {3,{48,55,61}}, {2,{57,65}} } },
      {4, { {4,{21,67,72,89}}, {2,{22,41}}, {4,{42,68,77,81}}, {2,{69,88}} } },
      {2, { {6,{28,35,59,66,84,97}}, {6,{29,36,54,67,88,95}} } },
      {3, { {4,{11,48,63,87}}, {4,{13,49,88,91}}, {4,{41,67,83,99}} } },
      {4, { {3,{11,34,76}}, {4,{16,28,45,74}}, {3,{18,25,31}}, {2,{35,44}} } },
      {6, { {2,{22,63}}, {2,{23,82}}, {2,{31,62}}, {2,{32,43}}, {2,{33,61}}, {2,{42,83}} } },
      {3, { {5,{21,53,65,84,92}}, {5,{23,54,62,86,91}}, {2,{66,85}} } },
      {4, { {4,{27,36,59,65}}, {4,{28,35,54,67}}, {2,{29,58}}, {2,{34,56}} } },
      {3, { {4,{11,67,89,93}}, {4,{13,48,61,97}}, {4,{43,69,81,98}} } },
      {3, { {5,{15,27,58,86,99}}, {2,{16,65}}, {5,{28,59,66,87,95}} } },
      {3, { {4,{13,36,45,62}}, {4,{15,32,73,96}}, {4,{43,66,75,92}} } },
      {5, { {3,{14,65,96}}, {3,{15,44,68}}, {2,{45,94}}, {2,{46,95}}, {2,{48,66}} } },
      {5, { {2,{34,76}}, {3,{36,45,84}}, {2,{43,65}}, {3,{46,63,74}}, {2,{64,85}} } },
      {5, { {2,{14,55}}, {2,{15,66}}, {2,{16,54}}, {3,{42,57,65}}, {3,{47,56,62}} } },
      {2, { {6,{15,22,38,49,54,96}}, {6,{18,26,32,45,59,94}} } },
      {3, { {5,{12,26,45,59,94}}, {5,{15,28,49,54,96}}, {2,{18,22}} } },
      {4, { {4,{21,36,65,74}}, {4,{25,31,54,76}}, {2,{51,64}}, {2,{55,61}} } },
      {5, { {3,{17,74,98}}, {2,{18,57}}, {3,{58,87,93}}, {2,{73,94}}, {2,{77,83}} } },
      {5, { {2,{18,89}}, {2,{19,58}}, {3,{41,59,64}}, {3,{49,61,88}}, {2,{54,68}} } },
      {3, { {4,{21,36,74,92}}, {4,{22,34,51,66}}, {4,{54,62,71,96}} } },
      {3, { {5,{42,66,79,81,95}}, {4,{45,61,72,96}}, {3,{71,86,99}} } },
      {5, { {4,{11,38,44,69}}, {2,{17,31}}, {2,{19,67}}, {2,{37,48}}, {2,{47,64}} } },
      {5, { {3,{14,58,66}}, {2,{16,38}}, {2,{18,24}}, {2,{28,34}}, {3,{36,54,68}} } },
      {4, { {3,{31,62,73}}, {3,{33,51,72}}, {3,{37,59,61}}, {3,{39,52,67}} } },
      {5, { {2,{12,39}}, {3,{19,22,87}}, {2,{21,42}}, {3,{27,31,89}}, {2,{32,41}} } },
      {3, { {5,{11,29,34,86,97}}, {5,{16,21,39,87,95}}, {2,{35,94}} } },
      {5, { {2,{24,45}}, {3,{25,54,62}}, {3,{34,52,65}}, {2,{35,46}}, {2,{36,44}} } },
      {6, { {2,{14,75}}, {2,{15,84}}, {2,{71,88}}, {2,{74,89}}, {2,{78,85}}, {2,{79,81}} } },
      {4, { {4,{14,22,39,66}}, {2,{16,38}}, {4,{18,26,32,64}}, {2,{19,36}} } },
      {5, { {2,{11,72}}, {3,{12,26,64}}, {3,{14,29,66}}, {2,{19,21}}, {2,{22,71}} } },
      {6, { {2,{41,54}}, {2,{43,91}}, {2,{44,58}}, {2,{48,63}}, {2,{51,93}}, {2,{53,68}} } },
      {3, { {2,{16,29}}, {5,{19,35,54,86,97}}, {5,{26,39,55,87,94}} } },
      {5, { {2,{12,39}}, {3,{19,22,87}}, {2,{27,68}}, {2,{28,32}}, {3,{38,67,89}} } },
      {5, { {2,{12,83}}, {3,{13,61,72}}, {2,{21,63}}, {3,{23,42,71}}, {2,{43,82}} } },
      {4, { {4,{11,29,63,78}}, {4,{18,23,49,71}}, {2,{48,79}}, {2,{61,73}} } },
      {3, { {5,{14,59,66,77,95}}, {5,{15,56,67,74,89}}, {2,{85,99}} } },
      {2, { {6,{14,59,66,77,85,93}}, {6,{15,56,67,74,83,99}} } },
      {4, { {4,{17,41,59,93}}, {2,{18,77}}, {2,{19,78}}, {4,{47,53,79,91}} } },
      {5, { {2,{41,54}}, {2,{44,58}}, {3,{48,71,89}}, {2,{49,51}}, {3,{59,78,81}} } },
      {4, { {3,{12,38,89}}, {4,{19,22,77,81}}, {3,{27,32,78}}, {2,{71,88}} } },
      {5, { {3,{27,79,93}}, {3,{29,85,97}}, {2,{73,84}}, {2,{74,89}}, {2,{83,95}} } },
      {5, { {2,{14,27}}, {2,{16,38}}, {3,{17,36,89}}, {2,{18,34}}, {3,{24,39,87}} } },
      {6, { {2,{41,73}}, {2,{42,91}}, {2,{43,82}}, {2,{51,92}}, {2,{52,83}}, {2,{53,71}} } },
      {5, { {2,{11,28}}, {2,{15,37}}, {3,{17,21,34}}, {3,{18,22,35}}, {2,{24,32}} } },
      {4, { {3,{21,53,72}}, {3,{22,51,83}}, {3,{73,89,96}}, {3,{76,82,99}} } },
      {5, { {2,{11,34}}, {3,{14,22,71}}, {3,{24,33,72}}, {2,{31,43}}, {2,{41,73}} } },
      {4, { {2,{43,55}}, {3,{45,59,88}}, {4,{46,53,67,89}}, {3,{48,66,87}} } },
      {5, { {3,{41,55,96}}, {2,{42,91}}, {3,{46,58,95}}, {2,{48,52}}, {2,{51,92}} } },
      {4, { {2,{14,95}}, {4,{15,44,52,68}}, {4,{34,42,58,65}}, {2,{35,94}} } },
      {3, { {4,{14,37,48,65}}, {4,{17,24,55,68}}, {4,{27,35,44,58}} } },
      {3, { {4,{14,37,42,55}}, {4,{17,24,48,52}}, {4,{27,35,44,58}} } },
      {5, { {2,{31,72}}, {4,{32,41,79,96}}, {2,{42,91}}, {2,{71,99}}, {2,{76,92}} } },
      {5, { {3,{15,38,46}}, {2,{18,25}}, {3,{28,36,55}}, {2,{45,96}}, {2,{56,95}} } },
      {4, { {3,{11,36,74}}, {4,{14,37,76,98}}, {3,{18,31,47}}, {2,{48,97}} } },
      {4, { {4,{44,67,76,89}}, {4,{47,56,79,94}}, {2,{57,66}}, {2,{84,99}} } },
      {4, { {2,{15,28}}, {4,{18,36,45,87}}, {4,{24,37,46,88}}, {2,{25,44}} } },
      {3, { {5,{11,25,66,74,83}}, {5,{14,33,65,76,81}}, {2,{23,35}} } },
      {5, { {2,{11,72}}, {3,{12,38,67}}, {2,{17,21}}, {2,{22,71}}, {3,{27,32,68}} } },
      {4, { {2,{11,72}}, {4,{12,46,51,68}}, {3,{42,66,73}}, {3,{58,63,71}} } },
      {3, { {3,{12,24,53}}, {4,{14,28,69,87}}, {5,{17,23,52,68,89}} } },
      {5, { {3,{21,75,82}}, {3,{22,71,95}}, {2,{85,98}}, {2,{86,92}}, {2,{88,96}} } },
      {4, { {4,{16,28,79,95}}, {3,{19,35,96}}, {3,{25,37,78}}, {2,{39,77}} } },
      {5, { {2,{31,63}}, {3,{33,89,91}}, {3,{61,83,94}}, {2,{74,99}}, {2,{79,84}} } },
      {3, { {4,{11,25,74,82}}, {4,{14,33,71,96}}, {4,{22,36,85,93}} } },
      {6, { {2,{14,96}}, {2,{16,64}}, {2,{34,66}}, {2,{36,74}}, {2,{54,76}}, {2,{56,94}} } },
      {3, { {4,{13,29,81,98}}, {3,{18,32,53}}, {5,{21,38,52,83,99}} } },
      {3, { {5,{37,59,66,78,84}}, {5,{39,53,68,74,87}}, {2,{56,63}} } },
      {5, { {2,{12,53}}, {4,{13,41,66,92}}, {2,{46,51}}, {2,{52,91}}, {2,{56,63}} } },
      {6, { {2,{27,49}}, {2,{29,87}}, {2,{47,78}}, {2,{48,89}}, {2,{74,88}}, {2,{77,84}} } },
      {5, { {2,{21,42}}, {2,{22,61}}, {3,{41,54,72}}, {2,{44,62}}, {3,{52,64,71}} } },
      {5, { {3,{54,73,96}}, {3,{56,84,98}}, {2,{74,88}}, {2,{75,93}}, {2,{78,95}} } },
      {4, { {3,{25,37,96}}, {4,{27,36,49,55}}, {3,{46,59,94}}, {2,{54,95}} } },
      {5, { {2,{24,46}}, {3,{26,74,87}}, {2,{44,59}}, {2,{49,56}}, {3,{54,77,86}} } },
      {6, { {2,{21,42}}, {2,{22,73}}, {2,{23,34}}, {2,{24,31}}, {2,{33,41}}, {2,{43,72}} } },
      {4, { {4,{12,49,54,63}}, {3,{13,61,92}}, {3,{44,51,69}}, {2,{52,91}} } },
      {5, { {2,{11,92}}, {3,{12,27,73}}, {3,{17,23,71}}, {2,{72,97}}, {2,{77,91}} } },
      {4, { {3,{11,72,98}}, {3,{12,27,73}}, {4,{17,23,75,91}}, {2,{78,95}} } },
      {4, { {3,{11,27,63}}, {4,{17,23,75,91}}, {3,{61,73,98}}, {2,{78,95}} } },
      {5, { {2,{37,98}}, {3,{38,74,87}}, {3,{73,84,97}}, {2,{75,93}}, {2,{78,95}} } },
      {4, { {3,{41,55,68}}, {3,{43,56,67}}, {3,{47,53,65}}, {3,{48,51,66}} } },
      {4, { {4,{24,58,75,89}}, {2,{25,64}}, {4,{59,65,84,98}}, {2,{78,95}} } },
      {5, { {2,{18,49}}, {2,{19,98}}, {2,{28,99}}, {3,{29,72,88}}, {3,{48,79,82}} } },
      {5, { {3,{11,29,98}}, {2,{17,21}}, {2,{18,49}}, {2,{19,47}}, {3,{27,48,99}} } },
      {4, { {3,{23,35,64}}, {3,{24,33,55}}, {3,{27,58,65}}, {3,{28,54,67}} } },
      {5, { {3,{43,65,82}}, {3,{45,52,83}}, {2,{51,72}}, {2,{55,61}}, {2,{62,71}} } },
      {4, { {4,{16,29,82,94}}, {4,{18,24,86,92}}, {2,{19,98}}, {2,{28,99}} } },
      {5, { {3,{13,29,47}}, {2,{14,38}}, {3,{17,24,49}}, {2,{18,33}}, {2,{23,34}} } },
      {5, { {2,{15,37}}, {3,{17,52,68}}, {3,{18,35,69}}, {2,{39,57}}, {2,{59,62}} } },
      {5, { {3,{65,78,96}}, {3,{66,75,87}}, {2,{74,88}}, {2,{77,84}}, {2,{86,98}} } },
      {5, { {3,{44,51,69}}, {3,{49,54,62}}, {2,{52,67}}, {2,{55,61}}, {2,{57,65}} } },
      {5, { {2,{22,73}}, {3,{23,82,97}}, {2,{72,89}}, {3,{79,85,93}}, {2,{87,95}} } },
      {5, { {2,{11,28}}, {2,{13,27}}, {3,{17,26,31}}, {3,{18,23,36}}, {2,{21,38}} } },
      {2, { {6,{34,45,62,76,83,99}}, {6,{36,42,64,73,89,95}} } },
      {4, { {4,{65,78,86,91}}, {3,{66,75,87}}, {3,{74,81,98}}, {2,{77,84}} } },
      {4, { {2,{14,96}}, {4,{16,54,61,83}}, {4,{21,53,66,94}}, {2,{23,81}} } },
      {5, { {3,{27,69,78}}, {3,{28,54,67}}, {2,{44,59}}, {2,{49,64}}, {2,{58,79}} } },
      {3, { {3,{13,29,47}}, {4,{17,32,53,98}}, {5,{23,38,49,52,97}} } },
      {5, { {3,{21,34,53}}, {3,{24,32,61}}, {2,{33,52}}, {2,{51,72}}, {2,{62,71}} } },
      {4, { {3,{27,69,78}}, {4,{28,59,85,97}}, {2,{58,79}}, {3,{67,89,95}} } },
      {5, { {2,{17,79}}, {3,{19,41,67}}, {2,{43,91}}, {3,{49,63,77}}, {2,{61,93}} } },
      {5, { {3,{21,48,62}}, {3,{22,51,67}}, {2,{27,68}}, {2,{28,57}}, {2,{41,58}} } },
      {6, { {2,{44,86}}, {2,{45,84}}, {2,{46,67}}, {2,{47,55}}, {2,{57,65}}, {2,{66,85}} } },
      {5, { {2,{41,69}}, {2,{44,86}}, {2,{46,51}}, {3,{49,65,84}}, {3,{56,61,85}} } },
      {6, { {2,{42,54}}, {2,{44,62}}, {2,{51,64}}, {2,{52,67}}, {2,{55,61}}, {2,{57,65}} } },
      {4, { {3,{42,84,93}}, {4,{43,59,78,82}}, {3,{58,74,89}}, {2,{73,94}} } },
      {5, { {3,{24,37,66}}, {2,{25,64}}, {3,{27,36,45}}, {2,{46,75}}, {2,{65,76}} } },
      {4, { {4,{45,57,63,76}}, {4,{47,52,66,75}}, {2,{53,69}}, {2,{59,62}} } },
      {4, { {4,{27,33,48,61}}, {3,{28,54,67}}, {3,{31,43,58}}, {2,{51,64}} } },
      {4, { {2,{27,49}}, {3,{29,57,88}}, {4,{34,45,58,89}}, {3,{35,47,54}} } },
      {6, { {2,{21,42}}, {2,{22,61}}, {2,{41,58}}, {2,{48,52}}, {2,{51,72}}, {2,{62,71}} } },
      {5, { {2,{52,67}}, {4,{57,61,79,92}}, {2,{62,71}}, {2,{69,77}}, {2,{72,97}} } },
      {4, { {4,{46,59,74,92}}, {3,{49,54,86}}, {3,{72,87,96}}, {2,{77,84}} } },
      {5, { {2,{42,83}}, {3,{43,75,92}}, {2,{65,76}}, {3,{66,82,95}}, {2,{73,86}} } },
      {4, { {3,{24,45,86}}, {4,{26,54,61,85}}, {3,{34,51,65}}, {2,{35,44}} } },
      {4, { {2,{24,46}}, {3,{25,44,68}}, {4,{26,41,65,93}}, {3,{48,63,91}} } },
      {2, { {6,{13,26,38,44,69,85}}, {6,{19,24,33,48,65,86}} } },
      {3, { {5,{13,38,46,69,85}}, {5,{19,33,48,65,84}}, {2,{44,86}} } },
      {3, { {5,{13,25,38,44,69}}, {5,{19,33,48,65,84}}, {2,{24,85}} } },
      {4, { {3,{34,45,59}}, {3,{35,47,54}}, {3,{46,57,63}}, {3,{49,53,66}} } },
      {4, { {3,{14,39,98}}, {3,{18,24,99}}, {3,{22,34,93}}, {3,{29,33,92}} } },
      {4, { {2,{12,53}}, {2,{13,62}}, {4,{45,57,63,76}}, {4,{47,52,66,75}} } },
      {3, { {5,{11,27,52,64,73}}, {5,{17,23,44,61,72}}, {2,{42,54}} } },
      {4, { {4,{13,38,45,69}}, {4,{19,33,58,62}}, {2,{42,65}}, {2,{48,52}} } },
      {4, { {4,{13,38,45,69}}, {4,{19,33,48,64}}, {2,{42,65}}, {2,{44,62}} } },
      {3, { {4,{44,58,77,95}}, {4,{45,59,86,98}}, {4,{46,57,74,89}} } },
      {4, { {3,{11,26,82}}, {4,{16,32,81,94}}, {3,{22,34,56}}, {2,{54,96}} } },
      {4, { {3,{11,39,77}}, {3,{17,22,81}}, {4,{27,31,59,82}}, {2,{57,79}} } },
      {5, { {3,{11,26,85}}, {2,{12,81}}, {2,{15,32}}, {2,{21,82}}, {3,{22,35,86}} } },
      {4, { {3,{13,37,81}}, {3,{17,23,89}}, {4,{29,31,63,87}}, {2,{61,83}} } },
      {5, { {3,{42,57,88}}, {3,{48,63,82}}, {2,{52,83}}, {2,{53,67}}, {2,{68,87}} } },
      {4, { {4,{37,58,89,91}}, {4,{39,57,85,98}}, {2,{81,97}}, {2,{87,95}} } },
      {4, { {4,{37,58,89,91}}, {4,{39,57,85,98}}, {2,{71,95}}, {2,{75,81}} } },
      {4, { {2,{42,58}}, {4,{48,63,72,89}}, {4,{52,68,79,81}}, {2,{61,83}} } },
      {3, { {5,{13,29,36,48,62}}, {5,{18,26,32,43,59}}, {2,{52,69}} } },
      {3, { {4,{42,59,86,93}}, {4,{44,53,82,96}}, {4,{49,56,83,94}} } },
      {5, { {2,{15,46}}, {3,{16,83,95}}, {3,{45,86,99}}, {2,{84,93}}, {2,{89,94}} } },
      {3, { {4,{15,21,38,67}}, {5,{17,25,31,76,88}}, {3,{68,77,86}} } },
      {4, { {3,{11,38,67}}, {2,{17,21}}, {4,{27,31,76,88}}, {3,{68,77,86}} } },
      {5, { {2,{11,25}}, {2,{14,31}}, {3,{15,84,99}}, {2,{21,35}}, {3,{34,89,95}} } },
      {5, { {3,{14,27,95}}, {3,{17,35,94}}, {2,{25,38}}, {2,{28,77}}, {2,{37,78}} } },
      {4, { {4,{11,37,49,63}}, {4,{19,23,41,57}}, {2,{27,33}}, {2,{53,67}} } },
      {2, { {6,{13,29,36,48,52,64}}, {6,{18,26,32,43,54,69}} } },
      {5, { {2,{14,76}}, {3,{16,27,55}}, {2,{17,34}}, {2,{24,37}}, {3,{25,56,74}} } },
      {4, { {3,{16,29,68}}, {4,{18,36,79,94}}, {2,{26,39}}, {3,{69,74,98}} } },
      {4, { {4,{11,36,43,68}}, {4,{16,23,48,51}}, {2,{26,33}}, {2,{58,61}} } },
      {4, { {4,{11,36,43,68}}, {2,{16,39}}, {4,{19,33,48,51}}, {2,{58,61}} } },
      {3, { {4,{13,29,71,98}}, {4,{14,23,75,91}}, {4,{19,25,78,94}} } },
      {3, { {5,{14,39,47,56,62}}, {5,{19,26,44,52,67}}, {2,{29,36}} } },
      {4, { {3,{41,82,95}}, {3,{42,59,83}}, {4,{49,53,87,91}}, {2,{85,97}} } },
      {3, { {4,{13,34,75,87}}, {4,{14,22,77,83}}, {4,{24,33,72,85}} } },
      {4, { {4,{11,35,84,93}}, {4,{14,23,55,91}}, {2,{25,33}}, {2,{54,85}} } },
      {4, { {2,{15,46}}, {4,{16,48,79,95}}, {3,{45,76,92}}, {3,{49,72,98}} } },
      {2, { {6,{12,26,37,74,81,99}}, {6,{17,21,36,79,84,92}} } },
      {3, { {4,{41,75,82,97}}, {3,{42,59,83}}, {5,{49,53,77,85,91}} } },
      {4, { {4,{54,68,71,85}}, {3,{56,74,88}}, {3,{58,65,86}}, {2,{78,81}} } },
      {3, { {4,{13,28,69,87}}, {5,{19,32,67,73,88}}, {3,{23,38,72}} } },
      {3, { {5,{11,29,54,77,85}}, {3,{14,21,55}}, {4,{17,25,79,84}} } },
      {5, { {3,{11,28,42}}, {2,{13,91}}, {2,{18,33}}, {2,{22,38}}, {3,{32,41,93}} } },
      {2, { {6,{11,26,43,74,82,98}}, {6,{14,23,42,78,86,91}} } },
      {5, { {2,{42,64}}, {2,{43,65}}, {3,{44,69,98}}, {2,{45,62}}, {3,{48,63,99}} } },
      {3, { {3,{27,86,99}}, {5,{29,42,57,78,96}}, {4,{48,52,76,87}} } },
      {6, { {2,{43,65}}, {2,{45,53}}, {2,{55,67}}, {2,{57,88}}, {2,{58,63}}, {2,{68,87}} } },
      {6, { {2,{37,59}}, {2,{39,67}}, {2,{53,69}}, {2,{57,88}}, {2,{58,63}}, {2,{68,87}} } },
      {4, { {3,{11,84,93}}, {4,{13,38,82,94}}, {3,{18,22,81}}, {2,{28,32}} } },
      {3, { {5,{12,27,75,81,98}}, {4,{13,72,88,95}}, {3,{17,21,83}} } },
      {4, { {3,{11,24,37}}, {3,{12,28,35}}, {3,{15,21,38}}, {3,{17,22,34}} } },
      {5, { {3,{11,24,37}}, {2,{12,38}}, {3,{17,22,34}}, {2,{18,21}}, {2,{28,32}} } },
      {4, { {2,{46,59}}, {4,{49,65,78,96}}, {4,{54,69,75,98}}, {2,{56,94}} } },
      {5, { {3,{25,47,56}}, {2,{26,85}}, {2,{43,57}}, {3,{45,63,86}}, {2,{53,66}} } },
      {6, { {2,{42,54}}, {2,{44,67}}, {2,{45,62}}, {2,{47,55}}, {2,{52,65}}, {2,{57,64}} } },
      {4, { {4,{11,28,77,82}}, {2,{17,99}}, {4,{19,21,88,92}}, {2,{72,97}} } },
      {6, { {2,{47,88}}, {2,{48,89}}, {2,{49,97}}, {2,{67,98}}, {2,{68,99}}, {2,{69,87}} } },
      {5, { {3,{72,89,96}}, {2,{73,85}}, {2,{75,92}}, {3,{76,83,99}}, {2,{82,95}} } },
      {4, { {4,{14,27,58,86}}, {4,{18,25,59,84}}, {2,{26,85}}, {2,{29,57}} } },
      {3, { {5,{23,39,48,52,91}}, {5,{29,31,43,68,92}}, {2,{58,62}} } },
      {3, { {4,{22,39,43,57}}, {4,{27,33,48,51}}, {4,{28,31,49,52}} } },
      {4, { {4,{52,64,73,95}}, {2,{55,96}}, {4,{56,61,74,92}}, {2,{63,71}} } },
      {4, { {4,{24,66,85,98}}, {3,{26,54,75}}, {3,{55,88,94}}, {2,{65,76}} } },
      {4, { {4,{24,42,57,96}}, {3,{25,44,52}}, {3,{26,47,55}}, {2,{46,94}} } },
      {4, { {3,{12,36,73}}, {4,{16,21,45,72}}, {3,{25,31,46}}, {2,{33,71}} } },
      {4, { {3,{14,43,55}}, {4,{15,44,68,76}}, {2,{48,53}}, {3,{58,66,75}} } },
      {4, { {3,{14,43,55}}, {3,{15,64,76}}, {3,{44,53,68}}, {3,{58,66,75}} } },
      {5, { {3,{15,38,57}}, {2,{16,29}}, {2,{17,26}}, {2,{19,25}}, {3,{27,35,58}} } },
      {5, { {2,{14,56}}, {2,{15,74}}, {2,{16,95}}, {3,{45,59,96}}, {3,{49,54,75}} } },
      {6, { {2,{17,98}}, {2,{18,89}}, {2,{19,77}}, {2,{75,97}}, {2,{79,85}}, {2,{88,95}} } },
      {3, { {5,{15,27,44,69,96}}, {4,{16,47,65,94}}, {3,{19,26,67}} } },
      {5, { {3,{43,56,94}}, {3,{44,53,68}}, {2,{55,96}}, {2,{58,65}}, {2,{64,95}} } },
      {4, { {2,{12,83}}, {3,{13,25,74}}, {4,{14,21,75,82}}, {3,{23,71,84}} } },
      {5, { {2,{21,53}}, {3,{22,51,69}}, {3,{23,44,62}}, {2,{43,54}}, {2,{59,64}} } },
      {4, { {4,{11,28,34,76}}, {2,{13,36}}, {3,{16,21,74}}, {3,{18,24,33}} } },
      {6, { {2,{24,96}}, {2,{26,74}}, {2,{27,78}}, {2,{28,97}}, {2,{76,98}}, {2,{77,94}} } },
      {4, { {3,{15,76,82}}, {2,{16,95}}, {4,{55,72,87,96}}, {3,{56,77,85}} } },
      {5, { {3,{42,58,87}}, {3,{47,62,88}}, {2,{52,69}}, {2,{54,68}}, {2,{59,64}} } },
      {3, { {4,{16,37,78,94}}, {4,{17,34,46,69}}, {4,{49,64,76,98}} } },
      {4, { {3,{12,37,99}}, {2,{18,22}}, {3,{19,88,93}}, {4,{28,32,83,97}} } },
      {2, { {6,{13,36,41,55,84,92}}, {6,{15,33,46,52,81,94}} } },
      {4, { {4,{13,25,54,62}}, {3,{14,22,63}}, {3,{24,53,66}}, {2,{26,65}} } },
      {5, { {2,{42,56}}, {2,{45,52}}, {3,{46,71,95}}, {2,{54,76}}, {3,{55,74,91}} } },
      {5, { {2,{13,27}}, {2,{14,23}}, {3,{17,34,78}}, {3,{24,38,67}}, {2,{68,77}} } },
      {3, { {5,{17,48,64,76,99}}, {5,{19,45,67,78,94}}, {2,{46,75}} } },
      {4, { {3,{21,86,92}}, {4,{22,43,56,81}}, {3,{46,52,93}}, {2,{83,96}} } },
      {4, { {2,{22,39}}, {2,{28,32}}, {4,{29,47,63,88}}, {4,{38,43,69,87}} } },
      {3, { {5,{34,43,55,86,92}}, {4,{35,56,83,94}}, {3,{46,52,93}} } },
      {4, { {4,{13,28,35,79}}, {2,{17,33}}, {2,{19,57}}, {4,{25,37,59,78}} } },
      {4, { {4,{13,29,36,72}}, {3,{16,21,73}}, {3,{22,39,71}}, {2,{26,31}} } },
      {5, { {2,{11,27}}, {3,{15,33,61}}, {2,{17,25}}, {2,{21,62}}, {3,{22,35,63}} } },
      {2, { {6,{23,34,52,66,81,95}}, {6,{25,33,54,62,86,91}} } },
      {4, { {4,{42,58,76,93}}, {3,{48,61,72}}, {2,{51,68}}, {3,{53,71,96}} } },
      {3, { {4,{12,34,77,81}}, {5,{14,38,51,72,87}}, {3,{18,31,52}} } },
      {2, { {6,{13,25,39,41,67,88}}, {6,{17,21,35,48,63,89}} } },
      {3, { {5,{42,55,68,86,91}}, {2,{43,82}}, {5,{46,58,61,83,95}} } },
      {4, { {4,{42,65,86,91}}, {2,{43,82}}, {4,{46,51,83,95}}, {2,{55,61}} } },
      {5, { {2,{42,65}}, {3,{45,76,97}}, {2,{46,62}}, {3,{66,77,85}}, {2,{87,95}} } },
      {3, { {4,{25,39,47,56}}, {5,{27,36,59,64,75}}, {3,{45,67,74}} } },
      {6, { {2,{11,35}}, {2,{14,46}}, {2,{15,54}}, {2,{16,31}}, {2,{34,55}}, {2,{36,44}} } },
      {3, { {5,{13,35,67,89,96}}, {4,{16,39,85,97}}, {3,{17,33,69}} } },
      {2, { {6,{15,39,48,54,87,96}}, {6,{16,37,45,58,84,99}} } },
      {3, { {4,{25,38,81,94}}, {5,{28,34,53,75,91}}, {3,{51,73,85}} } },
      {4, { {4,{13,35,51,69}}, {3,{15,31,43}}, {2,{41,93}}, {3,{59,63,91}} } },
      {4, { {3,{15,27,48}}, {4,{18,25,57,79}}, {3,{42,59,78}}, {2,{47,52}} } },
      {3, { {5,{13,25,41,56,92}}, {4,{16,33,45,51}}, {3,{22,35,93}} } },
      {4, { {2,{41,93}}, {4,{43,56,69,94}}, {3,{44,51,96}}, {3,{59,63,91}} } },
      {3, { {5,{24,37,59,65,96}}, {4,{29,35,44,67}}, {3,{49,56,94}} } },
      {4, { {3,{12,28,36}}, {2,{14,22}}, {3,{16,42,64}}, {4,{24,38,46,62}} } },
      {4, { {4,{11,27,49,63}}, {2,{12,41}}, {3,{13,42,59}}, {3,{29,53,67}} } },
      {4, { {3,{13,25,74}}, {2,{14,76}}, {4,{16,29,75,88}}, {3,{18,23,89}} } },
      {4, { {4,{35,46,62,84}}, {2,{36,55}}, {3,{42,56,68}}, {3,{58,64,85}} } },
      {3, { {3,{12,24,63}}, {4,{14,49,53,65}}, {5,{15,23,44,59,62}} } },
      {4, { {2,{28,89}}, {3,{29,58,97}}, {4,{57,61,88,92}}, {3,{62,81,99}} } },
      {4, { {4,{31,67,72,98}}, {3,{32,86,91}}, {3,{68,77,82}}, {2,{88,96}} } },
      {5, { {2,{15,46}}, {2,{16,95}}, {3,{45,71,86}}, {2,{65,96}}, {3,{66,75,81}} } },
      {5, { {2,{31,92}}, {2,{32,81}}, {3,{37,89,91}}, {2,{39,47}}, {3,{49,82,97}} } },
      {3, { {5,{11,28,35,42,56}}, {4,{16,22,38,41}}, {3,{36,45,51}} } },
      {4, { {4,{41,65,72,86}}, {3,{46,52,71}}, {2,{56,62}}, {3,{66,75,81}} } },
      {4, { {3,{37,89,91}}, {4,{39,47,76,81}}, {3,{49,77,86}}, {2,{71,97}} } },
      {4, { {4,{37,53,81,98}}, {4,{38,51,77,93}}, {2,{71,97}}, {2,{87,91}} } },
      {3, { {4,{41,56,69,92}}, {4,{49,63,84,91}}, {4,{52,66,83,94}} } },
      {3, { {4,{13,27,89,95}}, {3,{16,23,64}}, {5,{19,24,66,85,97}} } },
      {4, { {2,{12,27}}, {4,{16,32,45,74}}, {2,{17,26}}, {4,{22,35,44,76}} } },
      {4, { {4,{21,38,89,96}}, {4,{29,31,72,98}}, {2,{79,82}}, {2,{86,92}} } },
      {4, { {3,{11,38,82}}, {3,{15,22,81}}, {3,{18,36,55}}, {3,{25,32,56}} } },
      {6, { {2,{12,27}}, {2,{17,68}}, {2,{18,22}}, {2,{28,47}}, {2,{43,67}}, {2,{48,63}} } },
      {4, { {3,{13,38,52}}, {3,{15,37,56}}, {3,{17,32,53}}, {3,{18,36,55}} } },
      {4, { {3,{35,59,66}}, {4,{36,49,55,74}}, {2,{44,75}}, {3,{45,69,76}} } },
      {4, { {3,{16,29,98}}, {3,{18,26,47}}, {3,{21,38,99}}, {3,{27,31,48}} } },
      {5, { {2,{13,24}}, {2,{14,96}}, {2,{16,33}}, {3,{23,35,44}}, {3,{36,45,94}} } },
      {4, { {4,{28,56,62,99}}, {2,{29,88}}, {3,{52,69,98}}, {3,{58,66,89}} } },
      {3, { {4,{41,69,82,97}}, {4,{42,67,88,93}}, {4,{48,63,89,91}} } },
      {3, { {3,{46,58,74}}, {5,{48,55,76,89,91}}, {4,{54,71,85,99}} } },
      {3, { {4,{52,67,73,96}}, {4,{54,63,79,92}}, {4,{57,64,76,99}} } },
      {3, { {4,{14,38,47,66}}, {4,{17,26,54,68}}, {4,{28,36,44,57}} } },
      {5, { {2,{42,68}}, {3,{45,57,84}}, {2,{47,52}}, {3,{48,54,85}}, {2,{58,62}} } },
      {5, { {2,{14,29}}, {3,{16,54,67}}, {2,{19,26}}, {3,{24,46,57}}, {2,{47,66}} } },
      {5, { {3,{51,75,93}}, {3,{53,71,85}}, {2,{73,99}}, {2,{79,83}}, {2,{89,95}} } },
      {4, { {4,{14,38,46,97}}, {2,{15,44}}, {3,{17,25,98}}, {3,{28,36,45}} } },
      {4, { {3,{21,36,43}}, {3,{22,75,81}}, {4,{26,33,72,85}}, {2,{41,83}} } },
      {5, { {3,{24,82,96}}, {2,{25,54}}, {2,{26,45}}, {3,{46,85,92}}, {2,{55,84}} } },
      {3, { {4,{11,29,52,67}}, {5,{17,22,35,51,68}}, {3,{25,38,69}} } },
      {6, { {2,{72,87}}, {2,{73,99}}, {2,{74,93}}, {2,{77,84}}, {2,{79,92}}, {2,{82,94}} } },
      {4, { {3,{17,49,51}}, {3,{19,57,61}}, {3,{41,66,95}}, {3,{45,69,96}} } },
      {4, { {4,{25,54,67,86}}, {3,{26,47,65}}, {3,{46,57,85}}, {2,{55,84}} } },
      {4, { {3,{22,36,93}}, {4,{26,33,72,85}}, {3,{62,75,83}}, {2,{63,92}} } },
      {4, { {4,{37,51,69,78}}, {3,{39,77,92}}, {3,{58,61,99}}, {2,{72,98}} } },
      {4, { {4,{53,66,82,95}}, {4,{55,61,76,93}}, {2,{62,81}}, {2,{71,86}} } },
      {4, { {2,{12,43}}, {4,{13,28,34,59}}, {2,{14,32}}, {4,{23,39,42,58}} } },
      {4, { {4,{12,44,53,76}}, {4,{13,42,56,69}}, {2,{49,64}}, {2,{66,74}} } },
      {5, { {2,{44,96}}, {3,{45,78,94}}, {3,{46,85,92}}, {2,{72,98}}, {2,{75,82}} } },
      {4, { {4,{11,36,72,85}}, {2,{15,21}}, {3,{22,75,81}}, {3,{25,32,76}} } },
      {5, { {3,{15,27,59}}, {3,{19,35,98}}, {2,{25,38}}, {2,{28,97}}, {2,{57,99}} } },
      {5, { {4,{44,69,76,97}}, {2,{47,54}}, {2,{56,74}}, {2,{57,99}}, {2,{59,66}} } },
      {4, { {2,{41,54}}, {4,{44,59,87,91}}, {4,{47,51,84,99}}, {2,{81,94}} } },
      {2, { {6,{41,59,65,73,87,94}}, {6,{47,54,63,75,81,99}} } },
      {2, { {6,{13,27,34,76,82,98}}, {6,{18,26,32,73,87,94}} } },
      {4, { {3,{43,55,81}}, {3,{45,63,84}}, {4,{51,64,73,85}}, {2,{74,83}} } },
      {3, { {5,{13,24,56,62,75}}, {4,{16,32,55,63}}, {3,{22,35,74}} } },
      {3, { {5,{42,57,73,88,95}}, {4,{43,59,77,92}}, {3,{58,85,99}} } },
      {4, { {4,{12,39,45,94}}, {3,{14,32,96}}, {3,{19,36,44}}, {2,{46,95}} } },
      {5, { {3,{42,55,71}}, {2,{43,72}}, {3,{45,61,73}}, {2,{51,67}}, {2,{57,65}} } },
      {5, { {2,{57,89}}, {3,{58,74,87}}, {3,{59,78,81}}, {2,{71,94}}, {2,{84,91}} } },
      {4, { {2,{18,59}}, {4,{19,33,51,68}}, {3,{31,58,62}}, {3,{32,53,69}} } },
      {2, { {6,{11,34,43,56,82,95}}, {6,{16,32,44,51,85,93}} } },
      {6, { {2,{15,27}}, {2,{16,65}}, {2,{17,99}}, {2,{19,26}}, {2,{25,66}}, {2,{29,97}} } },
      {4, { {4,{11,39,57,83}}, {2,{12,81}}, {3,{17,23,82}}, {3,{27,33,59}} } },
      {3, { {5,{24,45,53,76,97}}, {5,{25,43,66,77,94}}, {2,{56,63}} } },
      {4, { {4,{43,66,77,95}}, {4,{44,53,76,97}}, {2,{45,94}}, {2,{56,63}} } },
      {4, { {3,{14,75,81}}, {4,{15,43,71,96}}, {3,{16,84,93}}, {2,{41,83}} } },
      {2, { {6,{13,28,36,42,54,69}}, {6,{16,22,38,44,59,63}} } },
      {4, { {4,{12,24,37,88}}, {4,{14,29,32,67}}, {2,{28,89}}, {2,{69,87}} } },
      {4, { {3,{21,42,83}}, {2,{23,71}}, {4,{41,58,77,82}}, {3,{57,73,88}} } },
      {4, { {4,{24,37,59,96}}, {4,{28,36,77,94}}, {2,{29,58}}, {2,{57,78}} } },
      {4, { {3,{45,77,86}}, {2,{46,95}}, {3,{62,83,96}}, {4,{63,75,87,92}} } },
      {4, { {4,{27,35,56,69}}, {3,{29,41,57}}, {3,{36,45,51}}, {2,{49,65}} } },
      {4, { {2,{21,35}}, {4,{23,38,82,97}}, {4,{25,32,87,93}}, {2,{28,31}} } },
      {4, { {4,{12,26,75,93}}, {4,{15,29,72,96}}, {2,{19,22}}, {2,{23,92}} } },
      {4, { {4,{27,61,78,93}}, {3,{28,76,87}}, {3,{63,71,96}}, {2,{86,97}} } },
      {4, { {4,{14,38,47,76}}, {4,{17,36,58,74}}, {2,{43,57}}, {2,{48,53}} } },
      {4, { {4,{27,56,69,78}}, {4,{29,48,62,77}}, {2,{42,58}}, {2,{52,66}} } },
      {4, { {3,{24,55,86}}, {3,{25,84,92}}, {3,{56,75,88}}, {3,{78,82,95}} } },
      {5, { {3,{37,48,55}}, {2,{38,97}}, {3,{45,69,98}}, {2,{57,99}}, {2,{59,65}} } },
      {6, { {2,{17,68}}, {2,{18,89}}, {2,{19,97}}, {2,{57,99}}, {2,{59,87}}, {2,{67,88}} } },
      {4, { {4,{11,48,52,83}}, {2,{12,61}}, {2,{21,62}}, {4,{22,43,58,81}} } },
      {4, { {2,{17,68}}, {4,{18,52,83,99}}, {4,{19,53,87,92}}, {2,{67,88}} } },
      {4, { {2,{14,28}}, {2,{16,44}}, {4,{18,25,76,83}}, {4,{24,46,73,85}} } },
      {4, { {3,{13,24,85}}, {4,{15,23,44,76}}, {3,{46,78,84}}, {2,{75,88}} } },
      {5, { {2,{42,56}}, {3,{45,62,94}}, {2,{46,57}}, {3,{47,54,95}}, {2,{52,64}} } },
      {4, { {3,{11,89,93}}, {2,{13,25}}, {3,{15,86,99}}, {4,{16,23,81,95}} } },
      {4, { {4,{44,67,88,96}}, {4,{48,66,77,84}}, {2,{76,87}}, {2,{86,97}} } },
      {3, { {4,{12,24,75,83}}, {4,{13,72,84,99}}, {4,{15,23,79,94}} } },
      {4, { {3,{42,55,76}}, {3,{43,75,92}}, {3,{46,53,95}}, {3,{52,73,96}} } },
      {4, { {3,{27,79,83}}, {3,{29,54,67}}, {3,{59,64,78}}, {3,{68,73,87}} } },
      {4, { {3,{21,57,62}}, {3,{22,37,58}}, {2,{23,51}}, {4,{28,32,53,67}} } },
      {4, { {3,{52,73,84}}, {3,{53,64,85}}, {3,{55,63,72}}, {3,{65,74,82}} } },
      {4, { {3,{24,76,89}}, {3,{26,54,69}}, {3,{55,79,84}}, {3,{59,66,75}} } },
      {5, { {2,{16,37}}, {3,{17,85,98}}, {2,{18,26}}, {2,{28,36}}, {3,{38,87,95}} } },
      {3, { {5,{16,37,54,85,98}}, {3,{18,24,56}}, {4,{28,34,87,95}} } },
      {4, { {3,{14,45,68}}, {4,{15,37,48,66}}, {2,{17,24}}, {3,{27,36,64}} } },
      {5, { {3,{27,79,92}}, {2,{28,87}}, {2,{29,88}}, {2,{47,89}}, {3,{49,72,97}} } },
      {4, { {3,{42,59,71}}, {2,{46,52}}, {4,{49,51,66,73}}, {3,{56,63,72}} } },
      {3, { {4,{21,67,88,93}}, {4,{23,57,62,81}}, {4,{53,68,82,97}} } },
      {4, { {3,{11,27,52}}, {4,{16,22,51,94}}, {2,{17,36}}, {3,{24,37,96}} } },
      {4, { {4,{23,37,59,61}}, {4,{29,32,53,67}}, {2,{31,72}}, {2,{62,71}} } },
      {4, { {2,{43,55}}, {2,{44,53}}, {4,{45,61,72,94}}, {4,{54,62,75,91}} } },
      {4, { {4,{11,37,59,83}}, {3,{14,23,81}}, {2,{19,24}}, {3,{29,33,57}} } },
      {4, { {2,{34,75}}, {2,{35,84}}, {4,{37,59,74,88}}, {4,{39,58,77,85}} } },
      {4, { {2,{34,75}}, {4,{35,46,59,88}}, {2,{36,44}}, {4,{48,55,74,89}} } },
      {4, { {2,{14,55}}, {2,{15,44}}, {4,{45,61,72,94}}, {4,{54,62,75,91}} } },
      {4, { {4,{21,59,62,98}}, {2,{22,91}}, {4,{51,68,82,99}}, {2,{88,92}} } },
      {4, { {4,{14,45,51,73}}, {3,{15,46,74}}, {3,{43,56,71}}, {2,{54,76}} } },
      {4, { {2,{14,55}}, {4,{15,29,76,98}}, {4,{19,26,78,94}}, {2,{54,95}} } },
      {4, { {2,{12,73}}, {2,{13,39}}, {4,{19,22,34,85}}, {4,{25,33,72,84}} } },
      {4, { {3,{11,27,88}}, {3,{16,21,94}}, {3,{18,36,87}}, {3,{24,37,96}} } },
      {3, { {5,{14,28,52,89,91}}, {4,{19,24,78,82}}, {3,{51,72,98}} } },
      {4, { {4,{41,58,72,89}}, {3,{49,61,88}}, {3,{52,68,81}}, {2,{78,82}} } },
      {4, { {2,{51,82}}, {4,{52,66,73,85}}, {4,{55,61,76,83}}, {2,{62,81}} } },
      {4, { {3,{11,29,68}}, {4,{18,22,41,69}}, {2,{21,92}}, {3,{48,62,91}} } },
      {3, { {5,{21,42,66,73,85}}, {5,{22,45,61,76,93}}, {2,{83,95}} } },
      {3, { {4,{12,26,43,58}}, {5,{16,23,38,49,52}}, {3,{29,36,48}} } },
      {5, { {2,{56,69}}, {3,{57,78,94}}, {2,{58,66}}, {2,{59,97}}, {3,{68,74,99}} } },
      {3, { {5,{31,43,57,72,99}}, {5,{33,49,51,78,92}}, {2,{58,77}} } },
      {5, { {3,{41,82,98}}, {3,{42,86,91}}, {2,{71,88}}, {2,{76,81}}, {2,{78,96}} } },
      {4, { {4,{13,29,34,77}}, {4,{17,24,33,99}}, {2,{76,97}}, {2,{79,96}} } },
      {5, { {2,{13,34}}, {3,{14,27,76}}, {2,{16,33}}, {2,{17,36}}, {3,{26,37,74}} } },
      {4, { {3,{31,82,97}}, {2,{32,73}}, {4,{33,67,79,91}}, {3,{69,72,87}} } },
      {3, { {4,{11,27,79,93}}, {4,{12,71,86,99}}, {4,{17,23,82,96}} } },
      {4, { {3,{21,36,63}}, {4,{26,33,41,65}}, {3,{35,46,51}}, {2,{55,61}} } },
      {4, { {3,{52,68,83}}, {4,{57,63,78,82}}, {3,{58,74,87}}, {2,{73,84}} } },
      {3, { {4,{14,37,79,95}}, {5,{19,35,54,61,77}}, {3,{51,65,94}} } },
      {4, { {4,{12,37,58,63}}, {3,{18,53,69}}, {3,{19,22,67}}, {2,{27,32}} } },
      {3, { {5,{11,27,52,83,99}}, {4,{19,22,51,87}}, {3,{23,89,92}} } },
      {4, { {3,{24,86,97}}, {3,{26,89,94}}, {3,{41,87,93}}, {3,{43,81,99}} } },
      {3, { {5,{11,29,36,88,93}}, {3,{13,78,81}}, {4,{26,38,73,99}} } },
      {4, { {3,{24,85,93}}, {3,{25,42,66}}, {3,{26,44,62}}, {3,{45,83,94}} } },
      {4, { {3,{42,64,75}}, {3,{45,62,96}}, {3,{53,66,95}}, {3,{55,63,74}} } },
      {3, { {4,{12,29,78,81}}, {5,{18,21,57,62,79}}, {3,{51,67,82}} } },
      {3, { {4,{24,37,49,66}}, {4,{28,36,64,87}}, {4,{29,44,67,88}} } },
      {5, { {2,{17,49}}, {3,{18,57,64}}, {3,{19,54,68}}, {2,{43,59}}, {2,{47,53}} } },
      {4, { {3,{14,28,65}}, {4,{17,25,54,69}}, {3,{18,57,64}}, {2,{29,68}} } },
      {4, { {3,{42,68,77}}, {4,{47,54,62,79}}, {3,{49,52,78}}, {2,{58,64}} } },
      {3, { {4,{34,57,86,98}}, {4,{36,43,82,94}}, {4,{42,58,87,93}} } },
      {5, { {2,{34,96}}, {2,{36,84}}, {3,{37,88,94}}, {3,{38,86,99}}, {2,{39,97}} } },
      {5, { {2,{21,82}}, {2,{22,91}}, {3,{31,83,95}}, {2,{32,81}}, {3,{33,85,92}} } },
      {3, { {5,{26,38,73,87,94}}, {4,{27,36,78,93}}, {3,{34,86,97}} } },
      {5, { {3,{11,83,95}}, {2,{13,36}}, {3,{16,32,91}}, {2,{33,82}}, {2,{85,92}} } },
      {4, { {2,{17,69}}, {4,{19,36,57,74}}, {3,{34,59,66}}, {3,{54,67,76}} } },
      {4, { {2,{17,69}}, {4,{19,57,66,75}}, {3,{54,67,76}}, {3,{59,65,74}} } },
      {4, { {4,{15,29,57,62}}, {3,{16,55,67}}, {2,{19,26}}, {3,{25,52,66}} } },
      {3, { {3,{32,83,95}}, {5,{33,41,69,75,92}}, {4,{49,63,71,85}} } },
      {4, { {4,{25,32,76,81}}, {4,{26,31,85,92}}, {2,{72,86}}, {2,{82,96}} } },
      {4, { {4,{43,67,88,92}}, {2,{48,53}}, {3,{58,82,99}}, {3,{59,63,97}} } },
      {3, { {5,{45,53,67,88,92}}, {4,{48,55,82,99}}, {3,{59,63,97}} } },
      {4, { {2,{21,42}}, {3,{22,71,89}}, {4,{41,79,85,93}}, {3,{43,82,95}} } },
      {4, { {3,{35,86,98}}, {4,{36,42,73,95}}, {3,{43,88,92}}, {2,{76,83}} } },
      {4, { {3,{24,48,56}}, {4,{26,41,58,74}}, {2,{44,61}}, {3,{51,64,76}} } },
      {3, { {4,{41,64,72,99}}, {4,{44,52,79,81}}, {4,{54,62,89,91}} } },
      {3, { {4,{41,59,87,94}}, {5,{47,56,61,84,99}}, {3,{51,64,96}} } },
      {4, { {3,{14,55,69}}, {3,{15,58,64}}, {3,{59,71,98}}, {3,{68,79,91}} } },
      {5, { {2,{11,53}}, {3,{12,44,51}}, {3,{13,59,62}}, {2,{42,64}}, {2,{54,69}} } },
      {4, { {4,{12,25,56,61}}, {2,{13,52}}, {3,{16,23,65}}, {3,{21,53,66}} } },
      {4, { {4,{16,39,45,58}}, {4,{18,35,46,69}}, {2,{48,56}}, {2,{49,65}} } },
      {4, { {4,{43,68,76,82}}, {4,{48,52,73,86}}, {2,{56,62}}, {2,{58,66}} } },
      {4, { {4,{42,58,87,96}}, {2,{47,89}}, {4,{49,52,76,98}}, {2,{79,86}} } },
      {4, { {2,{32,73}}, {2,{33,52}}, {4,{44,53,75,81}}, {4,{45,51,72,84}} } },
      {4, { {3,{13,25,41}}, {3,{14,21,43}}, {3,{15,27,69}}, {3,{19,24,67}} } },
      {4, { {3,{11,42,69}}, {3,{12,48,61}}, {3,{13,28,49}}, {3,{19,23,68}} } },
      {4, { {2,{37,89}}, {3,{38,46,57}}, {4,{39,56,67,88}}, {3,{48,66,87}} } },
      {4, { {3,{24,75,89}}, {3,{25,38,79}}, {3,{29,34,88}}, {3,{35,78,84}} } },
      {4, { {3,{22,53,64}}, {3,{23,72,86}}, {3,{54,61,83}}, {3,{62,76,81}} } },
      {4, { {4,{13,25,42,76}}, {4,{16,32,43,75}}, {2,{22,37}}, {2,{27,35}} } },
      {3, { {4,{18,49,55,84}}, {5,{19,44,58,87,96}}, {3,{56,85,97}} } },
      {4, { {4,{12,35,73,88}}, {4,{15,33,77,82}}, {2,{75,87}}, {2,{78,85}} } },
      {3, { {4,{15,29,73,86}}, {4,{17,26,89,95}}, {4,{19,75,83,97}} } },
      {4, { {3,{35,76,91}}, {3,{36,83,95}}, {3,{44,86,93}}, {3,{46,71,94}} } },
      {3, { {4,{42,55,86,91}}, {4,{45,51,73,96}}, {4,{46,53,75,82}} } },
      {4, { {3,{41,55,86}}, {3,{42,74,91}}, {3,{46,52,85}}, {3,{51,72,94}} } },
      // size 13 (850 types, but again this may not be the whole story)
      /*
      {5, { {2,{24,66}}, {3,{26,44,52}}, {3,{35,42,56}}, {3,{36,45,84}}, {2,{64,86}} } },
      {4, { {4,{13,35,44,57}}, {3,{14,27,33}}, {4,{24,37,48,55}}, {2,{28,47}} } },
      {5, { {4,{24,37,48,55}}, {3,{27,35,74}}, {2,{28,47}}, {2,{44,75}}, {2,{45,57}} } },
      {4, { {3,{31,43,55}}, {4,{33,51,75,92}}, {2,{45,57}}, {4,{47,52,73,95}} } },
      {4, { {3,{22,36,83}}, {3,{26,37,98}}, {4,{28,33,59,82}}, {3,{39,58,97}} } },
      {4, { {3,{22,36,83}}, {3,{26,39,48}}, {4,{27,33,79,82}}, {3,{28,49,77}} } },
      {6, { {2,{12,39}}, {2,{17,22}}, {2,{19,97}}, {2,{27,98}}, {3,{28,32,59}}, {2,{58,99}} } },
      {4, { {3,{14,29,45}}, {3,{19,34,88}}, {3,{25,44,76}}, {4,{26,38,74,89}} } },
      {5, { {3,{11,25,74}}, {3,{14,21,39}}, {3,{22,34,45}}, {2,{29,32}}, {2,{44,75}} } },
      {4, { {4,{42,54,79,85}}, {4,{45,59,74,97}}, {2,{47,52}}, {3,{57,89,95}} } },
      {4, { {3,{11,29,67}}, {4,{16,21,52,65}}, {4,{17,25,56,69}}, {2,{57,62}} } },
      {4, { {3,{15,37,94}}, {3,{17,68,99}}, {3,{18,24,95}}, {4,{28,34,69,97}} } },
      {4, { {4,{15,37,68,99}}, {3,{18,24,95}}, {4,{28,34,69,97}}, {2,{35,94}} } },
      {5, { {2,{12,53}}, {4,{13,42,89,91}}, {2,{41,54}}, {2,{44,52}}, {3,{51,83,99}} } },
      {5, { {3,{15,39,68}}, {2,{16,45}}, {3,{18,26,99}}, {3,{29,35,46}}, {2,{69,98}} } },
      {6, { {3,{24,46,57}}, {2,{26,64}}, {2,{37,69}}, {2,{39,47}}, {2,{49,66}}, {2,{54,67}} } },
      {6, { {3,{24,46,57}}, {2,{26,64}}, {2,{42,69}}, {2,{47,62}}, {2,{49,66}}, {2,{54,67}} } },
      {4, { {4,{14,21,38,47}}, {3,{17,24,99}}, {2,{26,31}}, {4,{29,36,48,97}} } },
      {6, { {3,{24,46,57}}, {2,{26,64}}, {2,{42,56}}, {2,{47,62}}, {2,{52,66}}, {2,{54,67}} } },
      {6, { {2,{14,39}}, {2,{16,94}}, {3,{18,35,96}}, {2,{19,98}}, {2,{34,95}}, {2,{38,99}} } },
      {6, { {2,{12,38}}, {2,{14,39}}, {2,{16,94}}, {3,{18,35,96}}, {2,{19,32}}, {2,{34,95}} } },
      {4, { {4,{17,24,49,56}}, {2,{18,57}}, {4,{19,26,48,55}}, {3,{25,44,58}} } },
      {5, { {3,{51,88,92}}, {4,{52,68,71,99}}, {2,{58,61}}, {2,{59,98}}, {2,{78,81}} } },
      {5, { {3,{17,58,63}}, {3,{18,47,99}}, {3,{46,53,67}}, {2,{49,56}}, {2,{59,98}} } },
      {4, { {2,{22,35}}, {5,{25,31,48,54,62}}, {4,{32,45,51,68}}, {2,{34,55}} } },
      {4, { {4,{51,65,84,92}}, {3,{53,86,91}}, {4,{54,62,83,96}}, {2,{66,85}} } },
      {4, { {4,{22,53,65,84}}, {3,{23,86,92}}, {4,{54,62,83,96}}, {2,{66,85}} } },
      {5, { {4,{57,63,71,89}}, {2,{58,77}}, {2,{59,88}}, {3,{61,74,83}}, {2,{78,84}} } },
      {6, { {2,{14,95}}, {2,{15,64}}, {2,{25,96}}, {2,{26,75}}, {3,{34,65,76}}, {2,{36,94}} } },
      {6, { {2,{14,95}}, {2,{15,64}}, {3,{34,65,76}}, {2,{36,94}}, {2,{75,97}}, {2,{77,96}} } },
      {4, { {3,{38,79,82}}, {4,{39,43,68,72}}, {4,{49,61,73,88}}, {2,{62,81}} } },
      {4, { {5,{12,35,44,53,91}}, {2,{14,46}}, {4,{16,31,52,93}}, {2,{36,45}} } },
      {5, { {2,{15,86}}, {3,{16,29,77}}, {2,{17,78}}, {4,{18,24,79,85}}, {2,{26,84}} } },
      {4, { {2,{15,86}}, {4,{16,29,67,78}}, {5,{17,24,68,79,85}}, {2,{26,84}} } },
      {5, { {2,{31,53}}, {4,{33,46,61,74}}, {2,{34,76}}, {3,{36,43,54}}, {2,{51,64}} } },
      {5, { {3,{12,28,35}}, {3,{15,29,97}}, {3,{17,22,98}}, {2,{25,39}}, {2,{38,99}} } },
      {4, { {5,{11,38,44,59,62}}, {4,{19,31,52,67}}, {2,{37,48}}, {2,{47,64}} } },
      {5, { {3,{41,75,92}}, {3,{42,71,98}}, {3,{67,78,85}}, {2,{68,97}}, {2,{87,95}} } },
      {5, { {3,{37,68,99}}, {2,{39,57}}, {3,{59,87,96}}, {3,{67,81,98}}, {2,{86,91}} } },
      {4, { {4,{11,26,44,62}}, {3,{14,29,66}}, {4,{19,22,51,64}}, {2,{41,54}} } },
      {4, { {4,{11,45,76,92}}, {4,{12,46,73,84}}, {2,{13,71}}, {3,{44,82,95}} } },
      {4, { {3,{11,24,96}}, {4,{16,28,74,97}}, {3,{17,21,99}}, {3,{29,78,94}} } },
      {5, { {2,{25,39}}, {4,{28,35,87,92}}, {2,{29,98}}, {3,{37,84,99}}, {2,{82,94}} } },
      {5, { {3,{27,89,96}}, {3,{29,77,81}}, {2,{37,78}}, {2,{38,97}}, {3,{71,86,98}} } },
      {5, { {3,{27,49,98}}, {3,{29,77,81}}, {2,{37,78}}, {2,{38,97}}, {3,{48,71,89}} } },
      {5, { {3,{51,68,73}}, {3,{52,71,86}}, {2,{55,62}}, {2,{58,65}}, {3,{63,76,82}} } },
      {4, { {3,{28,44,59}}, {4,{29,54,76,98}}, {4,{48,66,74,99}}, {2,{56,69}} } },
      {4, { {2,{11,28}}, {4,{18,24,32,89}}, {4,{19,21,34,87}}, {3,{22,37,88}} } },
      {4, { {4,{24,41,69,85}}, {3,{25,53,64}}, {3,{45,61,84}}, {3,{49,55,63}} } },
      {4, { {4,{42,54,75,83}}, {4,{45,53,68,82}}, {3,{52,65,74}}, {2,{58,62}} } },
      {5, { {3,{18,69,87}}, {3,{19,48,65}}, {2,{27,88}}, {3,{28,55,67}}, {2,{45,58}} } },
      {5, { {4,{64,76,81,97}}, {3,{65,82,94}}, {2,{66,75}}, {2,{71,85}}, {2,{87,92}} } },
      {5, { {3,{37,42,69}}, {2,{39,87}}, {2,{43,82}}, {3,{47,53,89}}, {3,{59,62,83}} } },
      {5, { {3,{41,56,95}}, {2,{44,51}}, {3,{45,84,97}}, {3,{54,76,87}}, {2,{77,96}} } },
      {5, { {3,{13,26,74}}, {3,{14,31,63}}, {3,{21,34,66}}, {2,{23,61}}, {2,{64,76}} } },
      {5, { {3,{24,56,67}}, {2,{25,44}}, {4,{26,45,57,79}}, {2,{49,64}}, {2,{69,77}} } },
      {5, { {3,{11,24,95}}, {2,{12,41}}, {3,{15,32,96}}, {2,{21,42}}, {3,{22,36,94}} } },
      {4, { {3,{35,86,97}}, {2,{36,65}}, {4,{52,66,81,95}}, {4,{55,61,87,92}} } },
      {4, { {3,{21,34,52}}, {2,{23,61}}, {4,{24,39,51,65}}, {4,{29,32,55,63}} } },
      {4, { {3,{52,64,86}}, {3,{56,62,75}}, {4,{65,78,84,92}}, {3,{76,82,98}} } },
      {5, { {3,{52,64,86}}, {3,{56,62,75}}, {3,{65,72,84}}, {2,{76,88}}, {2,{78,82}} } },
      {4, { {3,{52,64,86}}, {2,{55,62}}, {4,{56,75,82,98}}, {4,{65,78,84,92}} } },
      {5, { {3,{13,39,48}}, {2,{18,99}}, {2,{19,23}}, {4,{27,33,49,98}}, {2,{29,97}} } },
      {5, { {4,{13,25,54,62}}, {2,{14,56}}, {2,{16,29}}, {2,{19,23}}, {3,{26,52,65}} } },
      {5, { {2,{13,39}}, {3,{16,21,83}}, {3,{19,34,56}}, {3,{26,31,54}}, {2,{33,81}} } },
      {6, { {3,{13,29,35}}, {2,{17,33}}, {2,{19,97}}, {2,{25,38}}, {2,{28,99}}, {2,{37,98}} } },
      {4, { {3,{34,56,95}}, {3,{35,42,64}}, {4,{45,62,74,93}}, {3,{54,73,96}} } },
      {5, { {2,{18,49}}, {3,{19,78,96}}, {4,{48,56,84,99}}, {2,{54,76}}, {2,{74,88}} } },
      {6, { {2,{17,79}}, {2,{19,47}}, {3,{28,49,97}}, {2,{29,78}}, {2,{71,98}}, {2,{77,91}} } },
      {6, { {2,{17,79}}, {2,{19,47}}, {3,{28,49,97}}, {2,{29,78}}, {2,{37,98}}, {2,{38,77}} } },
      {5, { {4,{22,78,86,93}}, {3,{23,72,94}}, {2,{74,88}}, {2,{82,96}}, {2,{84,92}} } },
      {5, { {2,{37,89}}, {3,{39,58,77}}, {2,{59,88}}, {3,{72,87,96}}, {3,{78,86,92}} } },
      {5, { {2,{11,28}}, {2,{17,21}}, {4,{18,47,79,82}}, {2,{27,49}}, {3,{29,72,88}} } },
      {3, { {4,{17,59,78,96}}, {5,{18,46,57,84,99}}, {4,{49,54,76,88}} } },
      {3, { {5,{12,25,37,63,71}}, {5,{17,23,35,52,69}}, {3,{59,61,72}} } },
      {5, { {4,{24,42,55,86}}, {3,{26,64,85}}, {2,{44,62}}, {2,{52,67}}, {2,{57,65}} } },
      {6, { {2,{12,53}}, {3,{13,42,66}}, {2,{41,58}}, {2,{46,51}}, {2,{48,52}}, {2,{56,63}} } },
      {6, { {2,{12,53}}, {2,{13,27}}, {2,{17,32}}, {3,{23,39,52}}, {2,{29,87}}, {2,{37,89}} } },
      {4, { {4,{28,65,79,96}}, {4,{29,54,75,98}}, {3,{51,66,94}}, {2,{55,61}} } },
      {4, { {2,{17,58}}, {4,{18,36,85,97}}, {3,{26,38,57}}, {4,{27,35,88,96}} } },
      {5, { {2,{41,69}}, {3,{42,59,64}}, {3,{44,52,68}}, {3,{46,58,61}}, {2,{49,56}} } },
      {5, { {3,{34,45,59}}, {3,{35,46,74}}, {3,{49,54,86}}, {2,{76,87}}, {2,{77,84}} } },
      {5, { {4,{17,48,56,75}}, {2,{18,87}}, {2,{45,68}}, {3,{57,66,88}}, {2,{65,76}} } },
      {6, { {2,{12,53}}, {3,{13,61,92}}, {2,{21,63}}, {2,{23,81}}, {2,{51,83}}, {2,{52,91}} } },
      {6, { {3,{13,37,79}}, {2,{18,33}}, {2,{19,98}}, {2,{28,99}}, {2,{29,78}}, {2,{38,77}} } },
      {4, { {5,{15,39,44,58,87}}, {4,{17,35,69,88}}, {2,{49,64}}, {2,{54,68}} } },
      {5, { {2,{27,68}}, {3,{28,89,91}}, {2,{29,87}}, {3,{67,78,84}}, {3,{74,81,98}} } },
      {5, { {3,{24,45,86}}, {3,{26,64,85}}, {2,{34,66}}, {3,{36,44,75}}, {2,{65,76}} } },
      {5, { {2,{17,58}}, {4,{18,47,79,82}}, {2,{27,49}}, {2,{28,57}}, {3,{29,72,88}} } },
      {5, { {4,{34,52,65,71}}, {2,{35,44}}, {3,{41,54,72}}, {2,{45,68}}, {2,{48,61}} } },
      {5, { {4,{44,58,79,82}}, {2,{49,64}}, {2,{54,68}}, {2,{67,88}}, {3,{69,72,87}} } },
      {6, { {2,{17,79}}, {2,{18,87}}, {3,{19,88,93}}, {2,{67,99}}, {2,{69,77}}, {2,{83,97}} } },
      {6, { {2,{11,28}}, {2,{18,49}}, {3,{19,21,87}}, {2,{27,68}}, {2,{48,89}}, {2,{67,88}} } },
      {5, { {3,{65,71,86}}, {4,{66,75,82,97}}, {2,{77,91}}, {2,{81,94}}, {2,{84,92}} } },
      {4, { {3,{23,35,64}}, {4,{24,33,56,95}}, {3,{54,73,96}}, {3,{65,74,93}} } },
      {3, { {3,{14,86,92}}, {5,{16,48,65,77,94}}, {5,{47,66,75,82,98}} } },
      {4, { {3,{24,75,88}}, {3,{25,54,61}}, {4,{51,66,84,98}}, {3,{65,78,96}} } },
      {5, { {4,{44,58,76,89}}, {2,{45,84}}, {2,{46,75}}, {3,{59,85,98}}, {2,{78,95}} } },
      {4, { {4,{17,48,52,65}}, {3,{18,45,69}}, {3,{19,44,57}}, {3,{49,54,62}} } },
      {5, { {3,{43,58,97}}, {2,{47,63}}, {2,{53,69}}, {3,{59,85,98}}, {3,{67,89,95}} } },
      {5, { {3,{24,45,86}}, {3,{26,82,94}}, {2,{34,55}}, {2,{35,44}}, {3,{54,85,92}} } },
      {5, { {2,{34,55}}, {4,{35,43,64,82}}, {2,{42,54}}, {2,{44,62}}, {3,{45,52,83}} } },
      {6, { {2,{24,46}}, {2,{25,64}}, {3,{26,43,65}}, {2,{42,83}}, {2,{44,62}}, {2,{63,82}} } },
      {4, { {4,{13,38,41,59}}, {3,{19,33,68}}, {3,{48,63,91}}, {3,{51,69,93}} } },
      {4, { {5,{21,34,52,83,96}}, {4,{24,33,71,86}}, {2,{51,72}}, {2,{76,92}} } },
      {4, { {5,{14,29,33,45,57}}, {3,{17,24,49}}, {2,{23,34}}, {3,{35,47,54}} } },
      {4, { {4,{15,39,48,87}}, {4,{17,35,83,99}}, {2,{18,49}}, {3,{19,88,93}} } },
      {5, { {2,{42,83}}, {2,{43,57}}, {2,{44,62}}, {3,{47,64,99}}, {4,{53,69,82,97}} } },
      {4, { {4,{24,43,65,86}}, {5,{25,31,56,63,84}}, {2,{33,41}}, {2,{46,51}} } },
      {5, { {2,{14,96}}, {3,{16,65,74}}, {3,{54,77,86}}, {2,{56,94}}, {3,{66,75,87}} } },
      {6, { {2,{28,99}}, {2,{29,78}}, {2,{54,76}}, {3,{56,84,98}}, {2,{74,88}}, {2,{79,96}} } },
      {6, { {2,{11,28}}, {3,{13,41,72}}, {2,{16,23}}, {2,{18,26}}, {2,{21,42}}, {2,{22,73}} } },
      {6, { {3,{47,76,88}}, {2,{48,97}}, {2,{67,99}}, {2,{69,77}}, {2,{79,96}}, {2,{86,98}} } },
      {6, { {2,{24,46}}, {2,{25,64}}, {2,{26,45}}, {2,{34,66}}, {3,{36,44,75}}, {2,{65,76}} } },
      {6, { {2,{21,63}}, {2,{23,34}}, {3,{24,32,61}}, {2,{33,52}}, {2,{53,69}}, {2,{59,62}} } },
      {5, { {3,{11,72,98}}, {4,{12,27,78,83}}, {2,{17,21}}, {2,{23,81}}, {2,{88,91}} } },
      {5, { {2,{28,99}}, {3,{29,68,77}}, {3,{67,83,98}}, {2,{76,87}}, {3,{79,86,93}} } },
      {4, { {2,{17,79}}, {4,{19,21,43,67}}, {3,{23,61,82}}, {4,{49,62,77,83}} } },
      {6, { {2,{21,63}}, {3,{23,71,86}}, {2,{61,93}}, {2,{76,87}}, {2,{77,91}}, {2,{83,97}} } },
      {5, { {3,{13,29,34}}, {2,{18,33}}, {4,{19,24,31,98}}, {2,{21,38}}, {2,{28,99}} } },
      {5, { {2,{21,38}}, {4,{25,31,42,64}}, {2,{26,45}}, {2,{28,36}}, {3,{34,46,62}} } },
      {6, { {2,{12,53}}, {2,{13,62}}, {2,{21,63}}, {2,{23,34}}, {3,{24,32,61}}, {2,{33,52}} } },
      {5, { {2,{12,34}}, {4,{14,39,88,93}}, {2,{17,32}}, {2,{18,87}}, {3,{37,83,99}} } },
      {4, { {4,{27,69,72,88}}, {3,{28,54,67}}, {4,{44,58,79,82}}, {2,{49,64}} } },
      {6, { {2,{61,93}}, {2,{62,71}}, {3,{63,72,86}}, {2,{76,87}}, {2,{77,91}}, {2,{83,97}} } },
      {6, { {2,{21,63}}, {2,{22,61}}, {2,{23,92}}, {3,{51,69,93}}, {2,{52,91}}, {2,{59,62}} } },
      {4, { {2,{25,86}}, {4,{26,48,55,79}}, {4,{46,67,78,85}}, {3,{59,65,77}} } },
      {6, { {2,{34,55}}, {3,{35,46,74}}, {2,{43,66}}, {2,{45,53}}, {2,{54,76}}, {2,{56,63}} } },
      {6, { {2,{24,85}}, {2,{25,64}}, {3,{42,55,84}}, {2,{44,62}}, {2,{52,67}}, {2,{57,65}} } },
      {5, { {2,{13,39}}, {3,{15,54,96}}, {2,{16,23}}, {2,{19,35}}, {4,{26,33,55,94}} } },
      {4, { {4,{45,67,76,88}}, {3,{48,86,99}}, {4,{49,55,77,96}}, {2,{57,65}} } },
      {4, { {3,{21,43,92}}, {3,{22,41,65}}, {4,{31,55,62,93}}, {3,{33,45,51}} } },
      {4, { {3,{13,38,49}}, {5,{19,33,58,61,72}}, {3,{42,69,71}}, {2,{48,52}} } },
      {6, { {2,{27,49}}, {3,{28,47,52}}, {2,{29,78}}, {2,{42,54}}, {2,{44,59}}, {2,{58,79}} } },
      {3, { {5,{34,45,52,69,71}}, {4,{35,49,54,97}}, {4,{57,61,72,99}} } },
      {5, { {3,{13,25,37}}, {2,{16,23}}, {2,{17,79}}, {3,{19,26,78}}, {3,{28,35,77}} } },
      {6, { {2,{31,72}}, {2,{32,93}}, {3,{33,71,89}}, {2,{79,82}}, {2,{83,96}}, {2,{86,92}} } },
      {5, { {3,{51,82,97}}, {3,{52,86,93}}, {2,{53,71}}, {3,{73,87,96}}, {2,{77,91}} } },
      {4, { {3,{11,26,85}}, {4,{15,27,88,96}}, {4,{18,21,76,97}}, {2,{78,86}} } },
      {3, { {4,{37,58,73,99}}, {4,{38,47,51,66}}, {5,{46,59,61,77,93}} } },
      {4, { {3,{35,66,74}}, {4,{36,45,69,97}}, {3,{47,54,75}}, {3,{57,64,99}} } },
      {4, { {5,{15,21,39,74,97}}, {2,{17,98}}, {3,{18,25,31}}, {3,{38,79,94}} } },
      {6, { {2,{15,46}}, {2,{16,85}}, {2,{41,66}}, {2,{45,51}}, {3,{54,61,86}}, {2,{55,84}} } },
      {4, { {4,{42,57,81,98}}, {3,{48,63,82}}, {2,{52,83}}, {4,{53,68,87,91}} } },
      {5, { {4,{15,28,76,81}}, {2,{18,35}}, {3,{24,38,85}}, {2,{26,74}}, {2,{71,84}} } },
      {4, { {5,{12,28,35,76,81}}, {4,{18,24,32,85}}, {2,{26,74}}, {2,{71,84}} } },
      {4, { {3,{15,43,66}}, {2,{16,85}}, {4,{45,51,73,84}}, {4,{54,63,71,86}} } },
      {4, { {3,{15,43,66}}, {3,{16,45,84}}, {3,{44,51,73}}, {4,{54,63,71,86}} } },
      {4, { {4,{16,22,37,98}}, {5,{18,26,32,89,91}}, {2,{39,87}}, {2,{81,97}} } },
      {5, { {3,{12,37,98}}, {2,{18,22}}, {4,{28,32,89,91}}, {2,{39,87}}, {2,{81,97}} } },
      {4, { {4,{27,44,68,86}}, {2,{28,77}}, {4,{47,56,79,84}}, {3,{59,66,78}} } },
      {4, { {2,{45,69}}, {4,{47,54,71,85}}, {2,{49,97}}, {5,{57,65,74,81,99}} } },
      {4, { {4,{13,38,79,86}}, {3,{15,33,76}}, {4,{17,36,75,88}}, {2,{19,77}} } },
      {5, { {3,{37,49,55}}, {3,{39,85,97}}, {2,{45,69}}, {2,{57,65}}, {3,{67,89,95}} } },
      {6, { {2,{27,48}}, {2,{28,77}}, {2,{44,68}}, {3,{47,54,79}}, {2,{59,64}}, {2,{69,78}} } },
      {4, { {4,{13,29,35,54}}, {4,{19,24,33,66}}, {3,{36,64,85}}, {2,{55,84}} } },
      {4, { {3,{11,27,35}}, {3,{17,22,81}}, {3,{25,39,98}}, {4,{28,31,82,99}} } },
      {3, { {4,{28,44,59,62}}, {4,{29,48,77,93}}, {5,{47,52,64,73,99}} } },
      {5, { {2,{43,59}}, {2,{49,63}}, {4,{53,68,77,94}}, {2,{57,79}}, {3,{69,74,98}} } },
      {4, { {4,{38,59,87,91}}, {3,{39,52,68}}, {4,{42,57,81,98}}, {2,{48,62}} } },
      {4, { {3,{11,78,83}}, {4,{13,29,75,88}}, {2,{19,31}}, {4,{23,39,71,85}} } },
      {5, { {4,{11,36,53,65}}, {2,{16,21}}, {3,{23,41,55}}, {2,{26,33}}, {2,{45,61}} } },
      {5, { {4,{11,36,53,65}}, {2,{16,39}}, {3,{19,33,41}}, {2,{43,55}}, {2,{45,61}} } },
      {5, { {3,{42,56,94}}, {3,{44,75,93}}, {3,{45,52,86}}, {2,{73,85}}, {2,{83,96}} } },
      {5, { {3,{54,68,75}}, {3,{56,83,94}}, {3,{58,65,86}}, {2,{73,85}}, {2,{74,93}} } },
      {5, { {2,{42,54}}, {3,{44,75,93}}, {3,{45,52,86}}, {3,{56,83,94}}, {2,{73,85}} } },
      {4, { {2,{11,82}}, {5,{12,53,64,71,98}}, {4,{54,62,88,93}}, {2,{78,81}} } },
      {5, { {2,{14,38}}, {4,{18,25,69,84}}, {3,{29,36,85}}, {2,{34,86}}, {2,{39,68}} } },
      {4, { {3,{13,27,69}}, {2,{15,46}}, {4,{16,23,49,64}}, {4,{19,24,45,67}} } },
      {5, { {2,{15,46}}, {2,{16,37}}, {4,{17,24,45,69}}, {3,{26,49,64}}, {2,{27,36}} } },
      {6, { {2,{11,82}}, {2,{12,28}}, {2,{18,89}}, {2,{19,31}}, {3,{22,39,88}}, {2,{32,81}} } },
      {5, { {4,{14,47,55,96}}, {2,{16,54}}, {2,{35,56}}, {3,{36,45,94}}, {2,{44,57}} } },
      {4, { {4,{12,29,56,68}}, {2,{14,65}}, {3,{15,22,66}}, {4,{18,26,59,64}} } },
      {4, { {4,{13,29,68,87}}, {3,{17,33,88}}, {2,{19,77}}, {4,{23,38,67,79}} } },
      {5, { {3,{21,76,93}}, {2,{23,61}}, {2,{31,62}}, {3,{32,71,86}}, {3,{63,82,96}} } },
      {5, { {3,{12,35,46}}, {2,{13,42}}, {3,{16,23,44}}, {3,{24,33,45}}, {2,{32,43}} } },
      {5, { {3,{12,35,46}}, {3,{16,23,42}}, {3,{24,33,45}}, {2,{26,44}}, {2,{32,43}} } },
      {4, { {5,{31,43,65,74,92}}, {2,{32,81}}, {4,{45,61,73,84}}, {2,{82,94}} } },
      {4, { {3,{45,59,78}}, {4,{47,55,79,83}}, {3,{48,73,97}}, {3,{57,89,93}} } },
      {4, { {4,{14,38,45,89}}, {2,{19,24}}, {3,{28,35,94}}, {4,{29,44,88,95}} } },
      {5, { {2,{21,73}}, {2,{22,91}}, {4,{23,57,89,92}}, {3,{59,77,83}}, {2,{71,97}} } },
      {4, { {4,{13,27,48,92}}, {4,{18,21,47,83}}, {3,{22,61,93}}, {2,{63,81}} } },
      {5, { {2,{21,93}}, {3,{23,38,59}}, {4,{29,33,51,68}}, {2,{53,91}}, {2,{58,63}} } },
      {5, { {3,{13,25,64}}, {4,{14,31,65,83}}, {2,{21,35}}, {2,{23,61}}, {2,{63,81}} } },
      {4, { {3,{17,41,58}}, {4,{18,32,67,71}}, {3,{31,47,62}}, {3,{51,68,72}} } },
      {4, { {3,{44,57,99}}, {2,{48,54}}, {4,{49,78,86,91}}, {4,{58,76,81,97}} } },
      {5, { {2,{11,62}}, {4,{12,38,87,91}}, {2,{37,78}}, {3,{61,77,82}}, {2,{71,98}} } },
      {4, { {3,{21,42,57}}, {4,{22,37,43,59}}, {3,{23,51,69}}, {3,{39,47,63}} } },
      {5, { {3,{46,58,89}}, {2,{49,66}}, {2,{56,68}}, {3,{67,88,96}}, {3,{69,86,97}} } },
      {3, { {5,{21,53,64,72,95}}, {5,{23,55,62,76,91}}, {3,{56,63,74}} } },
      {4, { {3,{12,28,35}}, {4,{15,22,78,94}}, {3,{24,38,65}}, {3,{64,75,98}} } },
      {3, { {5,{44,58,71,85,97}}, {5,{48,66,74,87,91}}, {3,{55,68,86}} } },
      {5, { {4,{11,29,46,57}}, {2,{17,58}}, {2,{18,21}}, {3,{28,56,69}}, {2,{49,66}} } },
      {5, { {2,{11,52}}, {2,{12,38}}, {4,{18,25,33,51}}, {3,{22,35,53}}, {2,{28,32}} } },
      {6, { {2,{11,34}}, {2,{14,55}}, {2,{15,56}}, {2,{16,28}}, {2,{18,21}}, {3,{26,31,54}} } },
      {3, { {5,{18,57,61,79,92}}, {5,{19,31,52,68,77}}, {3,{32,71,97}} } },
      {5, { {3,{34,86,91}}, {3,{36,53,64}}, {3,{56,63,74}}, {2,{71,94}}, {2,{76,81}} } },
      {4, { {4,{15,28,49,64}}, {3,{17,35,68}}, {2,{19,47}}, {4,{25,38,44,67}} } },
      {5, { {2,{34,65}}, {4,{35,49,56,62}}, {3,{36,44,59}}, {2,{42,68}}, {2,{48,64}} } },
      {3, { {5,{15,38,54,86,97}}, {5,{17,34,56,68,95}}, {3,{67,88,96}} } },
      {4, { {4,{18,42,59,64}}, {2,{19,78}}, {3,{27,44,68}}, {4,{28,47,52,79}} } },
      {5, { {2,{42,93}}, {2,{43,69}}, {4,{49,51,84,92}}, {2,{59,61}}, {3,{63,81,94}} } },
      {5, { {4,{24,45,67,96}}, {2,{26,74}}, {2,{44,75}}, {3,{47,66,94}}, {2,{65,76}} } },
      {5, { {2,{21,92}}, {3,{22,83,99}}, {2,{23,81}}, {3,{44,89,96}}, {3,{46,84,91}} } },
      {4, { {3,{24,89,95}}, {4,{25,56,77,84}}, {4,{55,76,87,92}}, {2,{82,99}} } },
      {4, { {4,{17,58,79,94}}, {4,{18,59,76,87}}, {3,{73,86,97}}, {2,{74,93}} } },
      {6, { {2,{14,56}}, {2,{16,64}}, {2,{25,66}}, {3,{26,38,54}}, {2,{28,35}}, {2,{34,65}} } },
      {5, { {4,{14,38,57,65}}, {2,{16,64}}, {2,{17,26}}, {2,{25,66}}, {3,{27,35,58}} } },
      {5, { {3,{16,27,49}}, {3,{18,26,97}}, {3,{19,57,98}}, {2,{46,59}}, {2,{47,56}} } },
      {4, { {2,{11,28}}, {4,{18,32,49,61}}, {3,{21,39,82}}, {4,{29,42,68,81}} } },
      {4, { {4,{15,38,54,67}}, {3,{17,25,69}}, {3,{29,35,98}}, {3,{58,64,99}} } },
      {5, { {3,{17,42,59}}, {4,{18,47,51,93}}, {2,{19,58}}, {2,{43,92}}, {2,{52,91}} } },
      {3, { {5,{12,25,74,83,99}}, {5,{14,21,75,89,92}}, {3,{23,71,84}} } },
      {4, { {4,{11,74,89,92}}, {4,{12,24,83,99}}, {2,{14,21}}, {3,{23,71,84}} } },
      {5, { {2,{11,63}}, {3,{12,48,61}}, {4,{13,37,52,68}}, {2,{38,47}}, {2,{42,57}} } },
      {4, { {4,{16,27,42,59}}, {3,{18,26,97}}, {4,{19,47,55,98}}, {2,{45,52}} } },
      {6, { {2,{43,66}}, {2,{44,63}}, {2,{46,94}}, {3,{64,75,93}}, {2,{65,76}}, {2,{73,96}} } },
      {4, { {3,{35,49,56}}, {3,{36,75,89}}, {4,{45,59,86,92}}, {3,{79,82,95}} } },
      {4, { {5,{31,46,52,84,93}}, {3,{32,53,71}}, {3,{44,76,81}}, {2,{73,96}} } },
      {5, { {3,{54,69,78}}, {3,{58,64,99}}, {2,{71,88}}, {3,{79,81,94}}, {2,{84,98}} } },
      {5, { {4,{14,45,69,76}}, {2,{16,54}}, {3,{49,56,75}}, {2,{52,64}}, {2,{59,62}} } },
      {4, { {4,{21,34,77,92}}, {2,{24,76}}, {4,{26,32,71,84}}, {3,{74,82,97}} } },
      {4, { {2,{21,36}}, {4,{26,32,71,84}}, {3,{31,77,92}}, {4,{34,76,82,97}} } },
      {3, { {3,{37,41,68}}, {5,{38,55,61,77,84}}, {5,{47,51,64,75,88}} } },
      {3, { {5,{12,25,48,79,86}}, {5,{16,22,39,75,88}}, {3,{28,35,49}} } },
      {4, { {3,{14,27,99}}, {3,{18,24,33}}, {3,{19,78,94}}, {4,{23,38,74,97}} } },
      {3, { {5,{15,27,74,81,99}}, {5,{18,24,71,85,97}}, {3,{19,78,94}} } },
      {4, { {4,{15,27,58,66}}, {3,{18,56,69}}, {4,{19,26,47,65}}, {2,{46,57}} } },
      {3, { {5,{43,58,71,89,95}}, {5,{47,51,75,88,93}}, {3,{49,85,97}} } },
      {4, { {4,{11,34,42,66}}, {5,{16,32,48,61,84}}, {2,{44,68}}, {2,{64,86}} } },
      {4, { {4,{13,24,51,82}}, {4,{14,22,53,68}}, {3,{42,58,81}}, {2,{48,62}} } },
      {5, { {3,{67,82,99}}, {3,{68,73,87}}, {2,{69,78}}, {2,{75,83}}, {3,{79,85,92}} } },
      {4, { {3,{11,36,55}}, {4,{15,32,44,51}}, {3,{31,42,64}}, {3,{34,56,61}} } },
      {4, { {5,{42,54,73,86,98}}, {3,{46,52,85}}, {2,{55,94}}, {3,{78,83,95}} } },
      {4, { {4,{45,67,71,89}}, {4,{49,52,65,88}}, {3,{58,62,77}}, {2,{78,81}} } },
      {5, { {4,{12,35,81,97}}, {2,{15,39}}, {3,{19,31,72}}, {2,{71,87}}, {2,{77,92}} } },
      {4, { {4,{42,54,68,81}}, {4,{48,63,82,96}}, {3,{51,86,93}}, {2,{53,64}} } },
      {6, { {2,{72,94}}, {2,{73,99}}, {2,{74,82}}, {2,{75,83}}, {3,{79,85,92}}, {2,{84,93}} } },
      {4, { {2,{12,24}}, {3,{14,39,95}}, {5,{19,22,35,76,91}}, {3,{26,71,94}} } },
      {4, { {4,{22,38,81,97}}, {4,{28,31,52,83}}, {3,{53,84,92}}, {2,{87,94}} } },
      {5, { {3,{13,39,58}}, {2,{16,37}}, {3,{17,26,59}}, {3,{18,33,57}}, {2,{29,36}} } },
      {5, { {2,{13,38}}, {2,{14,23}}, {3,{18,24,32}}, {3,{22,73,85}}, {3,{33,75,82}} } },
      {4, { {4,{12,35,71,94}}, {4,{15,21,76,92}}, {3,{24,31,95}}, {2,{26,74}} } },
      {6, { {2,{73,99}}, {2,{75,83}}, {3,{79,85,92}}, {2,{82,97}}, {2,{84,93}}, {2,{87,94}} } },
      {5, { {4,{16,38,41,57}}, {3,{17,26,59}}, {2,{29,36}}, {2,{39,48}}, {2,{49,51}} } },
      {5, { {2,{12,26}}, {4,{15,22,41,54}}, {3,{16,45,51}}, {2,{24,55}}, {2,{25,56}} } },
      {4, { {4,{57,75,83,98}}, {3,{58,79,85}}, {3,{59,74,97}}, {3,{73,88,94}} } },
      {5, { {3,{43,55,91}}, {3,{44,62,93}}, {2,{45,84}}, {3,{51,64,92}}, {2,{54,85}} } },
      {5, { {2,{12,26}}, {4,{15,22,41,54}}, {3,{16,45,51}}, {2,{24,86}}, {2,{56,84}} } },
      {4, { {4,{14,33,72,85}}, {2,{15,86}}, {4,{16,32,74,91}}, {3,{31,83,96}} } },
      {4, { {4,{14,48,55,86}}, {3,{16,29,85}}, {4,{19,25,44,68}}, {2,{58,65}} } },
      {5, { {3,{15,76,83}}, {2,{16,37}}, {4,{17,24,73,85}}, {2,{26,74}}, {2,{27,36}} } },
      {5, { {4,{46,58,74,87}}, {3,{47,54,96}}, {2,{57,98}}, {2,{78,94}}, {2,{86,97}} } },
      {5, { {4,{46,58,74,87}}, {2,{47,56}}, {3,{54,78,96}}, {2,{57,98}}, {2,{86,97}} } },
      {5, { {3,{41,56,75}}, {3,{45,69,96}}, {3,{49,51,66}}, {2,{72,95}}, {2,{76,92}} } },
      {3, { {5,{46,51,67,82,94}}, {5,{47,54,62,86,93}}, {3,{57,63,91}} } },
      {3, { {3,{11,57,62}}, {5,{12,47,54,86,91}}, {5,{46,51,67,82,94}} } },
      {5, { {3,{43,55,68}}, {3,{48,53,89}}, {3,{59,65,98}}, {2,{83,99}}, {2,{88,93}} } },
      {5, { {3,{21,42,93}}, {3,{22,76,91}}, {2,{43,82}}, {2,{72,87}}, {3,{77,83,96}} } },
      {6, { {2,{41,93}}, {2,{42,91}}, {2,{43,82}}, {2,{72,87}}, {2,{76,92}}, {3,{77,83,96}} } },
      {6, { {3,{41,83,96}}, {2,{42,91}}, {2,{43,82}}, {2,{72,87}}, {2,{76,92}}, {2,{77,86}} } },
      {4, { {4,{34,45,59,88}}, {3,{35,84,92}}, {4,{48,54,89,95}}, {2,{82,98}} } },
      {4, { {2,{24,66}}, {4,{26,38,87,94}}, {4,{27,36,78,84}}, {3,{64,76,98}} } },
      {6, { {2,{21,92}}, {2,{22,71}}, {2,{31,73}}, {3,{33,84,91}}, {2,{72,94}}, {2,{74,83}} } },
      {5, { {3,{11,47,63}}, {2,{13,71}}, {3,{41,54,72}}, {2,{44,67}}, {3,{52,64,73}} } },
      {4, { {3,{31,77,93}}, {4,{33,46,61,82}}, {4,{42,66,71,87}}, {2,{83,97}} } },
      {4, { {3,{22,39,41}}, {4,{28,31,67,92}}, {2,{29,68}}, {4,{37,42,69,91}} } },
      {4, { {4,{15,29,78,96}}, {4,{18,25,74,99}}, {3,{26,54,75}}, {2,{56,94}} } },
      {4, { {4,{37,73,86,98}}, {3,{38,79,84}}, {3,{39,76,87}}, {3,{74,88,93}} } },
      {3, { {4,{31,77,83,95}}, {4,{33,46,61,82}}, {5,{42,66,71,85,97}} } },
      {4, { {4,{16,37,44,79}}, {4,{19,24,46,68}}, {2,{28,34}}, {3,{38,69,77}} } },
      {4, { {2,{13,29}}, {4,{18,32,83,99}}, {4,{19,31,88,92}}, {3,{23,39,81}} } },
      {3, { {4,{37,43,58,71}}, {5,{38,51,69,77,92}}, {4,{47,62,73,99}} } },
      {5, { {3,{15,39,76}}, {3,{19,36,95}}, {3,{75,88,92}}, {2,{78,86}}, {2,{82,96}} } },
      {4, { {3,{16,28,84}}, {3,{18,67,99}}, {4,{19,34,86,97}}, {3,{24,37,68}} } },
      {5, { {2,{27,68}}, {3,{28,72,87}}, {4,{44,57,78,82}}, {2,{48,64}}, {2,{54,67}} } },
      {5, { {2,{11,38}}, {3,{13,29,52}}, {3,{18,22,53}}, {3,{19,25,31}}, {2,{28,35}} } },
      {5, { {2,{13,26}}, {3,{16,28,97}}, {3,{17,79,95}}, {3,{19,23,78}}, {2,{75,98}} } },
      {5, { {3,{13,29,34}}, {2,{14,37}}, {3,{17,49,88}}, {3,{18,23,89}}, {2,{39,47}} } },
      {3, { {5,{23,35,46,58,62}}, {3,{26,39,55}}, {5,{29,32,43,56,68}} } },
      {6, { {2,{12,39}}, {2,{17,32}}, {3,{19,73,87}}, {2,{37,99}}, {2,{79,93}}, {2,{83,97}} } },
      {4, { {3,{13,25,74}}, {4,{14,33,75,92}}, {3,{22,35,93}}, {3,{23,72,94}} } },
      {3, { {4,{12,46,53,71}}, {5,{13,56,62,75,81}}, {4,{41,65,72,86}} } },
      {5, { {3,{12,28,36}}, {3,{15,22,64}}, {2,{16,95}}, {3,{24,38,66}}, {2,{65,96}} } },
      {5, { {2,{14,37}}, {4,{17,69,71,98}}, {2,{19,34}}, {3,{38,77,91}}, {2,{39,68}} } },
      {4, { {3,{14,27,31}}, {3,{17,44,58}}, {3,{18,34,59}}, {4,{21,39,47,54}} } },
      {5, { {2,{18,69}}, {3,{19,78,86}}, {3,{37,68,89}}, {3,{38,87,96}}, {2,{76,98}} } },
      {4, { {2,{16,29}}, {4,{18,35,41,56}}, {2,{19,88}}, {5,{26,38,45,51,89}} } },
      {5, { {3,{13,35,61}}, {3,{15,23,39}}, {3,{29,32,63}}, {2,{31,92}}, {2,{62,91}} } },
      {4, { {4,{11,36,85,92}}, {4,{15,31,76,83}}, {2,{73,89}}, {3,{79,82,96}} } },
      {5, { {3,{43,54,69}}, {3,{44,59,87}}, {2,{45,53}}, {3,{47,65,89}}, {2,{55,63}} } },
      {5, { {2,{44,75}}, {3,{45,77,96}}, {2,{46,94}}, {3,{67,74,88}}, {3,{68,84,97}} } },
      {4, { {5,{15,27,44,51,68}}, {2,{18,69}}, {4,{19,25,47,54}}, {2,{59,61}} } },
      {5, { {3,{13,26,44}}, {4,{14,33,55,96}}, {2,{23,35}}, {2,{25,56}}, {2,{46,94}} } },
      {4, { {5,{22,41,66,87,93}}, {4,{23,52,81,97}}, {2,{46,51}}, {2,{56,62}} } },
      {5, { {3,{11,25,37}}, {3,{15,31,86}}, {2,{24,55}}, {3,{27,36,84}}, {2,{54,85}} } },
      {5, { {3,{11,36,95}}, {2,{15,28}}, {4,{18,24,31,66}}, {2,{25,64}}, {2,{65,96}} } },
      {4, { {2,{15,28}}, {4,{18,59,64,85}}, {3,{19,25,57}}, {4,{27,55,68,84}} } },
      {6, { {2,{28,89}}, {2,{29,98}}, {2,{31,92}}, {3,{32,71,88}}, {2,{78,91}}, {2,{82,99}} } },
      {5, { {2,{11,27}}, {2,{16,31}}, {3,{17,69,78}}, {3,{19,36,68}}, {3,{21,38,77}} } },
      {5, { {3,{14,73,96}}, {2,{15,74}}, {3,{16,85,92}}, {3,{75,89,93}}, {2,{82,99}} } },
      {4, { {4,{12,39,58,65}}, {3,{13,55,62}}, {2,{19,23}}, {4,{29,32,53,68}} } },
      {4, { {4,{12,28,79,83}}, {2,{13,52}}, {3,{29,88,95}}, {4,{53,72,85,99}} } },
      {4, { {4,{14,41,56,75}}, {2,{16,29}}, {3,{19,35,74}}, {4,{26,39,45,51}} } },
      {4, { {4,{14,46,73,95}}, {3,{16,42,64}}, {4,{45,62,74,83}}, {2,{85,93}} } },
      {4, { {4,{31,74,82,97}}, {4,{32,46,84,91}}, {2,{44,75}}, {3,{45,77,96}} } },
      {5, { {3,{12,74,93}}, {4,{13,24,72,96}}, {2,{14,22}}, {2,{16,44}}, {2,{46,94}} } },
      {3, { {5,{12,28,34,76,97}}, {5,{17,22,36,78,85}}, {3,{35,87,94}} } },
      {4, { {5,{11,27,73,89,95}}, {2,{19,31}}, {4,{23,39,85,97}}, {2,{33,71}} } },
      {4, { {4,{11,56,63,92}}, {2,{13,41}}, {3,{43,84,91}}, {4,{52,66,83,94}} } },
      {3, { {5,{14,37,49,75,88}}, {3,{15,66,84}}, {5,{38,47,65,79,86}} } },
      {4, { {4,{14,56,62,71}}, {2,{15,64}}, {4,{16,23,51,65}}, {3,{21,63,72}} } },
      {3, { {5,{25,43,54,79,96}}, {5,{26,45,53,84,92}}, {3,{74,82,99}} } },
      {4, { {4,{14,56,62,71}}, {2,{16,84}}, {3,{21,63,72}}, {4,{23,51,64,86}} } },
      {6, { {2,{41,58}}, {2,{46,69}}, {2,{48,56}}, {3,{49,63,81}}, {2,{51,83}}, {2,{53,66}} } },
      {4, { {4,{16,27,31,42}}, {3,{17,25,96}}, {3,{22,39,41}}, {3,{29,36,95}} } },
      {4, { {4,{14,39,75,88}}, {3,{15,66,84}}, {2,{37,79}}, {4,{38,65,77,86}} } },
      {5, { {3,{13,35,96}}, {2,{16,28}}, {3,{18,33,87}}, {3,{25,37,88}}, {2,{26,95}} } },
      {4, { {4,{34,46,51,95}}, {3,{36,48,54}}, {4,{41,55,68,94}}, {2,{58,64}} } },
      {6, { {3,{27,83,98}}, {2,{28,99}}, {2,{29,87}}, {2,{74,89}}, {2,{79,94}}, {2,{84,93}} } },
      {5, { {2,{41,64}}, {3,{44,79,95}}, {2,{45,51}}, {4,{55,61,84,99}}, {2,{74,89}} } },
      {4, { {4,{21,35,86,92}}, {4,{26,33,51,95}}, {3,{32,83,96}}, {2,{53,81}} } },
      {5, { {3,{11,24,73}}, {3,{14,39,45}}, {2,{19,31}}, {3,{23,35,44}}, {2,{33,71}} } },
      {5, { {4,{31,62,83,96}}, {2,{33,71}}, {2,{61,72}}, {2,{73,87}}, {3,{77,86,92}} } },
      {4, { {4,{21,35,86,92}}, {4,{26,32,71,95}}, {2,{61,72}}, {3,{62,81,96}} } },
      {5, { {2,{24,55}}, {2,{25,96}}, {3,{26,74,89}}, {4,{35,54,79,86}}, {2,{36,95}} } },
      {4, { {4,{22,37,49,51}}, {4,{29,31,42,58}}, {3,{38,52,67}}, {2,{47,62}} } },
      {5, { {3,{41,66,72}}, {2,{46,51}}, {3,{52,77,83}}, {3,{53,71,87}}, {2,{56,62}} } },
      {5, { {3,{24,45,58}}, {3,{25,41,64}}, {3,{46,59,61}}, {2,{48,56}}, {2,{54,69}} } },
      {4, { {3,{12,29,71}}, {4,{19,21,47,62}}, {3,{27,46,69}}, {3,{41,66,72}} } },
      {4, { {2,{12,26}}, {5,{16,33,59,64,92}}, {3,{22,34,93}}, {3,{24,56,69}} } },
      {5, { {4,{12,34,45,81}}, {2,{15,46}}, {3,{16,31,42}}, {2,{36,44}}, {2,{41,82}} } },
      {5, { {2,{37,89}}, {3,{38,72,87}}, {3,{39,76,98}}, {2,{78,92}}, {3,{79,82,96}} } },
      {4, { {3,{41,54,69}}, {4,{42,59,66,81}}, {3,{44,53,82}}, {3,{56,61,83}} } },
      {5, { {3,{25,37,94}}, {3,{27,58,99}}, {3,{28,34,95}}, {2,{38,59}}, {2,{39,97}} } },
      {5, { {2,{41,54}}, {3,{44,86,99}}, {3,{46,57,85}}, {2,{47,51}}, {3,{55,89,94}} } },
      {5, { {2,{17,48}}, {4,{18,41,54,97}}, {3,{44,59,98}}, {2,{47,51}}, {2,{57,99}} } },
      {5, { {4,{13,25,48,54}}, {3,{14,46,58}}, {2,{16,29}}, {2,{19,23}}, {2,{26,45}} } },
      {5, { {2,{11,26}}, {2,{16,29}}, {4,{17,21,35,68}}, {2,{19,67}}, {3,{25,38,69}} } },
      {5, { {3,{25,38,69}}, {4,{29,34,85,98}}, {2,{35,94}}, {2,{68,99}}, {2,{89,95}} } },
      {4, { {3,{14,45,63}}, {3,{15,46,94}}, {4,{43,54,72,96}}, {3,{52,64,73}} } },
      {4, { {4,{14,45,61,83}}, {4,{15,48,51,64}}, {3,{43,54,81}}, {2,{44,58}} } },
      {4, { {5,{25,38,74,81,97}}, {3,{27,88,91}}, {3,{28,34,95}}, {2,{71,94}} } },
      {5, { {2,{13,24}}, {4,{14,31,73,95}}, {3,{23,34,91}}, {2,{35,94}}, {2,{74,93}} } },
      {4, { {4,{22,36,83,94}}, {4,{26,33,72,85}}, {3,{75,89,92}}, {2,{84,99}} } },
      {4, { {4,{17,54,69,85}}, {3,{19,57,61}}, {3,{45,58,84}}, {3,{48,51,65}} } },
      {4, { {4,{15,38,77,96}}, {3,{17,25,98}}, {3,{28,36,45}}, {3,{46,75,97}} } },
      {4, { {3,{17,45,58}}, {4,{18,35,77,96}}, {2,{36,55}}, {4,{47,56,75,98}} } },
      {4, { {3,{14,45,63}}, {4,{15,48,51,64}}, {3,{43,69,98}}, {3,{58,61,99}} } },
      {5, { {2,{37,68}}, {2,{38,59}}, {2,{39,97}}, {4,{56,67,74,99}}, {3,{58,64,76}} } },
      {5, { {4,{15,56,89,94}}, {2,{16,75}}, {2,{54,76}}, {2,{73,85}}, {3,{74,83,99}} } },
      {3, { {5,{27,48,54,63,99}}, {3,{28,85,97}}, {5,{43,59,64,88,95}} } },
      {4, { {3,{14,46,58}}, {3,{16,23,39}}, {5,{19,25,33,48,54}}, {2,{26,45}} } },
      {6, { {2,{11,26}}, {3,{14,21,95}}, {2,{16,29}}, {2,{19,34}}, {2,{25,39}}, {2,{35,94}} } },
      {4, { {4,{15,28,44,57}}, {3,{17,55,69}}, {3,{18,24,65}}, {3,{19,47,64}} } },
      {5, { {2,{11,26}}, {2,{14,65}}, {2,{15,21}}, {3,{16,84,92}}, {4,{25,64,82,96}} } },
      {5, { {4,{13,28,34,91}}, {3,{15,31,93}}, {2,{18,25}}, {2,{24,95}}, {2,{35,94}} } },
      {4, { {2,{11,26}}, {5,{16,22,41,57,65}}, {2,{21,52}}, {4,{25,46,51,67}} } },
      {3, { {5,{22,34,66,83,95}}, {4,{26,33,88,92}}, {4,{36,65,84,98}} } },
      {4, { {4,{12,27,34,98}}, {3,{18,22,69}}, {4,{24,39,68,97}}, {2,{29,32}} } },
      {4, { {2,{12,38}}, {4,{15,32,58,66}}, {4,{17,36,45,68}}, {3,{18,47,55}} } },
      {4, { {3,{27,58,64}}, {3,{28,87,95}}, {4,{45,67,84,98}}, {3,{48,54,85}} } },
      {5, { {4,{45,67,84,98}}, {3,{48,54,85}}, {2,{57,64}}, {2,{58,87}}, {2,{88,95}} } },
      {4, { {3,{12,29,36}}, {4,{16,27,85,92}}, {2,{17,22}}, {4,{25,39,82,96}} } },
      {4, { {2,{13,24}}, {5,{14,32,63,75,91}}, {4,{22,35,71,94}}, {2,{23,62}} } },
      {6, { {2,{13,24}}, {2,{14,45}}, {3,{15,32,63}}, {2,{22,34}}, {2,{23,62}}, {2,{35,44}} } },
      {5, { {3,{11,63,72}}, {3,{13,26,62}}, {2,{16,31}}, {2,{22,73}}, {3,{23,36,71}} } },
      {5, { {3,{41,56,93}}, {2,{46,61}}, {2,{52,66}}, {3,{53,82,96}}, {3,{62,86,91}} } },
      {4, { {3,{24,37,95}}, {2,{27,59}}, {4,{29,36,87,94}}, {4,{35,57,86,99}} } },
      {6, { {2,{13,35}}, {3,{15,28,89}}, {2,{17,33}}, {2,{19,87}}, {2,{25,37}}, {2,{27,88}} } },
      {3, { {4,{22,35,53,69}}, {4,{24,31,45,62}}, {5,{25,33,41,59,64}} } },
      {4, { {4,{31,48,54,62}}, {4,{32,53,68,99}}, {2,{33,51}}, {3,{44,59,98}} } },
      {5, { {4,{13,35,86,92}}, {2,{16,23}}, {2,{21,93}}, {2,{22,91}}, {3,{26,32,85}} } },
      {4, { {4,{12,37,71,88}}, {3,{13,72,85}}, {4,{18,31,77,83}}, {2,{75,87}} } },
      {4, { {4,{11,48,79,83}}, {3,{12,71,89}}, {4,{13,49,88,92}}, {2,{82,99}} } },
      {5, { {3,{11,35,73}}, {2,{15,28}}, {3,{18,21,34}}, {3,{24,33,71}}, {2,{25,38}} } },
      {6, { {2,{24,95}}, {3,{25,76,89}}, {2,{26,74}}, {2,{71,94}}, {2,{79,91}}, {2,{85,99}} } },
      {5, { {2,{14,39}}, {3,{16,38,57}}, {3,{17,24,59}}, {2,{19,36}}, {3,{29,34,58}} } },
      {4, { {4,{11,29,45,57}}, {4,{19,31,48,55}}, {3,{21,37,59}}, {2,{38,47}} } },
      {4, { {3,{43,57,89}}, {4,{48,53,87,96}}, {3,{49,78,83}}, {3,{73,86,98}} } },
      {4, { {2,{37,69}}, {4,{39,53,77,81}}, {3,{51,73,95}}, {4,{67,75,89,91}} } },
      {4, { {2,{15,86}}, {3,{16,58,65}}, {4,{21,43,56,68}}, {4,{23,46,61,85}} } },
      {4, { {4,{18,23,41,99}}, {3,{19,43,58}}, {3,{21,53,68}}, {3,{49,61,98}} } },
      {5, { {4,{41,65,73,84}}, {2,{44,86}}, {2,{46,51}}, {3,{56,63,71}}, {2,{66,85}} } },
      {5, { {3,{14,46,67}}, {4,{16,47,54,89}}, {2,{53,64}}, {2,{59,63}}, {2,{69,87}} } },
      {4, { {3,{41,56,69}}, {4,{46,58,62,81}}, {3,{53,68,82}}, {3,{59,61,83}} } },
      {4, { {4,{17,59,66,98}}, {3,{19,72,97}}, {4,{46,68,79,92}}, {2,{49,56}} } },
      {5, { {3,{25,86,93}}, {4,{26,32,81,95}}, {2,{31,53}}, {2,{33,92}}, {2,{51,83}} } },
      {5, { {2,{11,73}}, {2,{12,61}}, {2,{13,52}}, {3,{21,38,62}}, {4,{28,32,53,71}} } },
      {4, { {3,{12,35,74}}, {3,{14,21,82}}, {3,{23,75,81}}, {4,{24,32,73,85}} } },
      {4, { {3,{21,82,99}}, {4,{22,54,73,86}}, {3,{23,79,91}}, {3,{56,74,89}} } },
      {4, { {5,{13,24,37,42,68}}, {2,{18,69}}, {3,{19,22,43}}, {3,{29,34,67}} } },
      {4, { {3,{38,52,69}}, {4,{39,53,78,92}}, {3,{58,63,79}}, {3,{62,73,99}} } },
      {4, { {3,{41,56,69}}, {4,{44,51,77,96}}, {3,{45,74,97}}, {3,{49,66,95}} } },
      {4, { {4,{24,56,82,95}}, {2,{25,44}}, {3,{45,74,97}}, {4,{54,77,86,92}} } },
      {3, { {3,{24,53,65}}, {5,{25,37,54,69,96}}, {5,{29,36,57,63,94}} } },
      {5, { {3,{12,35,74}}, {3,{14,23,52}}, {3,{24,33,75}}, {2,{32,73}}, {2,{53,72}} } },
      {5, { {3,{11,29,36}}, {3,{12,26,74}}, {3,{14,37,76}}, {2,{17,31}}, {2,{19,22}} } },
      {3, { {3,{27,74,99}}, {5,{29,48,52,65,77}}, {5,{42,55,69,78,94}} } },
      {4, { {3,{25,37,54}}, {2,{27,98}}, {5,{28,34,55,87,96}}, {3,{56,84,97}} } },
      {4, { {3,{21,44,53}}, {4,{23,64,71,85}}, {4,{41,65,74,83}}, {2,{54,63}} } },
      {5, { {4,{24,52,65,86}}, {2,{25,94}}, {2,{34,95}}, {3,{35,56,62}}, {2,{36,84}} } },
      {4, { {5,{43,55,69,87,94}}, {3,{44,63,96}}, {3,{59,66,85}}, {2,{86,97}} } },
      {5, { {2,{12,24}}, {3,{14,32,95}}, {3,{22,37,98}}, {2,{25,94}}, {3,{28,35,97}} } },
      {5, { {3,{14,43,66}}, {2,{15,44}}, {2,{16,29}}, {2,{19,35}}, {4,{26,39,45,63}} } },
      {5, { {3,{14,29,66}}, {2,{15,44}}, {2,{19,35}}, {4,{26,39,45,63}}, {2,{43,64}} } },
      {3, { {5,{45,59,61,76,97}}, {4,{46,63,71,95}}, {4,{51,67,79,93}} } },
      {5, { {3,{11,29,97}}, {3,{17,21,35}}, {3,{25,38,59}}, {2,{37,58}}, {2,{57,99}} } },
      {5, { {2,{14,28}}, {3,{16,45,74}}, {2,{18,26}}, {3,{23,35,46}}, {3,{24,33,75}} } },
      {4, { {4,{21,68,89,92}}, {3,{22,77,81}}, {4,{69,72,84,98}}, {2,{74,87}} } },
      {4, { {5,{16,28,44,57,85}}, {4,{18,24,65,86}}, {2,{47,64}}, {2,{55,67}} } },
      {4, { {3,{21,37,98}}, {4,{28,31,43,69}}, {4,{39,48,66,97}}, {2,{46,63}} } },
      {5, { {4,{28,79,84,91}}, {2,{29,48}}, {2,{47,88}}, {2,{49,77}}, {3,{71,87,94}} } },
      {4, { {5,{37,42,69,73,88}}, {3,{39,57,63}}, {3,{48,52,87}}, {2,{53,72}} } },
      {6, { {2,{15,37}}, {2,{17,48}}, {3,{18,35,49}}, {2,{39,67}}, {2,{47,99}}, {2,{69,97}} } },
      {4, { {3,{11,25,53}}, {5,{15,22,44,51,69}}, {3,{23,49,62}}, {2,{43,54}} } },
      {5, { {3,{11,25,53}}, {4,{15,22,51,64}}, {2,{23,42}}, {2,{43,54}}, {2,{44,62}} } },
      {5, { {3,{31,82,95}}, {4,{32,43,75,91}}, {2,{41,73}}, {2,{42,81}}, {2,{71,85}} } },
      {4, { {4,{11,25,43,58}}, {4,{15,22,51,64}}, {2,{23,42}}, {3,{48,54,62}} } },
      {5, { {2,{22,73}}, {3,{23,34,76}}, {4,{26,31,74,92}}, {2,{33,71}}, {2,{72,96}} } },
      {4, { {3,{17,86,98}}, {4,{18,64,77,95}}, {3,{65,74,93}}, {3,{73,87,96}} } },
      {4, { {4,{41,54,68,86}}, {3,{48,51,99}}, {3,{52,66,84}}, {3,{59,62,98}} } },
      {4, { {4,{44,67,88,96}}, {4,{47,66,78,84}}, {3,{48,71,97}}, {2,{76,91}} } },
      {4, { {4,{25,38,87,96}}, {2,{26,85}}, {4,{27,49,86,98}}, {3,{29,36,48}} } },
      {5, { {3,{15,29,77}}, {3,{17,25,58}}, {2,{28,49}}, {3,{45,57,79}}, {2,{48,55}} } },
      {4, { {5,{12,26,63,79,94}}, {4,{14,23,62,86}}, {2,{76,89}}, {2,{84,99}} } },
      {4, { {3,{41,74,93}}, {2,{42,71}}, {4,{43,54,72,86}}, {4,{46,53,82,94}} } },
      {4, { {4,{34,52,73,85}}, {3,{35,88,96}}, {4,{36,53,74,92}}, {2,{82,98}} } },
      {3, { {5,{38,57,63,71,89}}, {4,{39,43,67,78}}, {4,{49,51,77,83}} } },
      {5, { {3,{11,27,98}}, {2,{12,51}}, {3,{18,22,36}}, {2,{21,52}}, {3,{26,38,97}} } },
      {4, { {3,{18,82,99}}, {4,{19,58,62,83}}, {4,{51,69,88,92}}, {2,{53,81}} } },
      {3, { {3,{18,82,99}}, {5,{19,23,58,62,81}}, {5,{21,53,69,88,92}} } },
      {3, { {5,{42,57,66,89,93}}, {4,{46,53,82,94}}, {4,{54,69,86,97}} } },
      {3, { {4,{18,62,81,99}}, {4,{19,23,58,61}}, {5,{21,53,69,88,92}} } },
      {5, { {3,{11,34,65}}, {3,{15,22,61}}, {3,{24,31,62}}, {2,{25,66}}, {2,{26,64}} } },
      {6, { {3,{24,82,96}}, {2,{25,84}}, {2,{26,95}}, {2,{72,87}}, {2,{77,92}}, {2,{85,97}} } },
      {4, { {4,{45,69,72,97}}, {4,{47,76,88,92}}, {2,{48,65}}, {3,{68,79,86}} } },
      {4, { {4,{12,28,53,67}}, {3,{17,31,72}}, {3,{23,37,68}}, {3,{33,52,71}} } },
      {6, { {2,{11,72}}, {3,{12,83,94}}, {2,{13,81}}, {2,{71,84}}, {2,{74,97}}, {2,{77,92}} } },
      {4, { {2,{31,42}}, {5,{32,49,53,64,87}}, {3,{33,54,61}}, {3,{41,67,89}} } },
      {4, { {5,{55,68,76,89,91}}, {3,{59,66,75}}, {2,{69,98}}, {3,{71,85,99}} } },
      {4, { {4,{15,38,44,57}}, {4,{17,26,45,59}}, {3,{28,34,46}}, {2,{29,58}} } },
      {4, { {4,{12,35,44,53}}, {4,{14,21,42,56}}, {3,{25,33,51}}, {2,{26,55}} } },
      {4, { {3,{31,42,73}}, {4,{32,43,89,91}}, {3,{57,71,99}}, {3,{59,77,83}} } },
      {5, { {2,{31,72}}, {2,{32,73}}, {3,{33,89,91}}, {3,{57,71,99}}, {3,{59,77,83}} } },
      {4, { {4,{25,73,89,96}}, {3,{26,75,92}}, {3,{28,72,99}}, {3,{29,78,83}} } },
      {5, { {3,{34,75,88}}, {3,{35,86,99}}, {2,{36,84}}, {2,{68,89}}, {3,{69,78,95}} } },
      {5, { {2,{25,76}}, {3,{26,85,97}}, {2,{47,98}}, {3,{48,79,96}}, {3,{49,75,87}} } },
      {5, { {3,{15,46,94}}, {3,{16,28,54}}, {3,{18,24,95}}, {2,{42,56}}, {2,{44,52}} } },
      {6, { {3,{15,37,84}}, {2,{17,88}}, {2,{18,25}}, {2,{24,85}}, {2,{28,34}}, {2,{38,87}} } },
      {4, { {3,{15,78,86}}, {4,{16,28,75,99}}, {4,{17,26,88,95}}, {2,{19,97}} } },
      {5, { {2,{12,27}}, {3,{16,22,65}}, {2,{17,36}}, {3,{24,37,96}}, {3,{25,66,94}} } },
      {4, { {4,{12,36,71,84}}, {3,{13,82,94}}, {4,{14,32,76,93}}, {2,{31,72}} } },
      {5, { {2,{41,82}}, {4,{42,55,88,93}}, {2,{45,58}}, {3,{48,53,91}}, {2,{81,98}} } },
      {5, { {3,{15,71,96}}, {3,{16,75,81}}, {2,{64,86}}, {3,{65,84,91}}, {2,{66,95}} } },
      {5, { {3,{11,27,88}}, {3,{18,33,81}}, {3,{21,36,83}}, {2,{26,37}}, {2,{38,87}} } },
      {3, { {5,{11,24,38,55,72}}, {5,{15,28,32,54,61}}, {3,{52,65,71}} } },
      {4, { {4,{42,69,78,81}}, {4,{44,61,72,88}}, {3,{48,54,89}}, {2,{59,64}} } },
      {4, { {3,{14,49,56}}, {2,{16,29}}, {4,{19,21,35,44}}, {4,{26,31,45,59}} } },
      {4, { {4,{38,56,67,79}}, {3,{39,76,98}}, {2,{53,66}}, {4,{58,63,77,96}} } },
      {4, { {4,{11,27,36,93}}, {4,{16,23,84,91}}, {3,{24,37,96}}, {2,{83,94}} } },
      {5, { {2,{15,76}}, {4,{16,55,87,94}}, {3,{54,75,92}}, {2,{72,86}}, {2,{82,97}} } },
      {4, { {4,{13,24,82,95}}, {3,{15,32,73}}, {4,{23,35,74,92}}, {2,{72,84}} } },
      {3, { {5,{13,24,72,88,95}}, {3,{15,32,73}}, {5,{23,35,78,84,92}} } },
      {3, { {5,{41,64,79,86,92}}, {3,{46,52,91}}, {5,{56,61,72,89,94}} } },
      {4, { {3,{13,25,51}}, {4,{15,33,58,64}}, {3,{21,34,53}}, {3,{24,55,68}} } },
      {4, { {3,{23,35,74}}, {3,{24,33,86}}, {4,{36,49,75,88}}, {3,{48,79,84}} } },
      {3, { {5,{13,25,42,64,96}}, {5,{16,22,39,43,94}}, {3,{29,34,65}} } },
      {3, { {4,{23,36,75,82}}, {5,{26,32,51,73,98}}, {4,{52,78,85,91}} } },
      {4, { {5,{22,38,43,57,79}}, {2,{27,58}}, {4,{28,33,49,52}}, {2,{59,78}} } },
      {4, { {4,{22,39,43,57}}, {3,{27,59,98}}, {4,{28,33,49,52}}, {2,{38,99}} } },
      {4, { {3,{27,59,98}}, {2,{28,77}}, {4,{43,57,78,91}}, {4,{47,51,73,99}} } },
      {4, { {4,{15,28,86,99}}, {3,{16,81,95}}, {4,{19,26,78,91}}, {2,{71,88}} } },
      {4, { {4,{26,39,85,98}}, {3,{28,35,86}}, {3,{38,47,89}}, {3,{48,87,96}} } },
      {5, { {4,{26,39,85,98}}, {3,{28,35,86}}, {2,{37,89}}, {2,{38,87}}, {2,{88,96}} } },
      {4, { {4,{21,42,66,75}}, {5,{22,45,63,76,91}}, {2,{61,73}}, {2,{71,95}} } },
      {5, { {3,{41,66,75}}, {2,{42,91}}, {4,{45,63,76,92}}, {2,{61,73}}, {2,{71,95}} } },
      {5, { {3,{22,43,58}}, {3,{23,72,86}}, {3,{48,51,83}}, {2,{52,71}}, {2,{76,81}} } },
      {4, { {4,{12,36,71,84}}, {2,{13,92}}, {5,{14,31,78,86,93}}, {2,{72,98}} } },
      {3, { {5,{13,27,36,74,98}}, {5,{16,23,38,89,94}}, {3,{29,77,84}} } },
      {4, { {2,{28,99}}, {4,{29,33,51,68}}, {4,{31,42,59,98}}, {3,{32,48,63}} } },
      {5, { {3,{11,36,53}}, {3,{15,22,51}}, {2,{16,37}}, {3,{17,25,33}}, {2,{23,52}} } },
      {5, { {3,{51,76,83}}, {2,{52,91}}, {4,{53,78,86,92}}, {2,{71,88}}, {2,{81,98}} } },
      {4, { {5,{15,29,31,68,72}}, {3,{18,35,67}}, {3,{22,37,71}}, {2,{27,69}} } },
      {4, { {4,{14,37,79,86}}, {3,{15,74,82}}, {4,{17,35,72,89}}, {2,{36,85}} } },
      {4, { {3,{12,38,79}}, {4,{13,37,78,82}}, {2,{19,23}}, {4,{29,32,73,87}} } },
      {4, { {3,{12,37,59}}, {4,{13,57,79,82}}, {2,{19,23}}, {4,{29,32,73,87}} } },
      {5, { {3,{13,27,91}}, {2,{17,98}}, {4,{18,21,83,99}}, {2,{29,97}}, {2,{88,93}} } },
      {4, { {4,{13,29,81,97}}, {3,{17,78,91}}, {3,{18,71,89}}, {3,{19,21,93}} } },
      {4, { {4,{13,34,82,96}}, {3,{15,33,74}}, {4,{16,75,89,92}}, {2,{79,84}} } },
      {5, { {3,{11,36,53}}, {3,{15,22,51}}, {2,{16,37}}, {2,{17,35}}, {3,{25,33,52}} } },
      {4, { {3,{24,48,66}}, {4,{26,44,59,78}}, {3,{37,56,68}}, {3,{38,57,79}} } },
      {4, { {3,{24,42,56}}, {4,{26,44,59,78}}, {3,{37,48,52}}, {3,{38,57,79}} } },
      {4, { {3,{11,25,63}}, {4,{15,23,41,72}}, {3,{42,76,81}}, {3,{61,73,86}} } },
      {3, { {5,{14,28,31,55,63}}, {5,{18,26,53,64,75}}, {3,{21,35,76}} } },
      {3, { {4,{11,25,77,83}}, {4,{15,23,38,42}}, {5,{18,32,43,71,87}} } },
      {3, { {3,{11,38,97}}, {5,{17,42,56,69,98}}, {5,{19,31,46,58,62}} } },
      {4, { {4,{13,37,51,65}}, {4,{16,31,47,63}}, {2,{17,36}}, {3,{41,55,67}} } },
      {5, { {4,{12,55,74,83}}, {2,{13,92}}, {3,{54,78,95}}, {2,{72,98}}, {2,{85,93}} } },
      {3, { {4,{12,27,35,58}}, {5,{18,25,32,59,61}}, {4,{19,21,57,62}} } },
      {3, { {4,{22,39,81,98}}, {4,{26,31,72,85}}, {5,{29,35,76,88,92}} } },
      {4, { {3,{26,38,57}}, {4,{27,36,49,61}}, {4,{39,41,58,66}}, {2,{56,67}} } },
      {5, { {2,{13,27}}, {4,{15,23,38,74}}, {3,{17,34,75}}, {2,{25,37}}, {2,{28,35}} } },
      {4, { {4,{11,43,59,87}}, {4,{13,47,51,64}}, {2,{27,89}}, {3,{29,54,67}} } },
      {5, { {3,{41,65,96}}, {2,{43,91}}, {3,{46,53,94}}, {2,{54,95}}, {3,{55,61,93}} } },
      {5, { {3,{15,37,86}}, {3,{17,36,55}}, {3,{54,85,93}}, {2,{56,94}}, {2,{83,96}} } },
      {4, { {4,{11,37,59,63}}, {2,{12,81}}, {3,{19,22,57}}, {4,{27,33,61,82}} } },
      {4, { {4,{13,37,49,56}}, {2,{14,33}}, {3,{19,22,34}}, {4,{29,32,46,57}} } },
      {4, { {4,{15,37,89,96}}, {4,{17,36,54,75}}, {3,{56,74,99}}, {2,{79,85}} } },
      {4, { {3,{15,37,86}}, {4,{17,36,45,59}}, {3,{49,64,85}}, {3,{56,69,84}} } },
      {4, { {3,{27,41,58}}, {5,{28,47,53,71,99}}, {2,{51,93}}, {3,{59,78,91}} } },
      {4, { {2,{25,37}}, {4,{27,79,81,96}}, {4,{29,35,76,98}}, {3,{38,87,91}} } },
      {4, { {4,{11,37,78,93}}, {2,{12,81}}, {4,{18,22,73,97}}, {3,{27,31,82}} } },
      {4, { {3,{15,41,66}}, {3,{16,85,93}}, {3,{45,51,84}}, {4,{54,61,83,96}} } },
      {5, { {2,{24,56}}, {2,{25,74}}, {3,{26,75,81}}, {3,{54,71,95}}, {3,{55,86,91}} } },
      {4, { {3,{16,39,87}}, {4,{17,24,86,99}}, {3,{29,36,44}}, {3,{46,89,94}} } },
      {3, { {5,{16,23,38,77,82}}, {4,{17,59,72,88}}, {4,{19,26,33,58}} } },
      {3, { {4,{11,25,38,59}}, {4,{16,21,85,97}}, {5,{19,35,58,87,96}} } },
      {5, { {3,{11,26,73}}, {3,{16,23,37}}, {2,{17,79}}, {3,{19,33,71}}, {2,{39,77}} } },
      {6, { {3,{31,53,64}}, {2,{32,91}}, {2,{33,52}}, {2,{42,54}}, {2,{44,62}}, {2,{61,92}} } },
      {4, { {5,{14,28,56,85,97}}, {2,{17,34}}, {3,{26,38,54}}, {3,{37,88,95}} } },
      {3, { {4,{11,25,73,86}}, {5,{14,23,48,65,71}}, {4,{16,44,68,85}} } },
      {3, { {4,{25,43,64,96}}, {5,{26,45,69,74,97}}, {4,{47,63,79,94}} } },
      {4, { {3,{22,34,96}}, {4,{25,38,74,97}}, {2,{26,95}}, {4,{27,32,78,94}} } },
      {5, { {3,{21,72,84}}, {2,{22,91}}, {3,{34,81,95}}, {3,{35,74,98}}, {2,{78,92}} } },
      {3, { {4,{57,61,78,83}}, {5,{58,62,71,87,95}}, {4,{63,72,85,98}} } },
      {3, { {4,{37,49,54,61}}, {5,{39,51,65,74,97}}, {4,{44,67,75,99}} } },
      {3, { {4,{46,59,77,95}}, {4,{47,66,73,88}}, {5,{55,68,79,83,96}} } },
      {5, { {2,{13,34}}, {4,{14,23,79,96}}, {2,{26,39}}, {2,{29,33}}, {3,{36,74,99}} } },
      {4, { {3,{17,74,99}}, {2,{19,67}}, {4,{26,38,69,77}}, {4,{29,36,78,94}} } },
      {5, { {3,{11,37,79}}, {3,{14,27,31}}, {3,{19,24,87}}, {2,{47,89}}, {2,{49,77}} } },
      {4, { {4,{13,26,38,52}}, {5,{16,27,32,53,68}}, {2,{28,57}}, {2,{58,62}} } },
      {5, { {4,{41,59,83,97}}, {2,{42,91}}, {3,{49,53,72}}, {2,{73,87}}, {2,{77,92}} } },
      {3, { {5,{14,27,46,68,89}}, {4,{16,28,49,65}}, {4,{19,25,64,87}} } },
      {5, { {3,{15,28,69}}, {2,{19,35}}, {2,{22,38}}, {4,{25,32,68,99}}, {2,{39,98}} } },
      {5, { {3,{11,89,93}}, {3,{13,78,81}}, {2,{27,88}}, {3,{28,73,99}}, {2,{29,87}} } },
      {4, { {4,{11,34,45,63}}, {2,{14,21}}, {4,{24,33,62,81}}, {3,{41,65,82}} } },
      {4, { {5,{31,42,57,83,99}}, {3,{32,53,81}}, {2,{49,52}}, {3,{59,87,91}} } },
      {3, { {5,{13,37,74,81,99}}, {5,{19,31,73,88,94}}, {3,{38,77,83}} } },
      {3, { {4,{11,37,74,99}}, {4,{13,38,77,81}}, {5,{19,31,73,88,94}} } },
      {4, { {3,{11,24,38}}, {3,{18,59,62}}, {4,{19,21,34,57}}, {3,{37,52,68}} } },
      {4, { {4,{12,29,71,88}}, {3,{18,22,79}}, {3,{21,42,73}}, {3,{43,78,82}} } },
      {4, { {4,{12,38,51,69}}, {4,{19,26,31,52}}, {3,{29,36,98}}, {2,{68,99}} } },
      {5, { {4,{41,59,83,97}}, {2,{42,91}}, {2,{49,52}}, {3,{53,72,87}}, {2,{77,92}} } },
      {4, { {3,{12,34,93}}, {2,{14,55}}, {4,{15,33,82,96}}, {4,{36,54,85,92}} } },
      {4, { {3,{15,66,74}}, {4,{16,62,75,81}}, {4,{24,61,72,86}}, {2,{26,64}} } },
      {5, { {4,{32,48,53,79}}, {2,{33,72}}, {2,{42,68}}, {3,{59,63,78}}, {2,{62,73}} } },
      {3, { {4,{41,69,83,97}}, {4,{44,61,75,93}}, {5,{47,65,73,89,94}} } },
      {4, { {3,{12,28,79}}, {3,{19,31,82}}, {3,{22,53,81}}, {4,{23,39,51,78}} } },
      {4, { {4,{12,34,88,95}}, {2,{15,22}}, {4,{25,32,74,93}}, {3,{73,84,98}} } },
      {4, { {5,{24,32,43,71,96}}, {3,{26,84,93}}, {3,{31,74,82}}, {2,{42,83}} } },
      {3, { {4,{14,29,86,98}}, {5,{18,26,59,77,94}}, {4,{19,57,76,88}} } },
      {4, { {4,{15,29,54,68}}, {3,{18,25,31}}, {3,{21,38,99}}, {3,{59,64,98}} } },
      {4, { {4,{24,39,72,97}}, {3,{27,78,84}}, {3,{28,34,79}}, {3,{74,87,92}} } },
      {3, { {5,{24,56,67,79,85}}, {4,{25,48,64,77}}, {4,{45,59,78,86}} } },
      {4, { {3,{46,69,94}}, {4,{49,55,73,96}}, {4,{59,64,75,83}}, {2,{84,93}} } },
      {4, { {2,{51,92}}, {5,{52,66,79,83,94}}, {4,{56,63,71,89}}, {2,{74,91}} } },
      {4, { {3,{16,37,89}}, {4,{19,22,36,81}}, {4,{29,31,87,92}}, {2,{82,99}} } },
      {4, { {3,{11,25,82}}, {3,{13,81,96}}, {3,{15,72,86}}, {4,{16,22,75,93}} } },
      {4, { {5,{13,25,38,49,56}}, {3,{19,23,35}}, {2,{36,55}}, {3,{39,45,58}} } },
      {5, { {3,{16,29,33}}, {3,{19,24,66}}, {3,{22,36,64}}, {2,{23,52}}, {2,{32,53}} } },
      {5, { {3,{13,25,52}}, {3,{15,21,43}}, {2,{22,91}}, {3,{31,53,92}}, {2,{33,41}} } },
      {3, { {5,{38,49,61,86,97}}, {5,{39,41,67,78,93}}, {3,{73,88,96}} } },
      {4, { {3,{12,27,91}}, {4,{17,33,78,92}}, {3,{21,38,93}}, {3,{28,73,97}} } },
      {5, { {3,{18,49,55}}, {2,{19,78}}, {4,{45,57,79,93}}, {2,{58,97}}, {2,{73,98}} } },
      {5, { {3,{15,28,56}}, {3,{18,35,57}}, {2,{22,37}}, {3,{26,32,55}}, {2,{27,58}} } },
      {3, { {5,{12,38,41,59,65}}, {4,{17,31,42,68}}, {4,{19,47,55,61}} } },
      {4, { {5,{13,38,51,72,99}}, {3,{19,33,78}}, {2,{31,53}}, {3,{52,79,93}} } },
      {4, { {4,{13,39,81,96}}, {4,{19,31,73,88}}, {2,{38,89}}, {3,{79,86,93}} } },
      {4, { {3,{12,25,44}}, {4,{14,31,46,52}}, {4,{22,35,41,54}}, {2,{36,45}} } },
      {4, { {4,{17,53,66,88}}, {3,{18,57,99}}, {3,{56,68,87}}, {3,{59,63,98}} } },
      {5, { {2,{31,72}}, {4,{32,53,81,96}}, {3,{52,79,93}}, {2,{71,89}}, {2,{86,99}} } },
      {4, { {2,{12,63}}, {3,{13,24,75}}, {4,{15,32,56,74}}, {4,{23,36,54,62}} } },
      {4, { {4,{21,34,75,92}}, {4,{24,32,41,83}}, {3,{43,71,85}}, {2,{82,95}} } },
      {5, { {4,{13,38,71,99}}, {3,{19,33,78}}, {2,{31,72}}, {2,{32,93}}, {2,{79,92}} } },
      {3, { {4,{14,58,77,85}}, {4,{15,48,51,64}}, {5,{45,57,61,74,88}} } },
      {3, { {5,{14,42,58,66,91}}, {4,{15,48,51,64}}, {4,{16,45,61,92}} } },
      {4, { {4,{13,37,42,79}}, {3,{14,22,43}}, {3,{19,24,78}}, {3,{28,32,77}} } },
      {3, { {5,{21,45,79,86,93}}, {3,{23,51,92}}, {5,{46,52,71,89,95}} } },
      {5, { {2,{15,29}}, {2,{16,55}}, {3,{19,36,48}}, {4,{25,38,47,56}}, {2,{27,49}} } },
      {4, { {4,{14,45,51,96}}, {5,{15,42,56,81,94}}, {2,{41,92}}, {2,{86,91}} } },
      {5, { {3,{31,43,55}}, {2,{32,71}}, {3,{33,48,62}}, {2,{45,58}}, {3,{51,68,72}} } },
      {5, { {3,{31,43,55}}, {3,{33,62,71}}, {2,{45,58}}, {2,{48,63}}, {3,{51,68,72}} } },
      {5, { {2,{14,76}}, {4,{15,41,57,64}}, {3,{16,45,51}}, {2,{56,67}}, {2,{66,74}} } },
      {3, { {5,{13,34,55,71,96}}, {5,{15,31,74,89,93}}, {3,{56,85,99}} } },
      {3, { {5,{11,53,69,77,92}}, {4,{13,39,52,67}}, {4,{17,32,71,99}} } },
      {4, { {4,{13,37,59,62}}, {3,{17,23,68}}, {2,{28,69}}, {4,{29,32,53,67}} } },
      {4, { {2,{41,83}}, {4,{43,54,89,96}}, {4,{46,53,81,99}}, {3,{56,84,93}} } },
      {4, { {5,{13,38,49,51,82}}, {4,{19,21,43,58}}, {2,{22,81}}, {2,{28,32}} } },
      {3, { {5,{23,35,44,62,96}}, {4,{25,32,43,54}}, {4,{26,52,65,94}} } },
      {4, { {4,{14,39,47,65}}, {3,{17,44,58}}, {3,{18,55,69}}, {3,{19,35,54}} } },
      {4, { {4,{12,24,87,91}}, {4,{14,23,72,95}}, {2,{21,93}}, {3,{75,82,97}} } },
      {4, { {4,{24,41,86,93}}, {4,{26,44,53,91}}, {2,{43,56}}, {3,{46,81,94}} } },
      {5, { {3,{41,84,93}}, {3,{43,54,86}}, {3,{46,53,91}}, {2,{56,94}}, {2,{81,96}} } },
      {4, { {4,{14,69,87,95}}, {3,{15,39,67}}, {3,{17,34,99}}, {3,{35,84,97}} } },
      {5, { {3,{14,89,95}}, {2,{15,37}}, {3,{17,34,99}}, {3,{35,84,97}}, {2,{39,87}} } },
      {4, { {4,{15,22,38,61}}, {4,{18,25,33,52}}, {3,{31,53,69}}, {2,{59,62}} } },
      {3, { {5,{13,27,59,78,92}}, {3,{14,22,93}}, {5,{19,24,58,73,97}} } },
      {3, { {5,{11,28,67,83,99}}, {4,{15,23,88,91}}, {4,{17,25,69,98}} } },
      {5, { {3,{21,46,62}}, {4,{22,51,66,74}}, {2,{41,55}}, {2,{45,76}}, {2,{54,75}} } },
      {5, { {3,{54,73,96}}, {3,{55,84,91}}, {3,{56,81,95}}, {2,{74,83}}, {2,{86,93}} } },
      {5, { {3,{31,73,98}}, {3,{33,51,68}}, {2,{52,91}}, {2,{58,62}}, {3,{63,78,92}} } },
      {3, { {4,{43,58,77,81}}, {5,{47,63,71,85,99}}, {4,{53,69,88,95}} } },
      {3, { {5,{22,39,47,61,93}}, {5,{29,33,42,56,67}}, {3,{53,66,91}} } },
      {4, { {4,{11,24,39,97}}, {5,{14,28,31,76,99}}, {2,{27,78}}, {2,{77,96}} } },
      {4, { {3,{15,29,96}}, {4,{17,26,88,95}}, {3,{19,75,87}}, {3,{28,79,85}} } },
      {4, { {3,{15,37,84}}, {4,{17,25,86,98}}, {2,{24,85}}, {4,{28,34,87,96}} } },
      {4, { {3,{24,89,95}}, {3,{25,76,98}}, {4,{26,68,84,99}}, {3,{69,78,86}} } },
      {4, { {5,{34,58,66,83,97}}, {3,{36,48,54}}, {2,{46,67}}, {3,{47,88,93}} } },
      {3, { {5,{21,36,52,65,83}}, {5,{23,54,61,75,82}}, {3,{35,56,74}} } },
      {5, { {3,{13,26,81}}, {2,{15,38}}, {2,{16,95}}, {2,{18,33}}, {4,{21,35,83,96}} } },
      {3, { {4,{42,57,88,93}}, {5,{44,52,78,83,95}}, {4,{48,54,75,97}} } },
      {3, { {4,{11,38,89,93}}, {5,{13,25,39,81,97}}, {4,{15,27,31,98}} } },
      {5, { {2,{52,64}}, {4,{54,63,72,96}}, {2,{61,93}}, {2,{62,91}}, {3,{66,74,92}} } },
      {4, { {5,{11,25,42,59,73}}, {4,{15,23,61,72}}, {2,{49,62}}, {2,{51,69}} } },
      {3, { {3,{11,27,48}}, {5,{18,33,42,69,71}}, {5,{21,39,47,62,73}} } },
      {3, { {5,{21,35,46,63,74}}, {4,{24,31,79,85}}, {4,{43,65,76,89}} } },
      {3, { {5,{12,39,43,67,88}}, {5,{18,21,49,62,87}}, {3,{29,33,41}} } },
      {5, { {4,{12,25,63,84}}, {2,{14,32}}, {3,{21,34,85}}, {2,{23,61}}, {2,{31,62}} } },
      {5, { {4,{41,57,64,82}}, {2,{43,59}}, {2,{44,63}}, {3,{49,52,81}}, {2,{53,67}} } },
      {6, { {2,{14,27}}, {3,{17,35,64}}, {2,{24,65}}, {2,{25,76}}, {2,{26,37}}, {2,{36,75}} } },
      {4, { {4,{12,26,38,44}}, {2,{18,69}}, {3,{19,22,67}}, {4,{27,34,46,68}} } },
      {5, { {2,{11,52}}, {4,{12,25,64,83}}, {3,{14,21,65}}, {2,{23,81}}, {2,{51,82}} } },
      {5, { {2,{22,83}}, {4,{23,51,69,92}}, {3,{59,63,81}}, {2,{82,99}}, {2,{89,91}} } },
      {5, { {2,{13,39}}, {4,{19,21,73,87}}, {2,{27,88}}, {3,{28,33,71}}, {2,{38,89}} } },
      {4, { {5,{27,53,64,72,99}}, {2,{29,87}}, {4,{54,62,77,83}}, {2,{89,93}} } },
      {3, { {5,{12,26,38,45,93}}, {5,{18,23,35,82,96}}, {3,{46,85,92}} } },
      {4, { {4,{11,35,44,69}}, {4,{14,49,65,96}}, {3,{16,31,55}}, {2,{56,95}} } },
      {5, { {3,{27,78,93}}, {2,{28,69}}, {2,{29,67}}, {3,{68,86,97}}, {3,{73,88,96}} } },
      {4, { {4,{13,37,48,52}}, {2,{18,49}}, {4,{19,33,57,62}}, {3,{39,42,67}} } },
      {4, { {5,{11,29,77,82,94}}, {4,{17,22,79,91}}, {2,{74,99}}, {2,{89,92}} } },
      {4, { {5,{13,25,51,64,76}}, {3,{14,23,45}}, {3,{41,56,75}}, {2,{44,61}} } },
      {6, { {2,{14,95}}, {2,{15,64}}, {2,{35,96}}, {3,{36,51,65}}, {2,{54,61}}, {2,{56,94}} } },
      {4, { {4,{15,28,86,99}}, {4,{17,25,71,88}}, {2,{19,97}}, {3,{77,81,96}} } },
      {4, { {2,{42,73}}, {4,{43,58,71,97}}, {4,{47,51,72,99}}, {3,{49,52,98}} } },
      {4, { {4,{11,39,73,94}}, {3,{12,89,91}}, {3,{19,33,72}}, {3,{74,82,99}} } },
      {5, { {2,{11,92}}, {3,{12,28,47}}, {3,{17,21,89}}, {2,{22,91}}, {3,{29,48,87}} } },
      {4, { {4,{24,61,83,96}}, {3,{25,74,99}}, {2,{26,95}}, {4,{63,79,84,91}} } },
      {5, { {3,{11,35,53}}, {3,{15,27,76}}, {2,{17,21}}, {3,{26,33,51}}, {2,{36,75}} } },
      {5, { {2,{21,93}}, {2,{23,51}}, {3,{48,79,91}}, {4,{49,53,72,98}}, {2,{52,71}} } },
      {4, { {4,{24,51,65,86}}, {2,{25,66}}, {3,{26,44,69}}, {4,{49,56,61,84}} } },
      {4, { {4,{11,38,42,59}}, {3,{14,32,51}}, {4,{17,34,48,52}}, {2,{19,57}} } },
      {3, { {5,{11,38,42,57,99}}, {3,{14,32,51}}, {5,{19,34,48,52,97}} } },
      {3, { {4,{16,29,47,55}}, {5,{17,23,34,49,76}}, {4,{25,33,56,74}} } },
      {5, { {3,{34,45,76}}, {3,{35,59,64}}, {2,{43,55}}, {3,{46,63,74}}, {2,{53,69}} } },
      {3, { {4,{27,53,68,99}}, {5,{28,42,59,73,97}}, {4,{49,63,78,92}} } },
      {4, { {3,{13,27,49}}, {3,{19,33,58}}, {4,{23,38,42,59}}, {3,{28,47,52}} } },
      {4, { {3,{46,57,89}}, {4,{49,66,78,82}}, {4,{56,68,72,97}}, {2,{87,92}} } },
      {5, { {2,{24,85}}, {3,{25,71,94}}, {3,{47,75,89}}, {3,{49,77,81}}, {2,{84,91}} } },
      {4, { {4,{28,69,87,96}}, {2,{29,48}}, {4,{46,67,84,98}}, {3,{49,64,86}} } },
      {4, { {3,{18,59,87}}, {4,{19,21,52,68}}, {4,{22,41,57,88}}, {2,{48,61}} } },
      {5, { {2,{31,82}}, {4,{32,45,63,76}}, {2,{33,61}}, {3,{41,66,75}}, {2,{42,81}} } },
      {5, { {2,{11,34}}, {3,{14,37,98}}, {4,{18,21,69,97}}, {2,{29,31}}, {2,{39,67}} } },
      {3, { {5,{11,29,45,53,97}}, {4,{17,23,59,61}}, {4,{41,55,67,99}} } },
      {4, { {4,{15,37,44,98}}, {4,{18,34,45,69}}, {2,{39,67}}, {3,{48,64,97}} } },
      {4, { {4,{15,39,84,92}}, {3,{19,24,85}}, {4,{29,35,72,94}}, {2,{75,82}} } },
      {4, { {4,{25,39,48,64}}, {2,{26,45}}, {4,{28,34,69,76}}, {3,{46,68,74}} } },
      {4, { {3,{22,38,57}}, {3,{25,58,66}}, {3,{26,47,55}}, {4,{27,32,46,68}} } },
      {5, { {3,{21,72,95}}, {4,{22,35,86,91}}, {2,{36,75}}, {2,{76,92}}, {2,{82,96}} } },
      {4, { {3,{14,29,47}}, {4,{17,24,32,73}}, {4,{23,37,49,71}}, {2,{31,72}} } },
      {4, { {3,{21,34,53}}, {4,{24,33,45,81}}, {3,{35,44,51}}, {3,{41,55,83}} } },
      {4, { {4,{15,29,88,96}}, {4,{18,26,65,99}}, {3,{66,78,85}}, {2,{76,98}} } },
      {3, { {5,{12,38,55,74,81}}, {4,{16,32,54,85}}, {4,{18,34,71,86}} } },
      {4, { {4,{21,36,45,83}}, {4,{25,33,41,84}}, {3,{34,46,85}}, {2,{43,81}} } },
      {5, { {3,{17,75,89}}, {3,{19,58,97}}, {2,{37,98}}, {2,{38,77}}, {3,{59,78,85}} } },
      {4, { {4,{15,29,64,78}}, {2,{18,35}}, {4,{24,37,49,65}}, {3,{38,47,79}} } },
      {4, { {2,{11,83}}, {4,{13,38,72,89}}, {4,{19,22,78,81}}, {3,{29,32,88}} } },
      {4, { {4,{41,54,76,83}}, {4,{46,61,84,93}}, {2,{53,64}}, {3,{63,71,96}} } },
      {4, { {4,{42,68,77,93}}, {3,{47,81,98}}, {4,{48,61,72,87}}, {2,{63,91}} } },
      {4, { {4,{12,38,73,87}}, {5,{17,33,42,78,91}}, {2,{41,82}}, {2,{81,97}} } },
      {3, { {4,{27,49,62,78}}, {5,{29,42,64,77,95}}, {4,{45,68,74,99}} } },
      {4, { {3,{12,48,53}}, {4,{13,27,58,62}}, {3,{18,22,49}}, {3,{29,42,67}} } },
      {5, { {2,{12,38}}, {2,{18,89}}, {2,{19,22}}, {3,{24,32,66}}, {4,{29,36,64,88}} } },
      {5, { {3,{11,83,94}}, {2,{13,34}}, {2,{14,21}}, {3,{24,32,91}}, {3,{33,84,92}} } },
      {5, { {3,{11,34,83}}, {2,{14,21}}, {3,{24,32,91}}, {3,{33,84,92}}, {2,{81,94}} } },
      {4, { {4,{21,34,42,93}}, {4,{24,33,41,55}}, {2,{45,62}}, {3,{53,65,92}} } },
      {5, { {3,{41,72,89}}, {3,{42,81,95}}, {3,{79,86,92}}, {2,{85,98}}, {2,{88,96}} } },
      {5, { {2,{12,83}}, {2,{13,35}}, {3,{15,31,62}}, {3,{22,36,61}}, {3,{26,33,82}} } },
      {4, { {3,{11,27,52}}, {4,{17,33,41,69}}, {4,{22,39,57,63}}, {2,{47,51}} } },
      {3, { {4,{22,45,51,73}}, {5,{23,36,48,55,71}}, {4,{26,31,42,58}} } },
      {4, { {5,{12,28,36,51,69}}, {3,{19,22,58}}, {3,{26,31,52}}, {2,{59,62}} } },
      {4, { {3,{13,26,51}}, {4,{16,38,41,57}}, {4,{17,31,48,53}}, {2,{21,36}} } },
      {3, { {5,{13,34,41,55,72}}, {5,{14,31,52,65,73}}, {3,{15,44,61}} } },
      {4, { {2,{13,26}}, {3,{16,29,67}}, {4,{17,23,88,96}}, {4,{28,69,86,97}} } },
      {5, { {4,{24,48,63,75}}, {2,{25,46}}, {2,{26,64}}, {2,{43,66}}, {3,{45,68,74}} } },
      {5, { {3,{15,39,96}}, {3,{19,26,95}}, {2,{25,86}}, {3,{29,34,85}}, {2,{36,84}} } },
      {5, { {3,{22,53,64}}, {3,{23,51,92}}, {2,{54,96}}, {3,{56,62,91}}, {2,{66,94}} } },
      {3, { {5,{43,58,65,82,94}}, {4,{44,52,76,93}}, {4,{56,68,72,85}} } },
      {3, { {5,{12,24,75,83,97}}, {4,{13,61,77,92}}, {4,{25,63,71,84}} } },
      {4, { {4,{31,46,63,75}}, {2,{33,71}}, {4,{41,59,65,73}}, {3,{49,55,76}} } },
      {3, { {5,{11,36,78,85,93}}, {4,{15,23,81,96}}, {4,{26,31,73,88}} } },
      {3, { {4,{12,27,49,63}}, {5,{14,33,47,62,99}}, {4,{19,23,34,97}} } },
      {4, { {4,{42,66,73,85}}, {4,{46,51,75,82}}, {3,{55,61,86}}, {2,{63,71}} } },
      {4, { {4,{14,47,65,96}}, {3,{16,44,67}}, {3,{45,77,94}}, {3,{66,75,97}} } },
      {4, { {2,{11,53}}, {4,{13,39,42,64}}, {5,{19,32,44,51,67}}, {2,{57,63}} } },
      {5, { {3,{11,27,42}}, {3,{13,39,62}}, {2,{17,23}}, {3,{19,32,41}}, {2,{22,63}} } },
      {4, { {3,{11,52,66}}, {4,{12,39,61,83}}, {3,{19,33,81}}, {3,{32,56,63}} } },
      {5, { {3,{15,42,56}}, {3,{16,73,95}}, {2,{44,52}}, {3,{45,74,93}}, {2,{54,76}} } },
      {4, { {4,{11,24,82,96}}, {2,{16,31}}, {4,{22,35,84,91}}, {3,{36,85,92}} } },
      {4, { {4,{31,57,88,93}}, {2,{33,51}}, {4,{53,69,81,97}}, {3,{58,61,89}} } },
      {4, { {4,{21,52,68,89}}, {4,{22,57,63,88}}, {2,{23,61}}, {3,{51,69,87}} } },
      {4, { {4,{11,27,48,62}}, {4,{12,29,67,91}}, {2,{28,49}}, {3,{41,69,92}} } },
      {3, { {5,{35,42,56,83,94}}, {3,{36,57,65}}, {5,{43,54,67,85,92}} } },
      {5, { {3,{22,44,63}}, {3,{23,52,67}}, {2,{42,59}}, {3,{49,51,64}}, {2,{57,61}} } },
      {3, { {5,{12,29,35,87,93}}, {5,{17,25,33,82,98}}, {3,{28,89,92}} } },
      {3, { {4,{37,46,74,89}}, {5,{38,49,65,77,86}}, {4,{39,44,68,75}} } },
      {4, { {3,{22,83,96}}, {4,{23,37,86,99}}, {4,{29,33,88,92}}, {2,{38,87}} } },
      {4, { {4,{21,36,42,65}}, {3,{22,55,61}}, {2,{23,52}}, {4,{25,32,46,53}} } },
      {5, { {2,{24,95}}, {3,{25,76,93}}, {3,{26,44,53}}, {3,{43,56,74}}, {2,{73,94}} } },
      {4, { {5,{22,47,61,83,94}}, {2,{23,52}}, {2,{42,57}}, {4,{53,67,84,91}} } },
      {5, { {3,{54,85,97}}, {3,{55,79,94}}, {3,{71,87,96}}, {2,{75,81}}, {2,{76,99}} } },
      {4, { {4,{14,23,37,68}}, {4,{18,24,49,63}}, {3,{29,33,47}}, {2,{43,67}} } },
      {4, { {4,{13,36,72,95}}, {2,{15,29}}, {4,{19,26,32,73}}, {3,{25,76,93}} } },
      {4, { {4,{16,39,57,65}}, {4,{17,35,46,69}}, {2,{42,56}}, {3,{45,52,67}} } },
      {4, { {4,{24,39,45,57}}, {4,{27,34,55,68}}, {3,{37,49,65}}, {2,{38,67}} } },
      {4, { {3,{17,48,56}}, {5,{18,44,57,79,86}}, {3,{46,74,88}}, {2,{49,78}} } },
      {5, { {2,{37,88}}, {3,{38,41,67}}, {3,{48,82,99}}, {3,{49,61,97}}, {2,{87,92}} } },
      {5, { {4,{31,42,58,97}}, {2,{32,91}}, {2,{43,92}}, {3,{47,53,98}}, {2,{51,93}} } },
      {4, { {3,{46,57,99}}, {5,{48,56,75,83,97}}, {3,{49,78,85}}, {2,{89,93}} } },
      {4, { {4,{11,44,52,75}}, {4,{12,55,63,74}}, {3,{13,45,51}}, {2,{43,64}} } },
      {3, { {4,{12,38,79,81}}, {5,{14,31,76,82,99}}, {4,{19,36,78,94}} } },
      {4, { {3,{41,69,73}}, {5,{47,63,71,89,95}}, {3,{49,83,97}}, {2,{75,93}} } },
      {4, { {3,{15,38,74}}, {4,{16,39,78,85}}, {3,{18,75,89}}, {3,{19,34,76}} } },
      {4, { {3,{22,39,87}}, {5,{27,31,63,89,92}}, {3,{32,61,83}}, {2,{82,97}} } },
      {5, { {2,{11,43}}, {3,{13,36,42}}, {4,{16,32,44,61}}, {2,{34,46}}, {2,{41,66}} } },
      {5, { {2,{11,24}}, {2,{14,85}}, {4,{15,33,86,91}}, {2,{21,93}}, {3,{23,36,84}} } },
      {3, { {5,{11,42,55,67,94}}, {3,{12,43,71}}, {5,{47,51,64,73,95}} } },
      {4, { {4,{12,38,71,97}}, {3,{16,28,32}}, {4,{17,26,88,91}}, {2,{78,81}} } },
      {3, { {4,{12,28,79,81}}, {5,{19,33,52,71,88}}, {4,{22,38,51,83}} } },
      {5, { {4,{43,58,79,82}}, {3,{46,52,83}}, {2,{49,66}}, {2,{56,68}}, {2,{69,78}} } },
      {3, { {5,{23,34,61,72,96}}, {5,{26,31,62,74,83}}, {3,{36,84,93}} } },
      {5, { {4,{15,38,47,56}}, {2,{16,45}}, {3,{17,58,66}}, {2,{18,36}}, {2,{46,67}} } },
      {4, { {3,{34,89,96}}, {3,{36,64,95}}, {3,{38,75,99}}, {4,{39,65,78,84}} } },
      {4, { {2,{13,24}}, {4,{14,33,41,55}}, {4,{23,35,44,86}}, {3,{46,51,85}} } },
      {4, { {3,{11,43,69}}, {3,{13,38,49}}, {3,{17,31,48}}, {4,{19,33,47,61}} } },
      {5, { {4,{16,39,48,51}}, {3,{18,36,69}}, {2,{41,54}}, {2,{44,68}}, {2,{59,64}} } },
      {5, { {2,{17,88}}, {4,{18,25,76,97}}, {3,{26,81,95}}, {2,{78,86}}, {2,{87,91}} } },
      {4, { {4,{34,56,67,98}}, {2,{36,74}}, {4,{45,58,76,97}}, {3,{47,64,75}} } },
      {4, { {2,{13,35}}, {4,{15,22,39,81}}, {4,{19,31,53,82}}, {3,{25,33,52}} } },
      {4, { {3,{44,59,85}}, {4,{47,55,88,94}}, {3,{49,87,95}}, {3,{58,84,99}} } },
      {4, { {3,{31,49,53}}, {4,{32,47,71,99}}, {2,{33,92}}, {4,{41,59,77,93}} } },
      {4, { {4,{14,29,48,55}}, {3,{19,34,67}}, {3,{25,37,69}}, {3,{35,44,58}} } },
      {4, { {4,{46,57,79,94}}, {2,{47,66}}, {4,{54,67,86,99}}, {3,{59,76,87}} } },
      {3, { {3,{14,39,78}}, {5,{18,26,44,75,97}}, {5,{27,34,45,79,96}} } },
      {4, { {3,{15,27,99}}, {4,{19,34,88,95}}, {4,{25,38,86,97}}, {2,{36,84}} } },
      {4, { {5,{16,27,44,52,69}}, {2,{19,36}}, {3,{24,37,46}}, {3,{39,57,62}} } },
      {3, { {5,{11,27,43,72,99}}, {3,{15,33,41}}, {5,{19,23,35,77,92}} } },
      {4, { {3,{44,68,86}}, {4,{48,54,61,93}}, {4,{53,65,84,91}}, {2,{66,85}} } },
      {5, { {2,{11,72}}, {4,{12,27,73,99}}, {2,{19,21}}, {2,{23,71}}, {3,{29,77,92}} } },
      {4, { {4,{14,37,88,96}}, {3,{18,24,89}}, {3,{29,36,94}}, {3,{39,84,97}} } },
      {4, { {2,{13,27}}, {4,{15,23,37,89}}, {3,{17,26,75}}, {4,{29,35,76,87}} } },
      {4, { {3,{11,53,66}}, {3,{13,34,55}}, {4,{15,32,54,61}}, {3,{33,56,62}} } },
      {5, { {2,{11,63}}, {3,{13,28,97}}, {4,{17,22,61,98}}, {2,{23,92}}, {2,{62,93}} } },
      {4, { {5,{44,57,78,86,92}}, {2,{46,94}}, {4,{48,56,72,97}}, {2,{87,96}} } },
      {4, { {3,{38,72,99}}, {4,{39,68,85,92}}, {3,{67,78,82}}, {3,{69,87,95}} } },
      {5, { {3,{13,26,71}}, {3,{16,37,98}}, {3,{18,21,73}}, {2,{27,36}}, {2,{28,97}} } },
      {4, { {4,{12,37,74,93}}, {2,{17,79}}, {5,{19,33,72,87,94}}, {2,{77,84}} } },
      {4, { {4,{12,35,54,61}}, {2,{13,37}}, {3,{14,33,55}}, {4,{17,31,52,65}} } },
      {3, { {5,{14,21,39,76,98}}, {5,{18,26,31,74,97}}, {3,{37,79,94}} } },
      {3, { {5,{15,39,63,78,91}}, {4,{17,35,81,99}}, {4,{18,61,73,87}} } },
      {4, { {3,{14,21,37}}, {4,{17,24,48,63}}, {4,{28,31,44,67}}, {2,{43,64}} } },
      {5, { {2,{12,53}}, {4,{13,24,56,61}}, {3,{16,21,62}}, {2,{26,64}}, {2,{52,66}} } },
      {4, { {3,{45,58,62}}, {3,{48,81,99}}, {3,{49,65,97}}, {4,{52,67,88,91}} } },
      {5, { {3,{11,24,62}}, {2,{14,55}}, {3,{15,33,61}}, {2,{22,63}}, {3,{23,35,54}} } },
      {4, { {4,{42,55,73,96}}, {3,{46,69,95}}, {4,{49,63,75,92}}, {2,{53,65}} } },
      {4, { {4,{13,34,46,62}}, {3,{14,32,53}}, {4,{16,43,58,64}}, {2,{52,68}} } },
      {4, { {4,{34,46,53,72}}, {2,{36,94}}, {4,{42,55,73,96}}, {3,{54,75,92}} } },
      {3, { {5,{22,36,43,74,81}}, {5,{24,31,42,86,93}}, {3,{33,71,94}} } },
      {3, { {4,{15,39,86,98}}, {4,{16,38,47,84}}, {5,{17,34,48,89,95}} } },
      {4, { {4,{53,69,71,97}}, {4,{59,62,84,93}}, {3,{61,74,82}}, {2,{77,94}} } },
      {3, { {4,{23,34,56,67}}, {4,{26,39,57,74}}, {5,{27,33,59,64,76}} } },
      {4, { {4,{21,39,78,95}}, {3,{23,75,91}}, {3,{28,33,99}}, {3,{31,73,98}} } },
      {5, { {3,{21,72,97}}, {4,{22,35,77,93}}, {2,{25,31}}, {2,{32,91}}, {2,{33,92}} } },
      {5, { {3,{14,39,46}}, {3,{19,24,57}}, {3,{27,35,44}}, {2,{36,45}}, {2,{37,59}} } },
      {5, { {2,{11,42}}, {3,{12,51,65}}, {3,{41,64,93}}, {2,{44,62}}, {3,{55,63,91}} } },
      {4, { {4,{14,29,48,85}}, {4,{18,25,77,84}}, {3,{27,49,88}}, {2,{75,87}} } },
      {4, { {3,{11,34,96}}, {4,{16,31,75,89}}, {4,{35,79,82,94}}, {2,{86,92}} } },
      {4, { {4,{24,39,77,95}}, {4,{25,38,84,99}}, {3,{37,72,88}}, {2,{74,82}} } },
      {4, { {4,{17,24,49,65}}, {4,{19,25,56,67}}, {3,{26,59,64}}, {2,{44,69}} } },
      {4, { {3,{13,36,64}}, {4,{14,31,55,63}}, {4,{33,51,66,85}}, {2,{35,86}} } },
      {5, { {2,{11,42}}, {3,{12,47,63}}, {2,{13,81}}, {3,{41,57,82}}, {3,{52,67,83}} } },
      {4, { {4,{21,38,43,69}}, {4,{29,33,61,98}}, {3,{47,63,99}}, {2,{48,97}} } },
      {4, { {3,{41,67,83}}, {3,{44,61,95}}, {4,{47,65,84,93}}, {3,{63,81,94}} } },
      {4, { {5,{23,38,69,71,87}}, {3,{29,33,67}}, {3,{31,77,83}}, {2,{37,78}} } },
      {3, { {5,{44,57,75,89,93}}, {4,{47,65,73,94}}, {4,{54,67,85,99}} } },
      {3, { {4,{11,39,52,68}}, {5,{18,33,51,82,99}}, {4,{32,69,88,93}} } },
      {4, { {2,{12,38}}, {4,{18,21,79,92}}, {4,{29,32,71,97}}, {3,{37,78,91}} } },
      {4, { {3,{12,39,67}}, {3,{16,29,32}}, {4,{17,26,48,61}}, {3,{28,41,69}} } },
      {4, { {3,{25,41,66}}, {5,{26,45,63,74,82}}, {3,{42,71,84}}, {2,{61,73}} } },
      {4, { {4,{21,37,42,59}}, {3,{29,31,47}}, {3,{32,51,66}}, {3,{49,56,62}} } },
      {4, { {4,{21,37,42,59}}, {4,{29,33,47,51}}, {2,{32,73}}, {3,{49,53,72}} } },
      {4, { {4,{13,29,75,87}}, {4,{17,23,85,98}}, {3,{28,79,93}}, {2,{73,95}} } },
      {3, { {5,{16,28,64,75,93}}, {4,{18,34,65,76}}, {4,{26,38,73,94}} } },
      {4, { {3,{13,25,86}}, {4,{16,38,85,99}}, {3,{19,23,35}}, {3,{39,88,96}} } },
      {4, { {2,{45,69}}, {3,{49,55,77}}, {4,{52,65,79,93}}, {4,{57,63,72,99}} } },
      {4, { {3,{51,69,77}}, {4,{52,65,79,93}}, {2,{55,61}}, {4,{57,63,72,99}} } },
      {3, { {4,{45,69,73,97}}, {4,{47,55,79,86}}, {5,{59,65,76,87,93}} } },
      {4, { {3,{14,37,89}}, {2,{15,86}}, {4,{16,38,74,87}}, {4,{19,34,78,85}} } },
      {4, { {2,{15,27}}, {4,{17,24,55,68}}, {3,{25,37,56}}, {4,{28,36,57,64}} } },
      {3, { {5,{15,22,43,66,91}}, {4,{16,41,83,95}}, {4,{23,65,86,92}} } },
      {3, { {5,{38,54,69,75,87}}, {4,{39,48,65,76}}, {4,{46,58,77,84}} } },
      {4, { {3,{15,38,94}}, {3,{16,75,89}}, {3,{18,79,95}}, {4,{19,34,86,98}} } },
      {4, { {4,{21,34,53,65}}, {4,{25,33,51,76}}, {2,{36,54}}, {3,{56,61,75}} } },
      {5, { {3,{38,74,89}}, {3,{39,81,98}}, {2,{64,75}}, {3,{65,84,91}}, {2,{78,95}} } },
      {4, { {4,{12,28,71,89}}, {4,{19,33,62,88}}, {3,{21,38,63}}, {2,{61,72}} } },
      {4, { {4,{23,38,47,52}}, {3,{24,32,63}}, {4,{27,34,43,68}}, {2,{58,62}} } },
      {4, { {4,{12,34,43,65}}, {3,{15,32,66}}, {4,{33,44,62,86}}, {2,{36,84}} } },
      {3, { {5,{14,32,43,66,75}}, {3,{15,71,84}}, {5,{33,44,62,76,81}} } },
      {3, { {5,{31,52,66,78,85}}, {5,{32,55,61,88,94}}, {3,{64,76,98}} } },
      {3, { {5,{27,48,62,76,91}}, {5,{28,32,41,67,93}}, {3,{33,72,96}} } },
      {5, { {3,{38,57,79}}, {2,{39,98}}, {3,{43,69,77}}, {3,{47,63,99}}, {2,{58,97}} } },
      {3, { {5,{14,27,31,53,98}}, {4,{18,25,44,97}}, {4,{23,34,45,51}} } },
      {5, { {2,{24,39}}, {3,{27,36,54}}, {3,{29,57,64}}, {2,{34,66}}, {3,{37,56,69}} } },
      {4, { {5,{16,28,44,51,67}}, {4,{17,26,48,61}}, {2,{24,46}}, {2,{41,56}} } },
      {3, { {5,{12,39,55,76,87}}, {4,{17,32,81,99}}, {4,{56,79,85,91}} } },
      {5, { {2,{14,29}}, {2,{16,74}}, {4,{17,25,76,99}}, {2,{19,77}}, {3,{24,79,95}} } },
      {3, { {4,{17,41,68,93}}, {5,{18,47,64,83,95}}, {4,{45,63,84,91}} } },
      {3, { {4,{12,27,53,71}}, {5,{17,23,42,59,88}}, {4,{48,51,72,89}} } },
      {3, { {5,{13,26,42,59,65}}, {5,{15,22,37,56,69}}, {3,{17,32,43}} } },
      {5, { {3,{12,27,68}}, {3,{17,21,62}}, {3,{18,42,67}}, {2,{22,41}}, {2,{48,61}} } },
      {5, { {3,{42,56,95}}, {3,{45,68,89}}, {2,{49,52}}, {3,{59,66,88}}, {2,{65,96}} } },
      {4, { {4,{24,68,76,89}}, {4,{25,78,84,92}}, {2,{26,75}}, {3,{69,82,98}} } },
      {3, { {5,{13,28,35,81,96}}, {3,{15,38,67}}, {5,{17,21,68,86,93}} } },
      {4, { {4,{11,26,73,95}}, {3,{15,33,81}}, {4,{23,35,86,91}}, {2,{76,83}} } },
      {4, { {4,{15,27,46,58}}, {4,{18,34,47,55}}, {3,{26,37,74}}, {2,{44,76}} } },
      {3, { {5,{11,24,79,86,92}}, {4,{13,76,81,99}}, {4,{16,22,74,93}} } },
      {3, { {5,{11,37,43,55,69}}, {5,{19,23,45,57,61}}, {3,{27,31,63}} } },
      {4, { {4,{11,59,77,82}}, {3,{12,49,53}}, {2,{13,71}}, {4,{42,57,73,89}} } },
      {3, { {4,{11,24,52,66}}, {5,{16,29,33,51,64}}, {4,{22,39,56,63}} } },
      {5, { {2,{21,83}}, {3,{23,76,91}}, {3,{27,73,89}}, {3,{29,81,97}}, {2,{77,96}} } },
      {5, { {2,{46,57}}, {3,{47,65,86}}, {2,{54,67}}, {3,{56,74,89}}, {3,{64,79,85}} } },
      {3, { {5,{25,37,43,58,64}}, {4,{28,36,57,85}}, {4,{34,45,63,86}} } },
      {4, { {4,{12,53,67,98}}, {4,{13,49,62,97}}, {3,{43,68,99}}, {2,{58,63}} } },
      {4, { {3,{46,69,85}}, {4,{49,55,86,98}}, {2,{54,95}}, {4,{58,65,89,94}} } },
      {4, { {3,{43,66,82}}, {3,{44,52,83}}, {4,{46,57,62,94}}, {3,{54,67,96}} } },
      {3, { {4,{12,35,78,93}}, {4,{15,28,72,87}}, {5,{17,25,33,82,98}} } },
      {4, { {2,{27,68}}, {3,{28,85,99}}, {4,{29,56,67,95}}, {4,{59,65,88,96}} } },
      {4, { {3,{14,37,99}}, {4,{19,26,78,94}}, {3,{28,36,77}}, {3,{34,76,97}} } },
      {5, { {3,{37,58,64}}, {2,{38,47}}, {3,{44,67,79}}, {2,{48,77}}, {3,{49,54,78}} } },
      {5, { {3,{11,73,98}}, {3,{13,76,81}}, {2,{61,82}}, {3,{62,78,91}}, {2,{72,86}} } },
      {4, { {3,{15,38,46}}, {4,{18,34,45,53}}, {3,{36,54,67}}, {3,{43,57,66}} } },
      {4, { {4,{14,53,81,96}}, {4,{16,61,84,92}}, {2,{51,64}}, {3,{54,62,93}} } },
      {4, { {3,{12,43,66}}, {4,{13,29,46,67}}, {3,{17,22,69}}, {3,{23,49,62}} } },
      {3, { {4,{44,62,75,83}}, {4,{45,57,78,94}}, {5,{58,63,72,84,97}} } },
      {4, { {3,{42,55,71}}, {3,{45,76,81}}, {4,{46,53,72,85}}, {3,{51,73,86}} } },
      {3, { {4,{23,34,41,66}}, {5,{26,38,44,61,97}}, {4,{27,31,63,98}} } },
      {3, { {5,{13,28,51,67,89}}, {4,{17,31,59,63}}, {4,{23,39,61,88}} } },
      {3, { {4,{44,67,76,98}}, {4,{48,51,64,83}}, {5,{58,63,77,81,96}} } },
      {3, { {4,{17,52,69,83}}, {5,{19,58,61,82,97}}, {4,{57,63,88,91}} } },
      {4, { {3,{21,52,64}}, {4,{22,35,43,59}}, {4,{25,33,49,61}}, {2,{54,69}} } },
      */
   };

// macros to make the code more readable
// apply permutation to row/column coordinate
#define COORD_ROW( coord, perm ) RowPerms[ perm ][ (coord) / 10 - 1 ]
#define COORD_COL( coord, perm ) RowPerms[ perm ][ (coord) % 10 - 1 ]
#define COORD( coord, perm1, perm2 )   \
   A[ COORD_ROW( coord, perm1 ) ][ COORD_COL( coord, perm2 ) ]

// IMPORTANT: aszProgress[ 0 ] and aszProgress[ 1 ] must both have the same length
// (currently 12, if this is changed modify the line following the comment starting
// with "progress indicator" below)
char const* aszProgress[ 2 ] = { "ssssuuuudddd", "ooookkkkuuuu" };

void FindUnavoidablesGeneral( int A[9][9], vector< MySet >& sets, bool swapped /* = false */ )
   {
   char RowPerms[ 6*6*6*6 ][ 9 ];
   ComputeRowPermutations( RowPerms );
   int cUnavSets = sizeof( unavpat ) / sizeof( unavpat[ 0 ] );
   for ( int k = 0;  k < cUnavSets;  k++ )
      {
      // progress indicator -- to show the user the program has not crashed
      if ( ((12 * (k+1)) % cUnavSets) < ((12 * k) % cUnavSets) )
         cout << aszProgress[ swapped ? 1 : 0 ][ 12 * k / cUnavSets ];
      UnavPat const& up = unavpat[ k ];
      for ( int i = 0;  i < 6*6*6*6;  i++ )
         for ( int j = 0;  j < 6*6*6*6;  j++ )
            {
            // see if, for each group, all cells in the group contain the same digit
            bool bMatch = true;
            for ( int k = 0;  bMatch  &&  k < up.digits;  k++ )
               {
               for ( int d = 0;  bMatch  &&  d < up.groups[ k ].size - 1;  d++ )
                  bMatch = (COORD( up.groups[ k ].data[ d ], i, j ) == 
                           COORD( up.groups[ k ].data[ d + 1 ], i, j ));
               }

            // now check if the different groups actually contain different digits
            if ( bMatch )
               {
               for ( int k = 0;  bMatch  &&  k < up.digits - 1;  k++ )
                  for ( int l = k + 1;  bMatch  &&  l < up.digits;  l++ )
                     bMatch = (COORD( up.groups[ k ].data[ 0 ], i, j ) !=
                              COORD( up.groups[ l ].data[ 0 ], i, j ));

               if ( bMatch )
                  {
                  // okay, found an instance of this particular type of unavoidable set
                  // add this to our collection of unavoidables
                  MySet set( swapped );
                  for ( int k = 0;  k < up.digits;  k++ )
                     for ( int l = 0;  l < up.groups[ k ].size;  l++ )
                        set.AddElement( COORD_ROW( up.groups[ k ].data[ l ], i ),
                                       COORD_COL( up.groups[ k ].data[ l ], j ) );
                  set.Sort();
                  // see if set is already in there
                  bool bFound = false;
                  for ( int i = 0;  ! bFound  &&  i < static_cast< int >( sets.size() );  i++ )
                     bFound = (sets[ i ] == set);
                  if ( ! bFound )
                     sets.push_back( set );
                  }
               }
            }
      }
   }


void TestSetsForUnavoidability( vector< MySet >& sets, char const* pszGrid )
   {
   // see if all the sets found are actually unavoidable
   for ( int i = 0;  i < static_cast< int >( sets.size() ); )
      if ( ! sets[ i ].IsUnavoidable( pszGrid ) )
         sets.erase( sets.begin() + i );
      else
         i++;
   }


// "calculate" the name of the file where the unavoidables are stored for this grid
string GetUnavFilename( char const* pszGridFilename )
   {
   char* const psz = ToLowerCase( strdup( pszGridFilename ) );
   // chop the trailing '.txt' or '.dat'
   if ( strlen( psz ) > 4  &&
       (0 == strcmp( psz + strlen( psz ) - 4, ".txt" )  ||
        0 == strcmp( psz + strlen( psz ) - 4, ".dat" )) )
      {
      psz[ strlen( psz ) - 4 ] = '\0';
      }
   std::string sFileUnav( psz );
   sFileUnav += ".unav";
   if ( IsSudokuX() )
      sFileUnav += ".x";
   else if ( IsSudokuDG() )
      sFileUnav += ".dg";
   sFileUnav += "." DATA_FILE_EXTN;
   delete psz;
   return sFileUnav;
   }


bool SaveUnavoidablesToFile( char const* pszUnavFilename, vector< MySet > const& sets )
   {
   ofstream* pFile = new ofstream( pszUnavFilename );
   if ( pFile == NULL )
      return false;
   for ( vector< MySet >::const_iterator it = sets.begin();  it != sets.end();  it++ )
      (*it).Print( *pFile );
   pFile->close();
   delete pFile;
   return true;
   }
